package com.blackledge.david.game;

import java.util.ArrayList;
import java.util.List;

public class BoardState {
	
	public final int width;
	public final int height;
	
	public List<List<Integer>> state;
	
	/**
	 * Initializes all data points to -1.
	 * @param width
	 * @param height
	 */
	public BoardState(int width, int height) {
		this.width = width;
		this.height = height;
		this.state = new ArrayList<List<Integer>>(width);
		for(int w = 0 ; w < this.width ; ++w) {
			this.state.add(w, new ArrayList<Integer>(height));
			for(int h = 0 ; h < this.height; ++h) {
				this.state.get(w).add(h, -1);
			}
		}
	}
	
	/**
	 * Initializes all data points to the same value as the supplied BoardState
	 * and uses the same width and height.
	 * 
	 * @param copy
	 */
	public BoardState(BoardState copy) {
		this.width = copy.width;
		this.height = copy.height;
		this.state = new ArrayList<List<Integer>>(width);
		for(int w = 0 ; w < this.width ; ++w) {
			this.state.add(w, new ArrayList<Integer>(height));
			for(int h = 0 ; h < this.height; ++h) {
				this.state.get(w).add(h, copy.state.get(w).get(h));
			}
		}
	}
	
	public int get(int x, int y) {
		return this.state.get(x).get(y);
	}
	
	public int set(int x, int y, int state) {
		return this.state.get(x).set(y, state);
	}
	
	/**
	 * Interpret data in row y as a CardStack. Stack stops at first 0 or
	 * uninterpretable value.
	 */
	public CardStack getCards(int y) {
		CardStack stack = new CardStack();
		for(int x = 0 ; x < this.width ; ++x) {
			Integer cardRef = this.get(x, y);
			if(cardRef == 0) {
				break;
			}
			Card card = getCard(cardRef);
			if(card == null) {
				break;
			}
			stack.add(card);
		}
		return stack;
	}
	
	/**
	 * Update data in row y as a CardStack.  Fills rest of row with 0s.
	 */
	public void setCards(int y, CardStack cards) {
		for(int i = 0 ; i < cards.size() ; ++i) {
			set(i, y, getCardRef((Card) cards.get(i)));
		}
		for(int i = cards.size() ; i < this.width ; ++i) {
			set(i, y, 0);
		}
	}

	/**
	 * Interpret BoardState value as a Card. Could return null for a
	 * non-interpretable value (currently doesn't actually happen, just ends up with a weird suit and/or value).
	 */
	public static Card getCard(Integer cardRef) {
		boolean up = true;
		if(cardRef < 0) {
			cardRef = -cardRef;
			up = false;
		}
		int val = cardRef % 100;
		int suit = (cardRef-val)/100;
		return new Card(suit, val, up);
	}

	/** Get BoardState value representing a Card */
	public static Integer getCardRef(Card card) {
		int val = card.getValue();
		int suit = card.getSuit();
		
		int ref = val+(100*suit);
		if(!card.isFaceUp()) {
			ref = -ref;
		}
		return ref;
	}

}
