package com.blackledge.david.game;

/**
 * Represents a Playing Card
 * @version $Revision: 1.1 $
 */
public class Card implements Comparable<Card> {
    /**
     * @return a 2-5 character representation of the card such as "2h" for the 2 of
     * hearts, or "(10s)" for the 10 of spades, but face down. {@inheritDoc}
     */
    public String toString() {
//      String[] suitstring = new String[] {" <3"," c3-"," <>"," -C>"};
//    	String[] suitstring = new String[] {"u2665","u2663","u2666","u2660"}; // doesn't work (with backslashes)
      String[] suitstring = new String[] {"h","c","d","s"};
        String result = getValueString()+suitstring[getSuit()];
        if(!faceUp) {
            result = "("+result+")";
        }
        return result;
            
    }
    private static final String letters[] = new String[] {
        "A","2","3","4","5","6","7","8","9","10","J","Q","K"//,"A"
    };
    /**
     * Constant for a suit as returned by {@link #getSuit()}
     */
    public static final int HEART = 0;
    /**
     * Constant for a suit as returned by {@link #getSuit()}
     */
    public static final int CLUB = 1;
    /**
     * Constant for a suit as returned by {@link #getSuit()}
     */
    public static final int DIAMOND = 2;
    /**
     * Constant for a suit as returned by {@link #getSuit()}
     */
    public static final int SPADE = 3;
    
    /**
     * Constant for a suit color as returned by {@link #getSuitColor()}
     */
    public static final int RED = 0;
    /**
     * Constant for a suit color as returned by {@link #getSuitColor()}
     */
    public static final int BLACK = 1;
    
    /**
     * Constant for a Card Value as returned by {@link #getValue()}
     */
    public static final int KING = 13;
    /**
     * Constant for a Card Value as returned by {@link #getValue()}
     */
    public static final int QUEEN = 12;
    /**
     * Constant for a Card Value as returned by {@link #getValue()}
     */
    public static final int JACK = 11;
    /**
     * Constant for a Card Value as returned by {@link #getValue()} - the Ace is
     * valued as 1 here.
     */
    public static final int ACE = 1;
    
    private final int suit;
    private final int value;
    private boolean faceUp;

    /**
     * Create a card with the given suit, value, and face-up state.
     * @param suit
     * @param value
     */
    public Card(int suit, int value, boolean faceUp) {
        this.suit = suit;
        this.value = value;
        this.faceUp = faceUp;
    }
    
    /**
     * Check if two cards are equal (same suit, same value - ignores faceUp)
     */
    public boolean equals(Object obj) {
        if(obj instanceof Card) {
            Card card = (Card) obj;
            return card.getSuit() == getSuit() && card.getValue() == getValue();
        }
        return false;
    }
    
    /**
     *have to implement so hashmap works right.
     *uses same stuff minimum information as equals, combined into string, and gets the hashcode of that string.
     * {@inheritDoc}
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        String stringToHash = getSuit()+"\n"+getValue();
        return stringToHash.hashCode();
    }

    /**
     * The minimum valid value for a card
     * @return 1
     */
    public static int getMinValue() {
        return 1;
    }
    /**
     * The maximum valid value for a card
     * @return 13
     */
    public static int getMaxValue() {
        return 13;
    }
    
    /**
     * 
     * @return The Suit as an integer Code 
     */
    public int getSuit() {
        return this.suit;
    }
    
    /**
     * 
     * @return the integer value of this card.
     */
    public int getValue() {
        return this.value;
    }
    
    /**
     * 1==A, 11==J, 12==Q, 13==K, all others are integers in string form.
     * @return the value of this card as a String.
     */
    public String getValueString() {
        return letters[this.value-1];
    }

    /**
     * 
     * @return the Suit's color as an integer Code
     */
    public int getSuitColor() {
        if(getSuit() == Card.HEART || getSuit() == Card.DIAMOND)
            return Card.RED;
        else
            return Card.BLACK;
    }
    
    /**
     * Report whether this card is currently face up.
     * @return
     */
    public boolean isFaceUp() {
        return faceUp;
    }
    
    /**
     * Set whether this card is currently face up.
     * @param faceUp
     */
    public void setFaceUp(boolean faceUp) {
        this.faceUp = faceUp;
    }
    
    /**
     * Turns this card over.
     * @return the resulting FaceUp state (true if Face Up, false if not).
     */
    public boolean turnOver() {
        faceUp = !faceUp;
        return faceUp;
    }

    /**
     * Just sorts numerically, ignores suit.
     */
	public int compareTo(Card o) {
		if(o != null) {
			return getValue() - o.getValue();
		}
		return 0;
	}

}
