package com.blackledge.david.game;

import java.util.Collection;
import java.util.Vector;

/**
 * A collection of {@link Card} objects
 *
 * @version $Revision: 1.1 $
 */
public class CardStack extends Vector {

    public CardStack() {
        super();
    }
    public CardStack(Collection list) {
        super(list);
    }
    /**
     * calls super.{@link #add(Object)} if the object is a card, calls super.{@link #addAll(Collection)}
     * if the object is a CardStack.
     * @return whatever te super implementation returns.
     * @throws IllegalArgumentException if the object is not one of the above
     *         types.
     */
    public boolean add(Object o) {
        if(o instanceof Card)
            return super.add(o);
        if(o instanceof CardStack)
            return super.addAll((CardStack)o);
        else
            throw new IllegalArgumentException();
    }


    /**
     * Override for stacks with rules.
     * @param card the card that wants to be stacked on top of this instance.
     * @return true if not overridden.
     */
    public boolean canStack(Card card) {
        return true;
    }
    /**
     * 
     * @param stack the stack that wants to be stacked on top of this instance.
     * @return {@link #canStack(Card)} for the stack argument's bottomCard
     */
    public boolean canStack(CardStack stack) {
        return canStack(stack.bottomCard());
    }
    //showSpread
    /**
     * Determine if we are allowed to call {@link #pickUp(int)} without getting a null back.
     * @return true always
     */
    public boolean canPickUp(int i) {
        return true;
    }
    /**
     * Grab part of the stack off of the current instance, removing them from this instance.
     * @param i how deep to reach when picking up from this instance. 0 == top card
     * @return Stack of face-up cards from 0-i, or null if can't pick up or no cards.
     */
    public CardStack pickUp(int i) {
        if(!canPickUp(i) || size() == 0) {
            return null;
        }
        int endOffset = size()-1;
        if(i >= size()) {
            i = endOffset;
        }
        
        CardStack result = new CardStack();
        for(int ix = endOffset ; ix >= (endOffset-i) ; --ix) {
            Card card = (Card)get(ix);
            //TODO there are surely some games where you can pick up cards 
            //without turning them over, so this is a bad assumption
            card.setFaceUp( true );
            result.add( card );
        }
        // remove those same ones after we finish that loop.
        for(int ix = endOffset ; ix >= (endOffset-i) ; --ix) {
            remove(ix);
        }
        return result;
    }
    /**
     * @return the card on the bottom of the stack. - don't call on an empty stack.
     */
    public Card bottomCard() {
        return (Card)get( 0 );
    }
    /**
     * @return the card on the top of the stack. - don't call on an empty stack.
     */
    public Card topCard() {
        return (Card)get( size()-1 );
    }


    /**
     * Get a new CardStack instance that contains the same cards as this stack, but in reverse order.
     * @return a new CardStack instance the same size as this one.
     */
    public CardStack getReverse() {
        CardStack result = new CardStack();
        for(int j = size()-1; j >= 0 ; --j) {
            result.add( get( j ) );
        }
        return result;
    }
    

}
