package com.blackledge.david.game;

import java.awt.Color;
import java.awt.Point;

public class PlayerState {
	public final String playerID;
	public final String name;

	// black/white, or just a "team color" for identification
	public Color color; 
	
	public static enum STATE {DISCONNECTED, CONNECTING, PLAYING, IDLE, CHATTING };
	public STATE playerState = STATE.DISCONNECTED;
	
	public boolean isHuman;
	public boolean hasTurn;
	// 1 for simple win, time representation for player-specific countdown/up, etc.
	public long score;
	
	public Point position;
	
	public PlayerState(String id, String name) {
		this(id, name, true);
	}
	public PlayerState(String id, String name, boolean isHuman) {
		if(id == null) throw new IllegalArgumentException("Player ID may not be null");
		this.playerID = id;
		if (name == null) {
			if (isHuman) {
				name = "(Anonymous)";
			} else {
				name = "(Computer Player)";
			}
		}
		this.name = name;
		this.isHuman = isHuman;
	}
	
	public PlayerState(PlayerState p) {
		playerID = p.playerID;
		name = p.name;
		color = p.color;
		playerState = p.playerState;
		isHuman = p.isHuman;
		hasTurn = p.hasTurn;
		score = p.score;
		if(p.position != null)
		position = new Point(p.position.x, p.position.y);
	}

	@Override
	public boolean equals(Object obj) {
		if(obj instanceof PlayerState) {
			return ((PlayerState) obj).playerID.equals(this.playerID);
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return playerID.hashCode();
	}
	
	@Override
	public String toString() {
		StringBuffer buff = new StringBuffer();
		buff.append(name);
		return buff.toString();
	}
}
