package com.blackledge.david.html;

import java.util.HashMap;

/**
 * Separating all Unicode references to another class to deal with possible
 * problem. Macintosh may have an issue with this, so all references to this
 * class need to be wrapped with a try/catch and handle gracefully.
 * 
 * @author David
 * 
 */
public class UnicodeHelper {

	private static HashMap<Character, String> fixTextMap = new HashMap<Character, String>();

	
	//TODO considering below information, should probably do a different Map per font and per TiVo model
//	StringBuffer testText=new StringBuffer();
//	char c = 120;
//	for(int j = 0 ; j < 40 ; ++j) {
//		for(int i = 0 ; i < 50 ; ++i) {
//			++c;
//			testText.append((char)c);
//		}
//		testText.append("\n");
//	}
//	//TiVoHD extremely limited, no added chars, no chars at all in the area of the tivo chars (non-printing control chars in Latin-1/ASCII, mostly chars in windows-1252 )
//	//Premiere (and simulator):
//	//default 129 tivo guy
//	testText.append((char)129);
//	//default 137 check (permilli o/oo)
//	testText.append((char)137);
//	//default 141 double-check
//	testText.append((char)141);
//	//default 143 star, 144 half star
//	testText.append((char)143);
//	testText.append((char)144);
////	//default 149 bullet (bullet)
////	testText.append((char)149);
//	//default 157 smaller bullet
//	testText.append((char)157);
////	10(default) doesn't have post-latin (above 255) through 
////	343(?) (bunch of international chars) and later greek chars, but system does.
//	
////	true on simulator, but not on Premiere:
////	11(system) doesn't have  after ~ (126) and before upsidedown ! (160) (except a few tivo-specific ones that are in both, but diff locations)
//	//system 128 tivo (euro)
//	testText.append((char)128);
//	//system 129 check
//	testText.append((char)129);
//	//system 130 double-check (sbquo)
//	testText.append((char)130);
//	//system 131 star (fnof), 132 half star (bdquo)
//	testText.append((char)131);
//	testText.append((char)132);
//	//system 133 smaller bullet (hellip)
//	testText.append((char)133);
//	new BView(getNormal(), 50, 50, getWidth()-100, getHeight()).setResource(createText("10-15.font", Color.white,testText.toString()), 
//			RSRC_VALIGN_TOP|RSRC_HALIGN_LEFT);
//	new BView(getNormal(), 50, 50, getWidth()-100, getHeight()).setResource(createText("11-15.font", Color.white,testText.toString()),
////RSRC_VALIGN_BOTTOM|RSRC_HALIGN_LEFT);
//			RSRC_VALIGN_TOP|RSRC_HALIGN_RIGHT);

	
	/**
	 * Fix any characters that won't display right on the TiVo e.g. the
	 * character used by JSoup for nbsp
	 * 
	 * @param text
	 * @return the improved text - should be an equivalent width as much as
	 *         possible - this is usually called only during render, not during
	 *         other calculations that figure width based on the original text
	 */
	public static String fixTextForRendering(String text) {
		if(text == null || text.length() == 0) return text;
		if(fixTextMap.size() == 0) {
			// initialize the map
			// See JSoup's org.jsoup.nodes/entities-base.properties (and full)
			
			//FIXME just do a reverse mapping of the jsoup properties file to "close enough" characters and strings
			
			// 00a0 is the official "non-breaking space" character used by jsoup for nbsp
			fixTextMap.put((char)0x00a0, " ");
			// en/em space, thinspace
			fixTextMap.put((char)0x2002, " ");
			fixTextMap.put((char)0x2003, "  ");
			fixTextMap.put((char)0x2009, " ");
			
			// commonly used horizontal ellipses character (&hellip;)
			fixTextMap.put((char)0x2026, "...");
			// quotes
			fixTextMap.put((char)0x2018, "'"); // ' left
			fixTextMap.put((char)0x2019, "'"); // ' right
			fixTextMap.put((char)0x201C, "\""); // " left
			fixTextMap.put((char)0x201D, "\""); // " right
			// em-dash (and en-dash) appears to be a problem, too
			fixTextMap.put((char)0x2013, "-");
			fixTextMap.put((char)0x2014, "--");
			// how often do the rest of these get used? just slows down the conversion IMO.
//			fixTextMap.put((char)0x201B, "'"); // high 9
//			fixTextMap.put((char)0x201A, ","); // low 9	
//			fixTextMap.put((char)0x201F, "\""); // " high 9
//			fixTextMap.put((char)0x201E, ",,"); // " low 9
			fixTextMap.put((char)0x2022, ""+((char)0x00B7)); // bullet - use mid-dot
			fixTextMap.put((char)0x2122, "tm"); // trademark
//			fixTextMap.put((char)0x301D, "\""); // double prime left
//			fixTextMap.put((char)0x301E, "\""); // double prime right
//			fixTextMap.put((char)0x301F, ",,"); // low double prime
		}
		
		for(Character ch : fixTextMap.keySet()) {
			String rep = fixTextMap.get(ch);
			if(rep.length() == 1) {
				text = text.replace(ch, rep.charAt(0));
			} else {
				text = text.replace(ch.toString(), rep);
			}
		}
		
		return text;
	}
	
	public static final String getDiscBulletCharacter() {
		return 
				" "+(char)0x00B7+" "; // mid-dot
//				" "+(char)0x00B0+" "; // degree
//				" "+(char)0x00BA+" "; // ordinal male
//				" "+(char)0x00A4+" "; // "currency" symbol
	}
	public static final String getCircleBulletCharacter() {
		return " "+(char)0x00B0; // degree
	}
	public static final String getSquareBulletCharacter() {
		return " "+(char)0x25A0; // black square - or a "unknown character box"
	}

}
