package com.blackledge.david.html.hme;

import java.awt.Color;
import java.net.URL;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import com.blackledge.david.tivo.V49BApplication;

import com.tivo.core.ds.TeDict;
import com.tivo.hme.bananas.BApplication;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.Resource;
import com.tivo.hme.sdk.StreamResource;
import com.tivo.hme.sdk.HmeEvent.Key;

public class AudioHandlerView extends BView {

	private AudioView audioHolder;
	private boolean dispatchOverride = false;
	private long playingUntil = -1;

	public AudioHandlerView(BApplication app, int x, int y, int width, int height, boolean visible) {
		super(app.getAbove(), x, y, width, height, visible);
		
		Object audioBG = 
//				((WebBrowserApp)app).getTheme().getFormResource(FORM.INPUT);
				//TODO drive this from theme
				"com/tivo/hme/bananas/keyboard-datafield.png";
		new BView(this,0, 0, width/2, height).setResource(audioBG, RSRC_HALIGN_LEFT|RSRC_VALIGN_CENTER|RSRC_IMAGE_VFIT);
		new BView(this,width/2, 0, width/2, height).setResource(audioBG, RSRC_HALIGN_RIGHT|RSRC_VALIGN_CENTER|RSRC_IMAGE_VFIT);
	}

	public boolean keyDispatchOverride(Key ir) {
		if (this.dispatchOverride) {
			boolean press = (ir.getAction() == KEY_PRESS);
			switch(ir.getCode()) {
			case KEY_FORWARD:
				if(press)
					this.skip();
				return true;
			case KEY_PAUSE:
				if(press)
					this.pause();
				return true;
			case KEY_SLOW:
				if(press)
					this.close();
				return true;
			}
			return false;
		} else
			return false;
	}
	
//	@Override
//	public boolean handleEvent(HmeEvent event) {
//        if(event instanceof HmeEvent.ResourceInfo) {
//            ResourceInfo resInf = (ResourceInfo) event;
//            if(resInf.getOpCode() == Resource.EVT_RSRC_STATUS) {
//            	try {
//            		//TODO ok, not much we can do with the usual default duration
//            		//TODO instead, use "bitrate" field vs. the size header, if we've seen it.
////            		int position = Integer.parseInt(String.valueOf(resInf.getMap().get("position")));
////            		int duration = Integer.parseInt(String.valueOf(resInf.getMap().get("duration")));
//            		String pos = String.valueOf(resInf.getMap().get("pos"));
//            		int position = Integer.parseInt(pos.substring(0, pos.indexOf("/")));
//            		int duration = Integer.parseInt(pos.substring(pos.indexOf("/")+1));
//            		
//            		if(duration == 180000) {
//            			// default duration when it's not known.
//            		}
//            	} catch(Exception e) {
//            	}
//            }
//        }
//		return super.handleEvent(event);
//	}

	private BView getResourceParent() {
		return this.audioHolder;
	}
	
	public void pause() {
		if (this.audioHolder != null
				&& this.audioHolder.getResource() != null) {
			if(this.audioHolder.getResource() instanceof StreamResource) {
				((StreamResource) this.audioHolder.getResource()).pause();
			}
//			HmeEvent pauseEvent = new HmeEvent.Key(1, KEY_PRESS,
//					KEY_PAUSE, 0);
//			this.audioHolder.postEvent(pauseEvent);
//			pauseEvent = new HmeEvent.Key(1, KEY_RELEASE,
//					KEY_PAUSE, 0);
//			this.audioHolder.postEvent(pauseEvent);
		}
	}

	public void close() {
		this.playingUntil = -1;
		close(null);
	}
	private void close(Resource anim) {
		((WebBrowserApp)getBApp()).setAllowIdleTimeout(true);
		this.dispatchOverride = false;
		if(this.audioHolder != null) {
			if(this.audioHolder.getResource() != null) {
				//TODO how do we make sure they get audio_ended? they might have already gotten it.
				this.audioHolder.getResource().remove();
			}
			this.audioHolder.remove();
		}
		this.setTransparency(1.0f, anim);
		this.setVisible(false, anim);
	}

	public void startAudio(URL playURL, URL referer, String contentType) {
		this.setVisible(true);
		this.setTransparency(0.0f, getResource("*200"));
		
		if(this.audioHolder != null) {
			if(this.audioHolder.getResource() != null) {
				this.audioHolder.getResource().remove();
			}
			this.audioHolder.remove();
		}
		this.audioHolder = new AudioView(this, 4, 4, this.getWidth()-8, this.getHeight()-8);
		this.audioHolder.setActions("audio_started", "audio_next", "audio_ended");
		this.playingUntil = 0; // indefinite play length.
		this.audioHolder.startAudio(playURL, referer, contentType);
	}
	
	@Override
	public boolean handleAction(BView view, Object action) {
		if("audio_started".equals(action) || "audio_next".equals(action)) {
			((WebBrowserApp)getBApp()).setAllowIdleTimeout(false);
			this.playingUntil = 0; // indefinite play length.
			this.dispatchOverride = true;
		}
		if("audio_ended".equals(action)) {
			int hideMillis = 1000;
			this.playingUntil = new Date().getTime() + hideMillis + 200; // active until fade is done (with some buffer time for network)
			Resource anim = this.getResource("*" + hideMillis);
			// idle & dispatch handled in close method
			close(anim);
		}
		return super.handleAction(view, action);
	}

	public boolean isPlaying() {
		if (playingUntil > 0) {
			if (new Date().getTime() > playingUntil) {
				playingUntil = -1;
				return false;
			} else {
				return true;
			}
		} else {
			return playingUntil == 0;
		}
//		if(this.audioHolder == null) return false;
//		return this.audioHolder.isPlaying();
	}

	public void skip() {
		if (this.audioHolder != null && this.audioHolder.audioWatcher != null) {
			StreamResource stream = this.audioHolder.audioWatcher.startNextStream();
			this.audioHolder.setResource(stream);
		}
	}

	public class AudioView extends BView {
	
			private AudioWatcher audioWatcher;
			private String[] actions;
	
			public AudioView(AudioHandlerView audioHandlerView, int i, int j,
					int width, int height) {
				super(audioHandlerView, i, j, width, height);
				//TODO drive this resource from theme, I guess
				new BView(this,0, 0, width, height).setResource(Color.green.darker());
			}
	
			public void startAudio(URL urlBase, URL referer, String contentType) {
				BView resourceParent = this;
				WebBrowserApp app = ((WebBrowserApp)getBApp());
				TeDict cookie = null;
				if(contentType != null && contentType.toLowerCase().startsWith("audio/")) { //$NON-NLS-1$
					// .m3u - 
	//				debugOut("setUpStream: "+contentType); //$NON-NLS-1$
					List<String> m3uContentTypes = Arrays.asList(new String[] {"audio/x-mpegurl","audio/mpeg-url","application/x-winamp-playlist", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
							"audio/scm3u","audio/x-scm3u" // "audio/x-scm3u" served by everfreeradio.net and fits the pattern of the pls ones below //$NON-NLS-1$ //$NON-NLS-2$
							});
					List<String> plsContentTypes = Arrays.asList(new String[] {"audio/scpls","audio/x-scpls"}); //$NON-NLS-1$ //$NON-NLS-2$
					if (m3uContentTypes.contains(contentType.toLowerCase())) {
	//						// try "text/" here to default as M3U because PLS's start is more
	//						// definitively expected than M3U's basic "just a url" format which
	//						// is a sensible default anyhow
	//						" audio/x-mpegurl audio/mpeg-url application/x-winamp-playlist "//website included these for some reason" audio/scpls audio/x-scpls"
	//						.indexOf(" "+contentType.toLowerCase()+" ") >= 0) {
	//					#EXTM3U
	//					#EXTINF:secs,Title
	//					URL
						StreamResource stream = null;
						String file = app.connect.getRawText(urlBase, referer);
	//					debugOut("M3U File\n"+file);
						if(file != null) {
							if(file.startsWith("[playlist]")) { //$NON-NLS-1$
								stream = setUpPLSStream(resourceParent, file, referer, cookie);
							} else {
								stream = setUpM3UStream(resourceParent, file, referer, cookie);
							}
						}
						if (stream == null) {
							getBApp().play(V49BApplication.SOUND_BONK);
	//					} else {
	//						app.setAllowIdleTimeout(false);
						}
						return;
	//					return stream;
					}
					else if(plsContentTypes.contains(contentType.toLowerCase())) {
	//						" audio/scpls audio/x-scpls ".indexOf(" "+contentType.toLowerCase()+" ") >= 0) {
					// .pls - popular, simple, but "text/?" or audio/x-scpls
	//				[playlist]
	//				File1=URL
	//				Title1=Title
	//				Length1=secs
	//				NumberOfEntries=maxEntryNum
	//				Version=2
						StreamResource stream = null;
						String file = app.connect.getRawText(urlBase);
						if (file != null) {
							if (file.startsWith("#EXTM3U")) //$NON-NLS-1$
								stream = setUpM3UStream(resourceParent, file, referer, cookie);
							else {
								stream = setUpPLSStream(resourceParent, file, referer, cookie);
							}
						}
						if (stream == null) {
							getBApp().play(V49BApplication.SOUND_BONK);
	//					} else {
	//						app.setAllowIdleTimeout(false);
						}
						return;
	//					return stream;
					}
					StreamResource stream = createStream(getSafeStreamURL(urlBase.toString(), referer),
							contentType, true, cookie);
					//TODO move this view to getBApp().getAbove(); and redirect pause to this stream, tell it to remove itself when complete
					//TODO support playlists
					this.audioWatcher = newAudioWatcher(this/*.getNormal()*/, resourceParent, contentType);
					stream.addHandler(audioWatcher);
	//				setFocusDefault(audioWatcher);
	//				setFocus(audioWatcher);
					resourceParent.setResource(stream);
//					app.setAllowIdleTimeout(false);
					return;
	//				return stream;
				}
		
			}
			
			private String getSafeStreamURL(String original, URL referrer) {
				return ((WebBrowserApp)getBApp()).connect.getFixedAudioResourceSrc(original, referrer);
			}

			private AudioWatcher newAudioWatcher(BView audioView,
					BView resourceParent, String contentType) {
				AudioWatcher watcher = new AudioWatcher(audioView, resourceParent, contentType);
				if(this.actions != null) {
					watcher.setActions(this.actions[0], this.actions[1], this.actions[2]);
					this.actions = null;
				}
				watcher.setTextFont("default-10-italic.font", Color.white);
				return watcher;
			}

			private StreamResource setUpPLSStream(BView resourceParent, String file, URL referer, TeDict cookie) {
				this.audioWatcher = newAudioWatcher(this/*.getNormal()*/, resourceParent, null);
				audioWatcher.setPLS(file, referer);
				StreamResource stream = audioWatcher.startNextStream();
	//			setFocusDefault(audioWatcher);
	//			setFocus(audioWatcher);
				resourceParent.setResource(stream);
				//FIXME return first stream? not currently used anyhow
				return stream;
			}
	
			private StreamResource setUpM3UStream(BView resourceParent, String file, URL referer, TeDict cookie) {
				this.audioWatcher = newAudioWatcher(this/*.getNormal()*/, resourceParent, null);
				audioWatcher.setM3U(file, referer);
				StreamResource stream = audioWatcher.startNextStream();
	//			setFocusDefault(audioWatcher);
	//			setFocus(audioWatcher);
				resourceParent.setResource(stream);
				//FIXME return first stream? not currently used anyhow
				return stream;
			}
			
			public void setActions(String start, String next, String end) {
				if(this.audioWatcher != null)
				this.audioWatcher.setActions(start, next, end);
				else this.actions = new String[] {start, next, end};
			}
	
			public boolean isPlaying() {
				//FIXME allow for transition between playlist entries
				Resource stream = this.getResource();
				if(stream != null && stream instanceof StreamResource) {
					return stream.status >= RSRC_STATUS_READY && stream.status <= RSRC_STATUS_CLOSED;
				}
				return false;
			}
		}
	
	
}
