package com.blackledge.david.html.hme;

import java.awt.Color;

import com.tivo.hme.bananas.BHighlights;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.bananas.BSkin.Element;
import com.tivo.hme.sdk.Resource;

public class BScrollBar extends BView {
	private static int pagerHeight = 20;
	private static int scrollerWidth = 20;
	private BView scrollerTop;
	private BView scrollerBottom;
	private BView pageUp;
	private BView pageDown;
	private int scrollMaxHeight = -1;
	private int width;
	
	public BScrollBar(BView targetArea, BView parent) {
		this(targetArea, parent, -1);
	}
	
	public BScrollBar(BView targetArea, BView parent, int width) {
		super(parent, targetArea.getWidth()+targetArea.getX(), targetArea.getY()+pagerHeight,
				width>0?width:scrollerWidth, targetArea.getHeight()-pagerHeight*2);
		
		this.width = width;
		BScrollBar scrollBase = this;
		
		Object baseRes = getSkinResource("scroll_bg", Color.DARK_GRAY);
		int middlebg = scrollBase.getHeight()/2;
		BView scrollBGTop = new BView(scrollBase, 0, 0, scrollBase.getWidth(), middlebg);
		scrollBGTop.setResource(baseRes, RSRC_HALIGN_CENTER | RSRC_VALIGN_TOP | RSRC_IMAGE_HFIT);
		BView scrollBGBottom = new BView(scrollBase, 0, middlebg, scrollBase.getWidth(), middlebg);
		scrollBGBottom.setResource(baseRes, RSRC_HALIGN_CENTER | RSRC_VALIGN_BOTTOM | RSRC_IMAGE_HFIT);
		
		Object scrollRes = getSkinResource("scroll_bar",
//				getSkinResource(H_BAR, Color.blue));
				Color.blue);
		int middle = scrollBase.getHeight()/2;
		scrollerTop = new BView(scrollBase, 0, 0, scrollBase.getWidth(), middle);
		scrollerTop.setResource(scrollRes, RSRC_HALIGN_CENTER | RSRC_VALIGN_TOP | RSRC_IMAGE_HFIT);
		scrollerBottom = new BView(scrollBase, 0, middle, scrollBase.getWidth(), middle);
		scrollerBottom.setResource(scrollRes, RSRC_HALIGN_CENTER | RSRC_VALIGN_BOTTOM | RSRC_IMAGE_HFIT);
		
		pageUp = new BView(parent,
				scrollBase.getX(), scrollBase.getY()-pagerHeight,
				getScrollerWidth(), pagerHeight);
		BHighlights pageUpHighlight = pageUp.getHighlights();
		pageUpHighlight.setPageHint(H_PAGEUP, A_CENTER, A_BOTTOM);
		
		pageDown = new BView(parent,
				scrollBase.getX(), scrollBase.getY()
				+ scrollBase.getHeight(), getScrollerWidth(), pagerHeight);
		BHighlights pageDownHighlight = pageDown.getHighlights();
		pageDownHighlight.setPageHint(H_PAGEDOWN, A_CENTER, A_TOP);
		
		setVisible(false);
	}
	
	private Object getSkinResource(String highlightName, Object defaultRes) {
		Element skinElement = null;
		try {
			skinElement = getBApp().getSkin().get(highlightName);
		} catch (Exception e) {
			// if highlight isn't registered, throws a RuntimeException.
			return defaultRes;
		}
		Object res = null;
		if(skinElement != null) {
			res = skinElement.getResource();
		}
		if(res == null) {
			return defaultRes;
		} else {
			return res;
		}
	}

	private int getScrollerWidth() {
		return width>0?width:scrollerWidth;
	}

	@Override
	public void setVisible(boolean visible, Resource animation) {
		pageUp.setVisible(visible, animation);
		pageUp.getHighlights().setVisible(H_PAGEUP, H_VIS_FALSE);
		pageDown.setVisible(visible, animation);
		pageDown.getHighlights().setVisible(H_PAGEDOWN, H_VIS_FALSE);
		super.setVisible(visible, animation);
	}

	public void updateScroller(
			int totalRows, int topRowIndex, int bottomRowIndex, Resource scrollAnim) {
		
//		example:  10 rows, row 2 through 9 are showing (only row 0,1 are not showing)
		// lastRowShowing! (9==(10-1))
		// 8 visible rows (9-2+1)
		boolean firstRowShowing = (topRowIndex <= 0);
		boolean lastRowShowing = (bottomRowIndex >= (totalRows-1));
		int scrollHeight = -1;
		int scrollerMinHeight = 20;
		int scrollTop = 0;

		if(totalRows > 0) {
			int visibleRowCount = bottomRowIndex - topRowIndex+1;
			float percentDisplayed = ((float) visibleRowCount / (float) totalRows);
			float positionPercent = 0.0f;
			if (topRowIndex >= 0) {
				positionPercent = ((float) topRowIndex / (float) (totalRows - (bottomRowIndex - topRowIndex)));
			}
			
			scrollHeight = (int) (percentDisplayed * this.getHeight());
			scrollHeight = Math.max(scrollHeight, scrollerMinHeight);
			// save scrollheight so it only shrinks, never grows
			if (this.scrollMaxHeight < 0) {
				this.scrollMaxHeight = scrollHeight;
			} else {
				this.scrollMaxHeight = Math.min(this.scrollMaxHeight, scrollHeight);
				scrollHeight = this.scrollMaxHeight;
			}
			scrollTop = (int) (positionPercent * (this.getHeight() - scrollHeight));
		}
		
		if(scrollHeight < 0) {
			 scrollHeight = this.getHeight();
		}
		
		
		if(firstRowShowing && lastRowShowing) {
			setVisible(false);
		} else {
			setVisible(true);
			// note: used to do position separately so that animates but size
			// doesn't, but this should be OK and is more hme efficient
			int middle = scrollHeight/2;
			scrollerTop.setBounds(scrollerTop.getX(), scrollTop, scrollerTop.getWidth(), middle, scrollAnim);
			scrollerBottom.setBounds(scrollerTop.getX(), scrollTop+middle, scrollerTop.getWidth(), middle, scrollAnim);
		}
		pageUp.getHighlights().setVisible(H_PAGEUP,
				firstRowShowing ? H_VIS_FALSE : H_VIS_TRUE);
		pageDown.getHighlights().setVisible(H_PAGEDOWN,
				lastRowShowing ? H_VIS_FALSE : H_VIS_TRUE);
	}

}
