package com.blackledge.david.html.hme;

import java.awt.Color;

import com.blackledge.david.tivo.RemotePicker;
import com.blackledge.david.tivo.TiVoRemoteInfo;
import com.blackledge.david.tivo.V49IHmeProtocol;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.Resource;

/**
 * Handles WebBrowserApp's remote control hint decorations. Uses
 * WebScreen.CONTROL_HINT_* messages.
 */
class Hints implements V49IHmeProtocol {
	
	private WebBrowserApp web;
	public Hints(WebBrowserApp web) {
		this.web = web;
	}

	private BView leftInfo;
	private BView mediaHint;
	private BView keyboardHint;
	private Color hintTextColor = Color.black.brighter().brighter();
	private boolean resetHints = false;
	/**
	 * show the right decorations for the conditions.
	 * @param show fals when showing media or non-webscreen entrance, true on webscreen entrance
	 */
	void showDecorations(boolean show) {
		//called with "false" by webscreen when showing media, or on non-webscreen enters... called with "true" on webscreen enters.
		if(!web.isHintsShown()) {
			show = false;
		}
		
		if(leftInfo != null) {
			// TODO could make a new view the common parent of all hints and do a single "hide" call
			// ... but that's another view...we're trying to REDUCE views.
//			leftInfo.getParent().setVisible(show);
			leftInfo.setVisible(show);
			if (!show) {
				showActiveMediaHint(show);
				showActiveKeyboardHint(show);
			}
		}
	}
	void setupLeftHints(WebScreen webScreen) {
		setupLeftHints(webScreen, resetHints);
		resetHints = false;
	}
	void setupLeftHints(WebScreen webScreen, boolean reset) {
		WebBrowserApp wba = web;
		
		if(leftInfo == null || reset) {
			if(reset && leftInfo != null) {
				//FIXME remove all views and text resources recursively
				leftInfo.remove();
				leftInfo = null;
			}
			if(wba.isMaximized()) {
				// leftInfo not used.
				leftInfo = new BView(wba.getAbove(), 0, 0, 1,1, false);
			} else {
				leftInfo = new BView(wba.getAbove(),
						0, wba.getSafeActionVertical(),
						wba.getSafeTitleHorizontal() - 10,
						wba.getNormal().getHeight() - 2 * wba.getSafeActionVertical(),
						web.isHintsShown());
				// leftInfo is filled after prepare so it can adjust via html object
			}
		} else {
			showDecorations(true);
			if(wba.isMaximized()) {
				// leftInfo not used.
				leftInfo.setBounds(0, 0, 1,1);
				leftInfo.setVisible(false);
			} else {
				leftInfo.setBounds(
						0, wba.getSafeActionVertical(),
						wba.getSafeTitleHorizontal() - 10,
						wba.getNormal().getHeight() - 2 * wba.getSafeActionVertical());
				// leftInfo is filled after prepare so it can adjust via html object
				if (!web.isHintsShown()) {
					leftInfo.setVisible(false);
				}
			}
		}
		
		
		
		
		
		if(mediaHint == null || reset) {
			if(reset && mediaHint != null && keyboardHint != null) {
				//FIXME remove all views and text resources recursively
				mediaHint.remove();
				mediaHint = null;
				keyboardHint.remove();
				keyboardHint = null;
			}
			int h = 200;
			int titleHeight = 25;
			mediaHint = new BView((BView) leftInfo.getParent(), leftInfo.getX(), leftInfo.getY() + leftInfo.getHeight() - h,
					leftInfo.getWidth(), 
					h-titleHeight, // this.leftInfo.getHeight(), 
					false);
			mediaHint.setResource(Color.white.darker());
			
			keyboardHint = new BView((BView) leftInfo.getParent(), leftInfo.getX(), leftInfo.getY(), 
					leftInfo.getWidth(), 
					340-titleHeight, false);
	//					"(enter)\n" +
	//					"submit\n" +
	//					"\n" +
			keyboardHint.setResource(Color.white.darker());
			
			// uses html object to drive some hints
			setLeftHints(leftInfo, null/* FIXME webScreen.html*/, mediaHint, keyboardHint, Color.black.brighter());
			
			// add a title.
			BView mTitle = new BView((BView) mediaHint, 0, -titleHeight, mediaHint.getWidth(), 
					titleHeight);
			mTitle.setResource(mediaHint.getResource());
			new BView(mTitle, 0, 0, mTitle.getWidth(), mTitle.getHeight()).setResource(
							mediaHint.createText("system-10-italic.font", Color.black, Messages.getString("WebScreen.CONTROL_HINT_TITLE_MEDIA")), //$NON-NLS-1$ //$NON-NLS-2$
							RSRC_HALIGN_RIGHT | RSRC_VALIGN_TOP | RSRC_TEXT_WRAP);
			mediaHint.setBounds(mediaHint.getX(),  mediaHint.getY(), mediaHint.getWidth(), mediaHint.getHeight()+titleHeight);
			mediaHint.setTranslation(0, titleHeight);
			
			// add a title.
			BView kbTitle = new BView((BView) keyboardHint, 0, -titleHeight, keyboardHint.getWidth(), 
					titleHeight);
			kbTitle.setResource(keyboardHint.getResource());
			new BView(kbTitle, 0, 0, kbTitle.getWidth(), kbTitle.getHeight()).setResource(
							keyboardHint.createText("system-10-italic.font", Color.black, Messages.getString("WebScreen.CONTROL_HINT_TITLE_TEXT")), //$NON-NLS-1$ //$NON-NLS-2$
							RSRC_HALIGN_RIGHT | RSRC_VALIGN_TOP | RSRC_TEXT_WRAP);
			keyboardHint.setBounds(keyboardHint.getX(),  keyboardHint.getY(), keyboardHint.getWidth(), keyboardHint.getHeight()+titleHeight);
			keyboardHint.setTranslation(0, titleHeight);
		}
		
		
	}

	/**
	 * Set the color used in {@link #addHint(BView, int[], String, boolean)}
	 * calls
	 * 
	 * @param hintTextColor
	 */
	void setHintTextColor(Color hintTextColor) {
		if(hintTextColor != null) {
			this.hintTextColor = hintTextColor;
		}
	}
	
	private void setLeftHints(BView leftInfo, HTMLBreakdown html, BView mediaHint, BView keyboardHint, Color hintTextColor) {
		setMainHints(leftInfo, html, hintTextColor);
		
		setMediaHints(mediaHint, hintTextColor);
		setKeyboardHints(keyboardHint, hintTextColor);
	}
	
	/**
	 * Set up the keyboard hint info
	 * 
	 * @param keyboardHint
	 * @param hintTextColor
	 */
	void setKeyboardHints(BView keyboardHint, Color hintTextColor) {
		setHintTextColor(hintTextColor);
		if (keyboardHint != null) {
			addHint(keyboardHint, new int[]{KEY_SELECT}, Messages.getString("WebScreen.CONTROL_HINT_TEXT_KEYBOARD")); //$NON-NLS-1$
			addHint(keyboardHint, new int[]{KEY_NUM0, KEY_NUM9}, Messages.getString("WebScreen.CONTROL_HINT_TEXT_NUMBERS")); //$NON-NLS-1$
			addHint(keyboardHint, new int[]{KEY_RECORD}, Messages.getString("WebScreen.CONTROL_HINT_TEXT_PERIOD")); //$NON-NLS-1$
			addHint(keyboardHint, new int[]{KEY_ADVANCE}, Messages.getString("WebScreen.CONTROL_HINT_TEXT_DASH")); //$NON-NLS-1$
			addHint(keyboardHint, new int[]{KEY_REPLAY}, Messages.getString("WebScreen.CONTROL_HINT_TEXT_COMMA")); //$NON-NLS-1$
			addHint(keyboardHint, new int[]{KEY_REVERSE}, Messages.getString("WebScreen.CONTROL_HINT_TEXT_BACKSPACE")); //$NON-NLS-1$
			addHint(keyboardHint, new int[]{KEY_FORWARD}, Messages.getString("WebScreen.CONTROL_HINT_TEXT_SPACE")); //$NON-NLS-1$
			addHint(keyboardHint, new int[]{KEY_CLEAR}, Messages.getString("WebScreen.CONTROL_HINT_TEXT_CLEAR")); //$NON-NLS-1$
		}
	}
	
	/**
	 * Set up the media hint info
	 * 
	 * @param mediaHint
	 * @param hintTextColor
	 */
	void setMediaHints(BView mediaHint, Color hintTextColor) {
		setHintTextColor(hintTextColor);
		if (mediaHint != null) {
			addHint(mediaHint, new int[]{KEY_PAUSE}, Messages.getString("WebScreen.CONTROL_HINT_MEDIA_PAUSE")); //$NON-NLS-1$
			addHint(mediaHint, new int[]{KEY_SLOW}, Messages.getString("WebScreen.CONTROL_HINT_MEDIA_CLOSE")); //$NON-NLS-1$
			//TODO only show this if tivo model supports video?
			addHint(mediaHint, new int[]{KEY_OPT_PIP}, Messages.getString("WebScreen.CONTROL_HINT_MEDIA_ZOOM")); //$NON-NLS-1$
			addHint(mediaHint, new int[]{KEY_FORWARD}, Messages.getString("WebScreen.CONTROL_HINT_MEDIA_NEXT")); //$NON-NLS-1$
		}
	}
	
	/**
	 * Set up the main hint area with no html breakdown info (no access
	 * shortcuts)
	 * 
	 * @param leftInfo
	 * @param hintTextColor
	 */
	void setMainHints(BView leftInfo, Color hintTextColor) {
		setMainHints(leftInfo, null, hintTextColor);
	}
	private void setMainHints(BView leftInfo, HTMLBreakdown html, Color hintTextColor) {
		setHintTextColor(hintTextColor);
		if(leftInfo != null) {
			addHint(leftInfo, new int[]{KEY_REVERSE, V49IHmeProtocol.KEY_OPT_ACTION_B}, Messages.getString("WebScreen.CONTROL_HINT_BACK")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{KEY_CLEAR}, Messages.getString("WebScreen.CONTROL_HINT_QUIT")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{KEY_INFO}, Messages.getString("WebScreen.CONTROL_HINT_INFO")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{KEY_REPLAY}, Messages.getString("WebScreen.CONTROL_HINT_RELOAD")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{V49IHmeProtocol.KEY_OPT_ACTION_D}, Messages.getString("WebScreen.CONTROL_HINT_DIM")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{KEY_THUMBSUP}, Messages.getString("WebScreen.CONTROL_HINT_BOOKMARK")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{KEY_RECORD}, Messages.getString("WebScreen.CONTROL_HINT_BOOKMARK_LINK")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{KEY_PLAY}, Messages.getString("WebScreen.CONTROL_HINT_BOOKMARK_LIST")); //$NON-NLS-1$
			if (html == null || html.accessNumLinks == null ||
					(html.accessNumLinks[1] == null && html.accessNumLinks[2] == null && html.accessNumLinks[3] == null
					&& html.accessNumLinks[4] == null && html.accessNumLinks[5] == null && html.accessNumLinks[6] == null
					&& html.accessNumLinks[7] == null && html.accessNumLinks[8] == null && html.accessNumLinks[9] == null)) {
				addHint(leftInfo, new int[]{KEY_NUM1, KEY_NUM9}, Messages.getString("WebScreen.CONTROL_HINT_FONT_SIZE")); //$NON-NLS-1$
			}
			if(html == null || html.accessNumLinks == null || html.accessNumLinks[0] == null) {
				addHint(leftInfo, new int[]{KEY_NUM0}, Messages.getString("WebScreen.CONTROL_HINT_FONT_SIZE_DEFAULT")); //$NON-NLS-1$
			}
			addHint(leftInfo, new int[]{KEY_OPT_PIP}, Messages.getString("WebScreen.CONTROL_HINT_MOBILE")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{KEY_SELECT, KEY_FORWARD}, Messages.getString("WebScreen.CONTROL_HINT_ACTIVATE")); //$NON-NLS-1$
			addHint(leftInfo, new int[]{KEY_CHANNELDOWN, KEY_DOWN, KEY_ADVANCE}, Messages.getString("WebScreen.CONTROL_HINT_MOVE")); //$NON-NLS-1$
		}
	}
	
	/**
	 * Add horizontally-arranged key hints to the view.
	 * @param hintParent
	 * @param keys
	 * @param text
	 * @return
	 */
	BView addHintH(BView hintParent, int[] keys, String text) {
		return addHint(hintParent, keys, text, true);
	}
	/**
	 * Add vertically-arranged key hints to the view.
	 * @param hintParent
	 * @param keys
	 * @param text
	 * @return
	 */
	BView addHint(BView hintParent, int[] keys, String text) {
		return addHint(hintParent, keys, text, false);
	}
	
	/**
	 * Add key hints to the view
	 * 
	 * @param hintParent
	 * @param keys
	 * @param text
	 * @param horizontal
	 *            if true, arrangement is horizontal, vertical otherwise.
	 * @return
	 */
	BView addHint(BView hintParent, int[] keys, String text, boolean horizontal) {
		TiVoRemoteInfo remote = web.getRemote();
		return RemotePicker.addHint(remote, hintParent, keys, text, hintTextColor, horizontal);
	}

	/**
	 * shortcut for {@link #showActiveKeyboardHint(boolean)} and
	 * {@link #showActiveMediaHint(boolean)}
	 * 
	 * @param keyboard
	 * @param media
	 */
	void showActiveHints(boolean keyboard, boolean media) {
		showActiveKeyboardHint(keyboard);
		
		showActiveMediaHint(media);
	}

	/**
	 * Show or hide keyboard hint based on whether a keyboard-needing entity is
	 * active
	 * 
	 * @param keyboard
	 */
	private void showActiveKeyboardHint(boolean keyboard) {
		Resource anim = web.getResource("*500,0.2"); //$NON-NLS-1$
		if (this.keyboardHint != null && this.leftInfo != null) {
			if(keyboard && web.isHintsShown()) {
				if(!this.keyboardHint.getVisible()) {
					this.keyboardHint.setVisible(true);
					this.keyboardHint.setLocation(-this.keyboardHint.getWidth(), this.keyboardHint.getY());
					this.keyboardHint.setLocation(this.leftInfo.getX(), this.keyboardHint.getY(), anim);
				}
			} else {
				if(this.keyboardHint.getVisible()) {
					this.keyboardHint.setVisible(false, anim);
					this.keyboardHint.setLocation(this.leftInfo.getX(), this.keyboardHint.getY());
					this.keyboardHint.setLocation(-this.keyboardHint.getWidth(), this.keyboardHint.getY(), anim);
				}
			}
		}
	}

	/**
	 * show or hide based on whether there is any media operating - doesn't show
	 * if maximized
	 * 
	 * @param media
	 */
	void showActiveMediaHint(boolean media) {
		Resource anim = web.getResource("*500,0.2"); //$NON-NLS-1$
		if (this.mediaHint != null && this.leftInfo != null) {
			if (media && !web.isMaximized() && web.isHintsShown()) {
				if(!this.mediaHint.getVisible()) {
					this.mediaHint.setVisible(true);
					this.mediaHint.setLocation(-this.mediaHint.getWidth(), this.mediaHint.getY());
					this.mediaHint.setLocation(this.leftInfo.getX(), this.mediaHint.getY(), anim);
				}
			} else {
				if(this.mediaHint.getVisible()) {
					this.mediaHint.setVisible(false, anim);
					this.mediaHint.setLocation(this.leftInfo.getX(), this.mediaHint.getY());
					this.mediaHint.setLocation(-this.mediaHint.getWidth(), this.mediaHint.getY(), anim);
				}
			}
		}
	}
	
	/**
	 * Something has changed to effect how hints are displayed... regenerate
	 * them at next opportunity.
	 */
	void resetHints() {
		this.resetHints = true;
	}
}
