package com.blackledge.david.html.hme;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/*
 * PBS custom handling:  
http://video.pbs.org/video/2365171147/    =add Info=> http://video.pbs.org/videoInfo/2365171147/
this is json which contains "alternate_encoding": {"url": "http://urs.pbs.org/redirect/cf539e84061a49fe863bced3cb0d875a/", "bitrate": 0, "type": "download", "eeid": "mp4-baseline-16x9", "mime_type": "video/mp4"}
The URL will redirect to the mp4 file.

also has .ts files in an m3u8 file when you follow redirect of: "recommended_encoding": {"url": "http://urs.pbs.org/redirect/6556512c4b2c4af48f745fbd99a293e3/", "bitrate": 0, "type": "download", "eeid": "hls-800k-16x9", "mime_type": "application/x-mpegURL"}

 * 
 */
public class PBSVideoHandler extends VideoSourceHandler {

	public PBSVideoHandler(WebBrowserApp web) {
		super(web);
	}
	public boolean canHandle(URL url) {
		return url != null? url.toString().startsWith("http://video.pbs.org/video/"):false;
	}
	public URL getPosterImage(URL url) {
		return null;
	}
	public URL getVideoLink(URL url) {
		return url;
	}
	
	private Pattern urlpattern = Pattern.compile(".*_encoding\"\\s*:\\s*\\{\\s*\"url\"\\s*:\\s*\"([^\"]*)\"[^\\}]*\"([^\"]*video/[^\"]*)\"[^\\}]*\\}.*", Pattern.DOTALL);
	// break into two patterns since combining and making things optional forced them to always fail for some reason.
	private Pattern detailpattern1 = Pattern.compile("\\s*\\{\\s*\"title\"\\s*:\\s*\"([^\"]*)\"\\s*,\\s*(?:\"description\"\\s*:\\s*\"([^\"]*)\")?.*");
	private Pattern detailpattern2 = Pattern.compile(".*(?:\"rating\"\\s*:\\s*\"([^\"]*)\").*(?:\"duration\"\\s*:\\s*(\\d+)).*");
	private Pattern programTitle = Pattern.compile(".*\\s+\"program\"\\s*:\\s*\\{[^\\{]*\"title\"\\s*:\\s*\"([^\"]*)\".*");
	
	public boolean launchVideo(URL url, URL referer, boolean PIP) {

		// get url with /videoInfo/ instead of /video/
		String surl = url.toString();
		surl = surl.replace("/video/", "/videoInfo/");
		try {
			url = new URL(url, surl);
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// get its content and json interpret it
		String jsonContent = _web.connect.getRawText(url, referer);

		// get field (recommended_encoding or) alternate_encoding if its subitem mime_type is video/(compatible)
		Matcher urlmatcher = urlpattern.matcher(jsonContent);
		URL videoURL = null;
		String vidURL = null;
		String vidType = "video/mp4";
		if(urlmatcher.matches()) {
			vidURL = urlmatcher.group(1);
			vidType  = urlmatcher.group(2);
			try {
				videoURL = new URL(url, vidURL);
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if(videoURL == null) {
			_web.log("PBS Video FAIL", new String[] {vidURL, vidType, jsonContent});
			return false;
		}
		
		String vidDescription = null;
		String vidTitle = null;
		String vidSubtitle = null;
		int vidDuration = 0;

		Matcher programTitlematcher = programTitle.matcher(jsonContent);
		if(programTitlematcher.matches()) {
			vidTitle = programTitlematcher.group(1);
		}
		
		Matcher detailmatcher1 = detailpattern1.matcher(jsonContent);
		if(detailmatcher1.matches()) {
			vidSubtitle = detailmatcher1.group(1);
			vidDescription = detailmatcher1.group(2);
		}
		
		Matcher detailmatcher2 = detailpattern2.matcher(jsonContent);
		if(detailmatcher2.matches()) {
			//"rating": "TV-PG"
			if(vidDescription == null) vidDescription = "";
			vidDescription += " - "+detailmatcher2.group(1);
			try {
				vidDuration = Integer.parseInt(detailmatcher2.group(2));
			} catch(Exception e) {
			}
		}
				
		// and launch with that entry's url (will have redirects) and general file's duration, title, and description, maybe use rating as subtitle
		return launchVideoContent(videoURL, referer, vidType, PIP, vidTitle, vidSubtitle, vidDescription, vidDuration);
	}
	
	/**
	 * PBS page is just ugly and not so useful, so returning false.
	 */
	public boolean launchHasPage(URL url) {
		return false;
	}
}
