package com.blackledge.david.html.hme;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

public class RSSBreakdown {

	private Pattern xsltStylesheetPattern = Pattern.compile(
			"<?xml-stylesheet" + "\\s+"+
					"(" + // group 1
					"(href)\\s*=\\s*\""+"([^\"]*)"+"\"\\s*" + // group 2 & 3, 5 & 6, ...
					".*?" +
					"|" +
					"(type)\\s*=\\s*\""+"([^\"]*)"+"\"\\s*" + // group 2 & 3, 5 & 6, ...
					".*?" +
					"|" +
					"(media)\\s*=\\s*\""+"([^\"]*)"+"\"\\s*" + // group 2 & 3, 5 & 6, ...
					".*?" +
					")+" +
					"?>"
			, Pattern.CASE_INSENSITIVE);
//application/rss+xml
	private Pattern rssDocumentPattern = Pattern.compile(
			".*" + Pattern.quote("<") +"(rss|feed)\\s.*"
			, Pattern.CASE_INSENSITIVE|Pattern.DOTALL);
	private Pattern tivoConnectDocumentPattern = Pattern.compile(
			".*" + Pattern.quote("<") +"(TiVoContainer).*"
			, Pattern.CASE_INSENSITIVE|Pattern.DOTALL);
	private String rssStylesheet = null;
	private String rssSlideStylesheet = null;
	private String tivoConnectStylesheet = null;
		
	public RSSBreakdown(WebBrowserApp bapp) {
		this.rssStylesheet = bapp.getContext().getBaseURI().toString()+"rss.xsl";
		this.rssSlideStylesheet = bapp.getContext().getBaseURI().toString()+"rss-slideshow.xsl";
		// not technically rss, but this is the best place to handle this
		this.tivoConnectStylesheet = bapp.getContext().getBaseURI().toString()+"TiVoConnect.xsl";
	}

	String convertXMLToHTML(URL urlBase, String body)
			throws IOException {
		if(body != null) {
			//FIXME HACKish
			// delete any content before the prolog... I think getRawText lets status and maybe some header in.
			int start = body.indexOf("<");
			if(start > 0) 
				body = body.substring(start);
		}
		
		Matcher stylesheets = xsltStylesheetPattern.matcher(body);
		String chosenstylesheet = null;
		while(stylesheets.find()) {
			String stylesheet = null;
			String type = null;
			String media = null;
			for(int instance = 1; instance*3 <= stylesheets.groupCount() ; ++instance) {
				String attribute = stylesheets.group(instance+1);
				String value = stylesheets.group(instance+2);
				if(attribute.equalsIgnoreCase("href")) {
					stylesheet = value;
				} else if(attribute.equalsIgnoreCase("type")) {
					type = value;
				} else if(attribute.equalsIgnoreCase("media")) {
					media = value;
				}
			}
//			debugOut("STYLESHEET "+stylesheet+" "+type+" "+media);
			if(type == null || type.toLowerCase().contains("xsl")) {
				// use this one if we don't find another.
				chosenstylesheet = stylesheet;
				if (stylesheet != null
						&& ((type != null && type.toLowerCase().contains("xsl")) 
								|| stylesheet.toLowerCase().contains("xsl") )
						&& (media == null
								|| media.toLowerCase().trim().contains("tv") 
								|| media.toLowerCase().trim().contains("handheld")
								|| media.toLowerCase().trim().contains("screen") 
								|| media.toLowerCase().trim().contains("all"))) {
					// this is the best choice... stop looking
					//TODO prefer current prefs selection (WebBrowserApp.isMediaStyleActive)
					break;
				}
			}
		}
		debugOut("XML with stylesheet "+chosenstylesheet);
		boolean useSlideshow=false;
		if((chosenstylesheet == null) && tivoConnectDocumentPattern.matcher(body).matches()) {
			chosenstylesheet = tivoConnectStylesheet;
			debugOut("USING TIVOCONNECT STYLESHEET");
		}
		// default stylesheet for rss feeds and feedburner
		if((chosenstylesheet == null || urlBase.getHost().indexOf("feedburner.com")>=0) && rssDocumentPattern.matcher(body).matches()) {
			if(useSlideshow) {
				chosenstylesheet = rssSlideStylesheet;
			} else {
				chosenstylesheet = rssStylesheet;
			}
			debugOut("USING RSS STYLESHEET");
		}
//		debugOut("STYLESHEET CHOSEN "+chosenstylesheet);
		if(chosenstylesheet != null) {
			URL xsltUrl = URLConnectionHandler.getURL(urlBase, chosenstylesheet);
			InputStream xsltUrlStream = xsltUrl.openStream();
			Source xslt = new StreamSource(xsltUrlStream);
			TransformerFactory factory = TransformerFactory.newInstance();
			try {
				Transformer transformer = factory.newTransformer(xslt);
				transformer.clearParameters();
				transformer.setParameter("pHost",urlBase.getHost());
				if(useSlideshow) {// && urlBase != null) {
					debugOut("RSS LINK: "+String.valueOf(urlBase));
					String anchor = urlBase.getRef();
					String entry = "ENTER_WEBZ_RSS_ENTRY_";
					int entrynum = 0;
					if(anchor != null && anchor.contains(entry)) {
						try {
							entrynum = Integer.parseInt(anchor.substring(anchor
									.indexOf(entry) + entry.length()));
						} catch (Exception e) {
						}
					}
					if(urlBase.toString().contains("?")) {
						String file = urlBase.getFile();
						String end = "&"+entrynum;
						if (file.endsWith(end)) {
							// don't keep building up &num entries
							file = file.substring(0, file.length() - end.length());
						}
						transformer.setParameter("pPath", file+"#ENTER_WEBZ_RSS_ENTRY_"+entrynum);
					} else {
						transformer.setParameter("pPath", urlBase.getFile()+"?#ENTER_WEBZ_RSS_ENTRY_"+entrynum);
					}
				}
				Source text = new StreamSource(new StringReader( body ));
				StringWriter stringWriter = new StringWriter();
				transformer.transform(text, new StreamResult(stringWriter));
				return stringWriter.toString();
			} catch (TransformerConfigurationException e) {
				e.printStackTrace(); //FIXME better output
				return null;
			} catch (TransformerException e) {
				e.printStackTrace(); //FIXME better output
				return null;
			}
	
		} else {		
			return null;
		}
	}

	private void debugOut(String string) {
		WebBrowserApp.debugOut(string);
	}

}
