package com.blackledge.david.html.hme;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.regex.Pattern;

import org.jsoup.Connection;
import org.jsoup.Connection.Method;
import org.jsoup.Connection.Response;
import org.jsoup.Jsoup;
import org.jsoup.UnsupportedMimeTypeException;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import com.blackledge.david.html.ProxyCoder;
import com.blackledge.david.tivo.V49IHmeProtocol;

public class URLConnectionHandler implements V49IHmeProtocol {
//	private static final String METHOD_POST = "POST"; //$NON-NLS-1$
//	private static final String METHOD_GET = "GET"; //$NON-NLS-1$

	/** Jsoup default is 3 seconds. not long enough for our purposes */
	private static final int CONNECTION_TIMEOUT = 15 * 1000;

	static final String HTTP_HEADER_USER_AGENT = "User-Agent"; //$NON-NLS-1$
	static final String HTTP_HEADER_COOKIE = "Cookie"; //$NON-NLS-1$
	static final String HTTP_HEADER_SET_COOKIE = "Set-Cookie"; //$NON-NLS-1$
	/**
	 * If the field value is a relative URI, it SHOULD be interpreted relative
	 * to the Request-URI. The URI MUST NOT include a fragment.
	 */
	static final String HTTP_HEADER_REFERER = "Referer"; //$NON-NLS-1$
	/**
	 * For 201 (Created) responses, the Location is that of the new resource
	 * which was created by the request. For 3xx responses, the location SHOULD
	 * indicate the server's preferred URI for automatic redirection to the
	 * resource. The field value consists of a single absolute URI.
	 */
	static final String HTTP_HEADER_LOCATION = "Location"; //$NON-NLS-1$
	/** sent with 401 (Unauthorized) to define what is needed */
	static final String HTTP_HEADER_WWW_AUTHENTICATE = "WWW-Authenticate"; //$NON-NLS-1$
	/** usually in response to a 401 */
	static final String HTTP_HEADER_AUTHORIZATION = "Authorization"; //$NON-NLS-1$

	private WebBrowserApp wba;
	private ProxyCoder proxyHandler = new ProxyCoder(LIMIT_URL_NBYTES);
	public Pattern redirectErrorPattern = Pattern
	.compile("^HTTP Error 30[12]\\.\\s*(.+)\\s*$"); //$NON-NLS-1$

	public URLConnectionHandler(WebBrowserApp webBrowserApp) {
		wba = webBrowserApp;
	}

	public Response followRedirects(Response resp, String referer) throws IOException {
		return followRedirects(resp, 1, referer);
	}

	public Response followRedirects(Response resp, int count, String referer)
			throws IOException {
		if(resp == null) { WebBrowserApp.debugOut("REDIRECT failure: null resp"); return null;}
		int status = resp.statusCode();
		boolean needsRedirect = false;
		if (status != HttpURLConnection.HTTP_OK) {
			if (status == HttpURLConnection.HTTP_MOVED_TEMP
					|| status == HttpURLConnection.HTTP_MOVED_PERM
					|| status == HttpURLConnection.HTTP_SEE_OTHER)
				needsRedirect = true;
		}
		if (!needsRedirect)
			return resp;
		if (count > 15) {
			throw new IOException(MessageFormat.format(
					Messages.getString("WebBrowserApp.too_many_redirects_exception"), //$NON-NLS-1$
					resp.url(), resp.header(HTTP_HEADER_LOCATION)));
		}
		// WebBrowserApp.debugOut(String.format("REDIRECT %d URL %s -> %s", count, resp.url(),
		// resp.header(HTTP_HEADER_LOCATION)));
		WebBrowserApp.debugOut("REDIRECT " + count + " URL " + resp.url() + "-->" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				+ resp.header(HTTP_HEADER_LOCATION));
		String location = resp.header(HTTP_HEADER_LOCATION);
		if(location == null) {WebBrowserApp.debugOut("REDIRECT failure: null"); return resp;}
		URL redirectUrl = getURL(resp.url(), location);
		if(redirectUrl == null) {WebBrowserApp.debugOut("REDIRECT failure: null URL"); return resp;}
		Connection con = connect(redirectUrl.toString())
				// always redirect with a get. any data param from original req are
				// dropped.
				.followRedirects(false).method(Method.GET) 
				;
		if(con == null) {WebBrowserApp.debugOut("REDIRECT failure: null connection"); return resp;}
		if(referer != null) {
			// not in spec, but general consensus is to maintain referer on redirects
			con.referrer(referer);
		}
		Response res = con.execute();
			
	
		return followRedirects(res, count + 1, referer);
	}

	public String getFixedImageResourceSrc(String original, URL referrer) {
		String proxy = wba.getImageProxyURL();
		return getProxiedResourceSrc(original, proxy, referrer);
	}

	public String getFixedAudioResourceSrc(String original, URL referrer) {
		String proxy = wba.getImageProxyURL();
		return getProxiedResourceSrc(original, proxy, referrer);
	}

	public String getFixedVideoResourceSrc(String original, URL referrer) {
		String proxy = wba.getImageProxyURL();
		return getProxiedResourceSrc(original, proxy, referrer);
	}

	private String getProxiedResourceSrc(String original, String proxy, URL referrer) {
			ArrayList<String> args = new ArrayList<String>(2);
			if(referrer != null) {
				try {
					args.add("r="+URLEncoder.encode(referrer.toString(), "UTF-8"));
				} catch (UnsupportedEncodingException e) {
					// drop referrer.
				}
			}
			try {
				URL url = new URL(referrer,original);
				String cookie = wba.getBookmarks().getSavedCookie(url);
				if(cookie!= null && cookie.length() > 0) {
					args.add("c="+URLEncoder.encode(cookie, "UTF-8"));
				}
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			// to use the cookie:
	//		CookieStore cookies = ((CookieManager) CookieHandler.getDefault()).getCookieStore();
	//				try {
	//					List<HttpCookie> cookieSet = HttpCookie.parse(cookie2);
	//					for (HttpCookie cookie : cookieSet) {
	//			//			//force cookies to version 1 so we keep path/domain information!
	//			//			cookie.setVersion(1);
	//						// now that it parsed as version 1 to get multi-cookie, set to version 0 so it works properly on connections(?)
	//						cookie.setVersion(0);
	//						cookies.add(uri, cookie);
	//					} 
	//					}finally {}
			
			//TODO could pass m=POST arg for those cases
			String proxied = proxyHandler.getProxiedResourceSrc(original, proxy, args);
			if(proxied == null || proxied.equals(original)) {
				WebBrowserApp.debugOut("NO PROXY with "+original);
			} else {
				WebBrowserApp.debugOut("PROXIED:"+proxied);
			}
			return proxied;
		}

	static String getSrc(URL urlBase, String src) {
		if(src == null) return null;
		try {
			// fix spaces in url
			src = src.trim().replaceAll(" ", "%20");
			return new URL(urlBase, src).toString();
		} catch (MalformedURLException e) {
			WebBrowserApp.debugOut(e);
			return src;
		}
	}

	static URL getURL(URL urlBase, String src) {
		if(src == null) return null;
		try {
			src = src.trim().replaceAll(" ", "%20");
			return new URL(urlBase, src);
		} catch (MalformedURLException e) {
			WebBrowserApp.debugOut(e);
			return null;
		}
	}

	/* BEGIN JSoup impl */
	public Response doPost(String submitAction, String encoding,
			ArrayList<String[]> formSubmission, URL referer) {
		Connection soupCon = connect(submitAction)
				.method(Method.POST);
		if (formSubmission != null) {
			for(String[] submission : formSubmission) {
				if(submission[1] == null) {
					WebBrowserApp.debugOut("Null value for "+submission[0]+" not submitted");
					continue;
				}
				soupCon.data(submission[0],submission[1]);
			}
		}
		String referrer = null;
		if (referer != null) {
			referrer = referer.toString();
		}
		soupCon.referrer(referrer);
		
		WebBrowserApp.debugOut("POST:" + submitAction + "?" + formSubmission); //$NON-NLS-1$ //$NON-NLS-2$
		Response res;
		try {
			soupCon.followRedirects(false);
			res = soupCon.execute();
			res = followRedirects(res, referrer);
		} catch (IOException e) {
			// this throws IOException only on a real error (not http error, and
			// doesn't parse so not contenttype).
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		return res;
	}

	/**
	 * perform {@link Jsoup#connect(String)} and do some standard configuration
	 * (timeout, ignoreHttpErrors, and userAgent)
	 * 
	 * @param connectURL
	 * @return
	 */
	Connection connect(String connectURL) {
		//TODO find a way to add application/vnd.wap.xhtml+xml at beginning of "accept" to improve results
		return Jsoup.connect(connectURL)				
				.timeout(CONNECTION_TIMEOUT) // override the 3 second default
				.ignoreHttpErrors(true) // show errors as a page
				.userAgent(wba.getUserAgent())
				.header("Accept-Language", wba.getConnectionLanguages());
	}

	public Response getResponse(URL urlBase, URL referer) throws IOException {
		return getResponse(urlBase, referer, false);
	}

	public Response getResponse(URL urlBase, URL referer, boolean ignoreContentType) throws IOException {
			// This was just a stopgap, now JSoup throws
			// UnsupportedMimeTypeException
			// which gives the final content type and url without downloading stream
			// String contentGuess = guessContentType(urlBase);
			// // if we know this isn't expected to be a webpage, return null
			// if(contentGuess == null || contentGuess.startsWith("text/")) {
	
			Connection con = connect(urlBase.toString())
					// TODO could allow HTTP Exception instead and show as a popup
	//				.ignoreHttpErrors(false)
					;
			
			// special handling to do a TiVo passthrough for TiVoConnect URLs
			if(urlBase.getPath().equals("/TiVoConnect")) {
				WebBrowserApp.debugOut("/TiVoConnect: passing tivoID and tivoVersion in HTTP headers");
				con.header(V49IHmeProtocol.HTTP_HEADER_TIVO_ID, wba.tivoID)
				.header(V49IHmeProtocol.HTTP_HEADER_TIVO_VERSION, wba.tivoVersion);
			}
			
			con.ignoreContentType(ignoreContentType);
			
			String referrer = null;
			if (referer != null) {
				referrer = referer.toString();
				con.referrer(referrer);
			}
			// this throws IOException only on a real error (not http error, and
			// doesn't parse so not contenttype).
			// it also throws UnsupportedMimeTypeException and
			// SocketTimeoutException
			con.followRedirects(false);
			Response result = con.execute();
			result = followRedirects(result, referrer);
	
			return result;
			// } else {
			// return null;
			// }
		}

	public Response doGet(String submitAction, String encoding,
			ArrayList<String[]> formSubmission, URL referer) {
	
		Connection soupCon = connect(submitAction)
				.method(Method.GET);
		if (formSubmission != null) {
			for(String[] submission : formSubmission) {
				if(submission[1] == null) {
					WebBrowserApp.debugOut("Null value for "+submission[0]+" not submitted");
					continue;
				}
				soupCon.data(submission[0],submission[1]);
			}
		}
		String referrer = null;
		if (referer != null) {
			referrer = referer.toString();
		}
		soupCon.referrer(referrer);
	
		WebBrowserApp.debugOut("GET:" + submitAction + "?" + formSubmission); //$NON-NLS-1$ //$NON-NLS-2$
		Response res;
		try {
			soupCon.followRedirects(false);
			res = soupCon.execute();
			res = followRedirects(res, referrer);
		} catch (IOException e) {
			// this throws IOException only on a real error (not http error, and
			// doesn't parse so not contenttype).
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		return res;
	}

	/**
	 * {@link HttpURLConnection#guessContentTypeFromName(String)} plus some
	 * additional guesses
	 * 
	 * @param urlBase
	 * @return
	 */
	static String guessContentType(URL urlBase) {
		String contentGuess = HttpURLConnection
				.guessContentTypeFromName(urlBase.getPath());
		if (contentGuess == null) {
			if (urlBase.getPath() != null) {
				String path = urlBase.getPath().toLowerCase();
				if (path.endsWith(".mp4")) { //$NON-NLS-1$
					contentGuess = "video/mp4"; //$NON-NLS-1$
				} else if (path.endsWith(".mp3")) { //$NON-NLS-1$
					contentGuess = "audio/mpeg"; //$NON-NLS-1$
				}
			}
		}
		// WebBrowserApp.debugOut("guess: "+contentGuess+" "+urlBase.getPath()
		// +" --- "+urlBase);
		return contentGuess;
	}

	// public boolean handleStreamRedirectError(final BView view, HmeEvent
	// event,
	// String errorText, String contentType) {
	// if (errorText != null) {
	// Matcher m = redirectError.matcher(errorText);
	// if (m.matches()) {
	// WebBrowserApp.debugOut("Matched redirect error... "+m.group(1));
	// try {
	// //unfortunately, this url isn't from the error, it's the original one
	// String url = m.group(1);
	// if (url != null) {
	// URL resURL = new URL(url.trim());
	// URL newresURL = resolveRedirects(resURL, null);
	// if(!resURL.equals(newresURL)) {
	// Resource newres = view.createStream( newresURL.toString(), contentType,
	// true);
	// // Resource newres = view.getResource(newresURL);
	// if(newres != null) {
	// newres.addHandler(this);
	// view.setResource(newres);
	//
	// Resource res = this.getResource(event.getID());
	// if(res != null) {
	// res.removeHandler(this);
	// }
	// return true;
	// }
	// }
	// }
	// } catch (MalformedURLException e) {
	// // give up and continue with regular error display
	// }
	// }
	// }
	// return false;
	// }
	
	/**
	 * tries to go through all the redirects, including with cookies. If the
	 * result we don't get anywhere, returns null. Caller should check if result
	 * matches starting arg.
	 * 
	 * @param newresURL
	 * @param referer
	 * @return
	 */
	public URL resolveRedirects(URL newresURL, URL referer) {
		boolean redirect = true;
		while (redirect) {
//			URLConnection c;
			try {
				// FIXME replace withJSoup version! and add Referer to JSoup
				// version if that's correct to do!
				// TODO ok, did that, sortof...haven't really checked my work, though
				// FIXME this is only used by VideoWatcher currently
				String referrer = null;
				if (referer != null) {
					referrer = referer.toString();
				}
				Connection connection = connect(newresURL.toString()).followRedirects(false);
				if(referrer != null) {
					connection.referrer(referrer);
				}
				Response resp = followRedirects(connection.execute(), referrer);
				return resp.url();
				// c = newresURL.openConnection();
				// if(c instanceof HttpURLConnection) {
				// HttpURLConnection h = (HttpURLConnection) c;
				// h.setInstanceFollowRedirects(false);
				// if(referer != null) {
				// h.addRequestProperty(HTTP_HEADER_REFERER,
				// referer.toString());
				// }
				// try {
				// h.connect();
				// int respcode = h.getResponseCode();
				// if(respcode == 301 || respcode == 302) {
				// String newString = h.getHeaderField(HTTP_HEADER_LOCATION);
				// WebBrowserApp.debugOut("Redirect "+newresURL+" to "+newString);
				// if(newString == null || newString.trim().length() == 0) {
				// // give up and return what we have.
				// return newresURL;
				// }
				// newresURL = new URL(newresURL, newString.trim());
				// } else {
				// WebBrowserApp.debugOut("not redirect "+respcode+" "+h.getResponseMessage());
				// redirect = false;
				// }
				// } finally {
				// h.disconnect();
				// }
				// }
			} catch (UnsupportedMimeTypeException e) {
				// this is normal... we were just resolving redirects for a
				// media url, this proves that we found it!
				WebBrowserApp.debugOut("REDIRECT got mimetype exception: "+e);
				newresURL = getURL(newresURL, e.getUrl());
	
				// BUT that last step could have been a redirect AND a special
				// mime type (like on some archive.org links), so we try one
				// more time with redirects FOLLOWED to make sure.
				try {
					String referrer = null;
					if (referer != null) {
						referrer = referer.toString();
					}
					Connection connection = connect(newresURL.toString()).followRedirects(true);
					if(referrer != null) {
						connection.referrer(referrer);
					}
					Response resp = followRedirects(connection.execute(), referrer);
					WebBrowserApp.debugOut("REDIRECT extra MIME step: "+newresURL+" TO "+resp.url());
					return resp.url();
				} catch(UnsupportedMimeTypeException e2) {
					// use THIS version of exception now... this is DEFINITELY our best url source.
					e = e2;
				} catch (IOException e1) {
					// well, THAT was unexpected
					WebBrowserApp.debugOut("resolving redirects failed. "+e);
					// return the best we have.
					return newresURL;
				}
				
				try {
					return new URL(e.getUrl());
				} catch (MalformedURLException e1) {
					WebBrowserApp.debugOut("resolving redirects failed. "+e);
					return newresURL;
				}
			} catch (IOException e) {
				WebBrowserApp.debugOut("resolving redirects failed. "+e);
				// return the best we have.
				return newresURL;
			}
		}
		return newresURL;
	}

	public String getRawText(URL urlBase, URL referer) {
		try {
			// first try JSoup with ignoreContentType
			Response response = getResponse(urlBase, referer, true);
			return response.body();
		} catch (Exception e1) {
			
			try {
				URLConnection connection = urlBase.openConnection();
				//FIXME set headers!
				InputStream stream = connection.getInputStream();
				BufferedReader reader = new BufferedReader(new InputStreamReader(
						stream));
				StringBuffer result = new StringBuffer();
				for (String line = reader.readLine(); line != null; line = reader
						.readLine()) {
					result.append(line).append("\n"); //$NON-NLS-1$
				}
				stream.close();
				return result.toString();
			} catch (IOException e) {
				WebBrowserApp.debugOut("getRawText("+urlBase+"): "+e);
				return null;
			}
		}
	}

	/**
	 * Download a url's content and return as one big String. First tries JSoup
	 * with ignoreContentType, then just tries a direct connection (with no
	 * headers set) - which will probably never happen, actually.
	 * 
	 * @param urlBase
	 * @return
	 */
	public String getRawText(URL urlBase) {
		return getRawText(urlBase, null);
	}

	public String getHmeURL(URL link) {
		WebBrowserApp webBrowserApp = wba;
		// FIXME check out possible alternative "URL" for the apps:
		// at 204.176.49.116
		// Amazon: x-tivo:hme:uuid:35FE011C-3850-2228-FBC5-1B9EDBBE5863
		// Blockbuster: x-tivo:hme:uuid:63ED0C3D-D49C-9602-C322-E0A3D3EA5A3D
		// Browse Web Videos:
		// x-tivo:hme:uuid:19452283-D00B-C411-595D-447ABF62F37C
		// Browse Web Videos:
		// x-tivo:hme:uuid:6784571A-016C-8A46-DF2F-FD842D78006F
		// Fandango: x-tivo:hme:uuid:B9DDBECF-BC6D-5E33-388A-4CF840068D79
		// Music Choice: x-tivo:hme:uuid:7FDF1BEB-AF92-B8B6-6269-FAE44C8F887D
		// Netflix: x-tivo:hme:uuid:7edeb291-0db8-487f-b842-6b97bc71ad9a
		// One True Media: x-tivo:hme:uuid:D1AF7BED-325E-5E6E-B087-72381C92D077
		// Swivel: x-tivo:hme:uuid:2BD40EF7-D9D2-3CD4-0130-6C18855CE898
		// TiVoCast RSS: x-tivo:hme:uuid:863cb78f-efdd-4106-b572-51733983dc76
		// Youtube: x-tivo:hme:uuid:9AA364C9-CF8A-1E1D-B50F-CC65C40D4A96
	
		// It connects to port 80 and requests a file called TiVoConnect
		// http://209.97.196.52/TiVoConnect
		// which needs to return an xml file (described in the HMO docs I
		// believe)
		// apps.tv creates this on the fly based on the TiVo's ID which is sent
		// in the headers.
	
		// "lb._dns-sd._udp.0.0.168.192.in-addr.arpa."
		// 204.176.49.116, reverse it for this purpose
		// service+"116.49.176.204"+".in-addr.arpa."
		if (link == null)
			return null;
	
		String loc = null;
		int itemcount = -1;
		try {
			// 204.176.49.116 main one
			// 204.176.49.123: web page titled
			// "TiVo Premiere: much more than a DVR - TiVo"
			// 208.73.181.192 - no luck... apparently jmdns heartbeat thing
			// "have just noticed a "heartbeat" connection to 208.73.181.192 on port 5223"
			// mdns > 224.0.0.251 // nothing
			// http > 204.176.49.2 // nothing
			// 206.112.115.224,225 (0 entries)
			// 216.235.91.x (Live365) nothing (also with :20000)
			/*
			 * I was able to figure out some ports by examing my firewall log
			 * for dropped messages. I added TCP ports 7287, 7288, and 8000 (to
			 * IPs 204.176.49.x) which got the menu items and their icons to
			 * appear. For Live365 to work, I added ports 20000 through 22000
			 * (to IPs 216.235.91.x).
			 */
			// convert host to IP since that's the way TiVo requests TiVoConnect
			// (and it matters in some cases, like apps.tv)
			String ip;
			try {
				ip = InetAddress.getByName(link.getHost()).getHostAddress();
			} catch (UnknownHostException uhe) {
				// not that it will do any good, but continue with just the host
				// name.
				ip = link.getHost();
				webBrowserApp.log("hmeLaunchWarning", new String[] {link.toString(), uhe.toString()});
			}
			Connection connection = connect("http://" + ip //$NON-NLS-1$
					+ "/TiVoConnect?Command=QueryContainer&Container=%2F&DoGenres=1"); //$NON-NLS-1$
			// from a premiere:
	//			/TiVoConnect?Command=QueryContainer&Container=%2F&DoGenres=1
	//			/192.168.0.11
	//			User-Agent: TmkHttpRequest/1.0
	//			Host: 192.168.0.20
	//			TiVo_TCD_ID: 746000190AA690F
	//			TiVo_SW_VER: 20.2.2.1-01-2			
			// FIXME should do proper redirects with followRedirects and
			// .followRedirects(false)
			String tivoID = webBrowserApp.tivoID;
			String tivoVersion = webBrowserApp.tivoVersion;
			// if(this.tivoVersion == null)
			// // tivoVersion = "7.2.1-oth-01-2";
			// tivoVersion = "11.0c-01-2";
	
			WebBrowserApp.debugOut("app with tivo headers:" + tivoID + ":" + tivoVersion); //$NON-NLS-1$ //$NON-NLS-2$
			if (tivoID != null)
				connection.header(V49IHmeProtocol.HTTP_HEADER_TIVO_ID, tivoID);
			if (tivoVersion != null)
				connection.header(V49IHmeProtocol.HTTP_HEADER_TIVO_VERSION, tivoVersion);
			Document doc = connection.get();
			
			if(WebBrowserApp.getDebugLevel() > 0)
			{
				// store last xml attempt's result to deal with issues.
				try {
					String saveLocation = "/root/.tivo/"+webBrowserApp.tivoID+"-"+ip+"-TiVoConnect.xml";
					FileWriter fileWriter = new FileWriter(saveLocation);
					BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);
					bufferedWriter.write(doc.outerHtml());
					bufferedWriter.close();
				} catch(Throwable e) {
				}
			}			
	//debugOut(connection.response().headers());
	//debugOut(connection.response().body());
			// tivocontainer details
			// contenttype [x-container/tivo-server]
			// sourceformat [x-container/folder]
			// title
			// totalitems [int]
	
			// tivocontainer itemcount [int]
			// tivocontainer itemstart [int]
	
			// tivocontainer item
			Elements items = doc.select("tivocontainer item"); //$NON-NLS-1$
			itemcount  = items.size();
			for (Element item : items) {
				// details / contenttype [application/x-hme]
				// details / hidden [present-or-not boolean]
				boolean hidden = (item.select("details hidden").size() > 0); //$NON-NLS-1$
				// details / sourceformat [x-container/folder]
				// details / title
				String title = item.select("details title").text(); //$NON-NLS-1$
				// details / uuid [2BD40EF7-D9D2-3CD4-0130-6C18855CE898]
				String uuid = item.select("details uuid").text(); //$NON-NLS-1$
				// details / x-tivo-topleveldisplayorderhint [int]
				String orderHintText = item.select(
						"details x-tivo-topleveldisplayorderhint").text(); //$NON-NLS-1$
				// links / content / contenttype [application/x-hme]
				// links / content / url
				String contentUrlText = item.select("links content url").text(); //$NON-NLS-1$
				// links / customicon / url
				String iconUrlText = item.select("links customicon url").text(); //$NON-NLS-1$
	
	//				debugOut((hidden ? "(hidden)" : "") + title + "[" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	//						+ orderHintText + "]" + contentUrlText + " icon: " //$NON-NLS-1$ //$NON-NLS-2$
	//						+ iconUrlText);
	
				if (contentUrlText != null
						&& contentUrlText.trim().length() > 0) {
					try {
						URL contentUrl = new URL(contentUrlText);
	//						debugOut(contentUrl.getPath() + " vs. " //$NON-NLS-1$
	//								+ link.getPath() + " of " + link); //$NON-NLS-1$
						if (link.getPath().trim().length() > 0) {
							if (contentUrl.getPath().startsWith(link.getPath())) {
								WebBrowserApp.debugOut("matched " + link.getPath()); //$NON-NLS-1$
								if(loc == null) {
									loc = contentUrlText;
								} else {
									webBrowserApp.log("hmeLaunchWarning", new String[] {contentUrlText, "using "+loc+", but also matched "+contentUrlText});
								}
	//								not breaking so we can see other matches in log. break;
							}
						}
					} catch (MalformedURLException ex) {
						// just continue.
						webBrowserApp.log("hmeLaunchWarning", new String[] {contentUrlText, ex.toString()});
					}
	//				} else {
	//					if (contentUrlText != null) {
	//						loc = contentUrlText;
	//					}
				}
			}
	
			// Elements urls =
			// doc.select("tivocontainer item links content url");
			// debugOut(doc.toString());
			// debugOut("urls: "+urls);
			// if(urls != null && urls.size() > 0) {
			// loc = urls.get(0).text().trim();
			// }
			/*
			 * MY TIVO <tivocontainer> <details> <contenttype>
			 * x-container/tivo-server </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>TiVo Server</title>
			 * <totalitems> 27 </totalitems> </details> <itemcount> 27
			 * </itemcount> <itemstart> 0 </itemstart> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <hidden />
			 * <sourceformat> x-container/folder </sourceformat> <title>Credit
			 * Card Update (PCI enabled)</title> <uuid>
			 * 984BBCD1-F705-6F3C-8E1A-BA7F4C26B570 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 15
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.59:7287/CreditCard_20080912-
			 * 20080912_131340/1221250421/1289849840-1605209840/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/CreditCard_20080912
			 * -20080912_131340/1221250421/1289849840-1605209840/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Buy a TiVo
			 * Premiere</title> <uuid> 0F64DC01-096D-D7A9-1145-060E26DD0CE5
			 * </uuid> <x-tivo-topleveldisplayorderhint> 3
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.59:7287/TivoUpgrade-
			 * 20120424_122953/1335295796/1335399021-1650759021/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/TivoUpgrade
			 * -20120424_122953/1335295796/1335399021-1650759021/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Amazon v1.9
			 * (2010.10.25-0632)</title> <uuid>
			 * 35FE011C-3850-2228-FBC5-1B9EDBBE5863 </uuid> </details> <links>
			 * <content> <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/amazon-1-9.2011.03.16-0632-
			 * 20110316_154048/1300315250/1317939394-1633299394/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/amazon-1-9.2011.03.16-0632-
			 * 20110316_154048/1300315250/1317939394-1633299394/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>netflix 1.6</title>
			 * <uuid> 7EDEB291-0DB8-487F-B842-6B97BC71AD9A </uuid> </details>
			 * <links> <content> <contenttype> application/x-hme </contenttype>
			 * <url>
			 * https://204.176.49.65:7287/bundle-netflix-standalone.2011.08
			 * .10-1225-
			 * 20110810_150805/1313014088/1319001823-1634361823/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/bundle
			 * -netflix-standalone.2011.08.10-1225
			 * -20110810_150805/1313014088/1319001823-1634361823/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Order Domino's Pizza
			 * Now</title> <uuid> 6BA2A86B-619B-67A6-BFA3-0E137B167A5A </uuid>
			 * <x-tivo-topleveldisplayorderhint> 21
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176
			 * .49.65:7287/dominos.2009.05.07.r1021.logerror.signed-
			 * 20090507_174812/1241743696/1289849802-1605209802/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/dominos
			 * .2009.05.07.r1021.logerror.signed
			 * -20090507_174812/1241743696/1289849802-1605209802/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Browse and Buy Movie
			 * Tickets</title> <uuid> B9DDBECF-BC6D-5E33-388A-4CF840068D79
			 * </uuid> <x-tivo-topleveldisplayorderhint> 30
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/fandivo-1.1.2011.11.10-1645-
			 * 20111115_112508/1321385109/1347427227-1662787227/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/fandivo-1.1.2011.11.10-1645-
			 * 20111115_112508/1321385109/1347427227-1662787227/icon_SD.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Photos &amp;
			 * Slideshows</title> <uuid> BBC7824A-40C6-8B61-4D87-6A3D799F365B
			 * </uuid> <x-tivo-topleveldisplayorderhint> 50
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/hdphotos-ml_2010.09.13-1339-
			 * 20100913_134517/1284410719/1347904431-1663264431/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/hdphotos-ml_2010.09.13-1339-
			 * 20100913_134517/1284410719/1347904431-1663264431/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>TiVo Search 1.4 Actual
			 * App (2011.03.16-0112)</title> <uuid>
			 * 1D9DF84A-BDF7-421E-B1E9-9BA375C2EF0C </uuid>
			 * <x-tivo-topleveldisplayorderhint> 50
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/hdui-1.4.1.2011.03.16-0112-
			 * 20110316_154124/1300315286/1301509488-1616869488/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/hdui-1.4.1.2011.03.16-0112-
			 * 20110316_154124/1300315286/1301509488-1616869488/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Hme Down</title> <uuid>
			 * 9D99EC4B-B08C-191C-3255-40B58B1F5DAF </uuid> </details> <links>
			 * <content> <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/hmedown_20080317-
			 * 20080402_170823/1207181304/1240249987-1587405187/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/hmedown_20080317
			 * -20080402_170823/1207181304/1240249987-1587405187/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Guru Guides (3/08 -
			 * 1143)</title> <uuid> 403871C3-A324-21B8-1D8D-826834E758EB </uuid>
			 * </details> <links> <content> <contenttype> application/x-hme
			 * </contenttype> <url>
			 * https://204.176.49.65:7287/kidzone_v1.0-20100622-
			 * 20100622_141902/1277241543/1289849391-1605209391/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/kidzone_v1
			 * .0-20100622-20100622_141902/1277241543/1289849391-1605209391/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <hidden />
			 * <sourceformat> x-container/folder </sourceformat> <title>KidZone
			 * (3/8-1143)</title> <uuid> 5D524EE8-B977-1DC8-CD30-39BF60E7749C
			 * </uuid> </details> <links> <content> <contenttype>
			 * application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/kidzone_v1.0-20100622-
			 * 20100622_141902/1277241543/1289849349-1605209349/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/kidzone_v1
			 * .0-20100622-20100622_141902/1277241543/1289849349-1605209349/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>TiVo Search
			 * Beta</title> <uuid> AB306EFD-EB85-47C3-A38E-B29E52DB3C73 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 2
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/launcher-
			 * 1.3_2010.08.06-1858-20100806_190059/1281146460/1282091089-1597451089/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/launcher-
			 * 1.3_2010.08.06-1858-20100806_190059/1281146460/1282091089-1597451089/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Live365</title> <uuid>
			 * 9B26EF4C-BF08-A9A5-F18E-F898DA9ADEB6 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 30
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/live365-
			 * 1.0_2010.08.02-2340-20100802_234348/1280817829/1347904368-1663264368/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/live365-
			 * 1.0_2010.08.02-2340-20100802_234348/1280817829/1347904368-1663264368/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Music Videos from Music
			 * Choice</title> <uuid> 7FDF1BEB-AF92-B8B6-6269-FAE44C8F887D
			 * </uuid> <x-tivo-topleveldisplayorderhint> 49
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/musicchoice-1.0-2010.08.04-0632-
			 * 20100804_161353/1280963634/1328046275-1643406275/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/musicchoice-1.0-2010.08.04-0632-
			 * 20100804_161353/1280963634/1328046275-1643406275/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Rhapsody Music
			 * Service</title> <uuid> AAF73CE2-1AD4-304A-1272-EB8E817238A9
			 * </uuid> <x-tivo-topleveldisplayorderhint> 20
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/norhapsody.2011.08.15-1619-
			 * 20110815_162006/1313450407/1347904314-1663264314/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/norhapsody.2011.08.15-1619-
			 * 20110815_162006/1313450407/1347904314-1663264314/icon_SD.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Pandora</title> <uuid>
			 * 68AD6946-B063-C92F-091B-C3F769FA5B81 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 10
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/pandora-1.1.2011.05.24-1649-
			 * 20110524_165839/1306281520/1347904283-1663264283/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/pandora-1.1.2011.05.24-1649-
			 * 20110524_165839/1306281520/1347904283-1663264283/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Podcaster</title>
			 * <uuid> 6838E9A3-F968-683E-110E-6591066452A1 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 40
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/podcast-20100730-qbert-
			 * 20100730_180250/1280538174/1347904398-1663264398/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/podcast-20100730-qbert-
			 * 20100730_180250/1280538174/1347904398-1663264398/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>SameGame</title> <uuid>
			 * DC47F5CD-C97F-1C8F-EA91-CDDB4CFA1EFA </uuid>
			 * <x-tivo-topleveldisplayorderhint> 65
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/samegame_20071012_01-
			 * 20080402_173700/1207183021/1347427605-1662787605/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/samegame_20071012_01
			 * -20080402_173700/1207183021/1347427605-1662787605/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Skull &amp;
			 * Bones</title> <uuid> 1A73F688-60B2-561D-BBEA-354BB36D3DA4 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 80
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/skullbones_20071012-
			 * 20080402_173743/1207183066/1347427626-1662787626/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/skullbones_20071012
			 * -20080402_173743/1207183066/1347427626-1662787626/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Survey App</title>
			 * <uuid> 5A695917-238E-4218-8066-D4713A314063 </uuid> </details>
			 * <links> <content> <contenttype> application/x-hme </contenttype>
			 * <url> https://204.176.49.65:7287/survey_20080530-
			 * 01_signed-20080530_150815/1212185297/1289848835-1605208835/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/survey_20080530
			 * -01_signed-20080530_150815/1212185297/1289848835-1605208835/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <hidden />
			 * <sourceformat> x-container/folder </sourceformat> <title>Swivel
			 * 2.8 U.S. Jump Target (03/16/11-0631)</title> <uuid>
			 * 49C791F9-7D48-38AB-8177-9C756D4EE2C0 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 50
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/swivel-2.8.3-2012-05-08-0603at553106-
			 * 20120508_094009/1336495211/1336595307-1651955307/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/swivel-2.8.3-2012-05-08-0603at553106-
			 * 20120508_094009/1336495211/1336595307-1651955307/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Swivel 2.8
			 * (03/16/11-0631)</title> <uuid>
			 * 2BD40EF7-D9D2-3CD4-0130-6C18855CE898 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 50
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/swivel-2.8.3-2012-05-08-0603at553106-
			 * 20120508_094009/1336495211/1336595232-1651955232/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/swivel-2.8.3-2012-05-08-0603at553106-
			 * 20120508_094009/1336495211/1336595232-1651955232/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>TiVoCast 2.8
			 * (2011.03.16-0115)</title> <uuid>
			 * 19452283-D00B-C411-595D-447ABF62F37C </uuid> </details> <links>
			 * <content> <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/tivocast-2.8-2012-06-26-0603at563743-
			 * 20120626_120132/1340737293/1344881428-1660241428/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/tivocast-2.8-2012-06-26-0603at563743-
			 * 20120626_120132/1340737293/1344881428-1660241428/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Video Player</title>
			 * <uuid> 863CB78F-EFDD-4106-B572-51733983DC76 </uuid> </details>
			 * <links> <content> <contenttype> application/x-hme </contenttype>
			 * <url> https://204.176.49.65:7287/videoplayer-0023-release-sst-
			 * 20090716_122445/1247772287/1333658049-1649018049/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/videoplayer-0023-release-sst-
			 * 20090716_122445/1247772287/1333658049-1649018049/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Wordsmith</title>
			 * <uuid> 41731A91-6B5D-7E7D-C495-9A1F399965C7 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 70
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/wordsmith_20071012-
			 * 20080402_180020/1207184421/1347428154-1662788154/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/wordsmith_20071012
			 * -20080402_180020/1207184421/1347428154-1662788154/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>YouTube</title> <uuid>
			 * 9AA364C9-CF8A-1E1D-B50F-CC65C40D4A96 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 5
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/youtube-1.0-2010.07.27-1808-
			 * 20100727_181419/1280279661/1336773245-1652133245/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/youtube-1.0-2010.07.27-1808-
			 * 20100727_181419/1280279661/1336773245-1652133245/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>YouTube 1.0 (tcast jump
			 * 7/10)</title> <uuid> 06F36A5F-FE5D-4F03-99A0-64B92896B47F </uuid>
			 * <x-tivo-topleveldisplayorderhint> 50
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/youtube-1.0-2010.07.27-1808-
			 * 20100727_181419/1280279661/1282259262-1597619262/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/youtube-1.0-2010.07.27-1808-
			 * 20100727_181419/1280279661/1282259262-1597619262/icon.png </url>
			 * </customicon> </links> </item> </tivocontainer>
			 */
	
			//
			//
			//
	
			/*
			 * TIVO: <tivocontainer> <details> <contenttype>
			 * x-container/tivo-server </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>TiVo Server</title>
			 * <totalitems> 20 </totalitems> </details> <itemcount> 20
			 * </itemcount> <itemstart> 0 </itemstart> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <hidden />
			 * <sourceformat> x-container/folder </sourceformat> <title>Credit
			 * Card Update (PCI enabled)</title> <uuid>
			 * 984BBCD1-F705-6F3C-8E1A-BA7F4C26B570 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 15
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.59:7287/CreditCard_20080912-
			 * 20080912_131340/1221250421/1289849840-1605209840/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/CreditCard_20080912
			 * -20080912_131340/1221250421/1289849840-1605209840/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Buy a TiVo
			 * Premiere</title> <uuid> 0F64DC01-096D-D7A9-1145-060E26DD0CE5
			 * </uuid> <x-tivo-topleveldisplayorderhint> 3
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.59:7287/TivoUpgrade-
			 * 20120424_122953/1335295796/1335399021-1650759021/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/TivoUpgrade
			 * -20120424_122953/1335295796/1335399021-1650759021/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Amazon v1.9
			 * (2010.10.25-0632)</title> <uuid>
			 * 35FE011C-3850-2228-FBC5-1B9EDBBE5863 </uuid> </details> <links>
			 * <content> <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/amazon-1-9.2011.03.16-0632-
			 * 20110316_154048/1300315250/1317939394-1633299394/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/amazon-1-9.2011.03.16-0632-
			 * 20110316_154048/1300315250/1317939394-1633299394/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Order Domino's Pizza
			 * Now</title> <uuid> 6BA2A86B-619B-67A6-BFA3-0E137B167A5A </uuid>
			 * <x-tivo-topleveldisplayorderhint> 21
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176
			 * .49.65:7287/dominos.2009.05.07.r1021.logerror.signed-
			 * 20090507_174812/1241743696/1289849802-1605209802/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/dominos
			 * .2009.05.07.r1021.logerror.signed
			 * -20090507_174812/1241743696/1289849802-1605209802/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Browse and Buy Movie
			 * Tickets</title> <uuid> B9DDBECF-BC6D-5E33-388A-4CF840068D79
			 * </uuid> <x-tivo-topleveldisplayorderhint> 30
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/fandivo-1.1.2011.11.10-1645-
			 * 20111115_112508/1321385109/1343238702-1658598702/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/fandivo-1.1.2011.11.10-1645-
			 * 20111115_112508/1321385109/1343238702-1658598702/icon_SD.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Photos &amp;
			 * Slideshows</title> <uuid> BBC7824A-40C6-8B61-4D87-6A3D799F365B
			 * </uuid> <x-tivo-topleveldisplayorderhint> 47
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/hdphotos-ml_2010.09.13-1339-
			 * 20100913_134517/1284410719/1343239081-1658599081/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/hdphotos-ml_2010.09.13-1339-
			 * 20100913_134517/1284410719/1343239081-1658599081/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Hme Down</title> <uuid>
			 * 9D99EC4B-B08C-191C-3255-40B58B1F5DAF </uuid> </details> <links>
			 * <content> <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/hmedown_20080317-
			 * 20080402_170823/1207181304/1240249987-1587405187/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/hmedown_20080317
			 * -20080402_170823/1207181304/1240249987-1587405187/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Guru Guides (3/08 -
			 * 1143)</title> <uuid> 403871C3-A324-21B8-1D8D-826834E758EB </uuid>
			 * </details> <links> <content> <contenttype> application/x-hme
			 * </contenttype> <url>
			 * https://204.176.49.65:7287/kidzone_v1.0-20100622-
			 * 20100622_141902/1277241543/1289849391-1605209391/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/kidzone_v1
			 * .0-20100622-20100622_141902/1277241543/1289849391-1605209391/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <hidden />
			 * <sourceformat> x-container/folder </sourceformat> <title>KidZone
			 * (3/8-1143)</title> <uuid> 5D524EE8-B977-1DC8-CD30-39BF60E7749C
			 * </uuid> </details> <links> <content> <contenttype>
			 * application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/kidzone_v1.0-20100622-
			 * 20100622_141902/1277241543/1289849349-1605209349/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/kidzone_v1
			 * .0-20100622-20100622_141902/1277241543/1289849349-1605209349/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Live365</title> <uuid>
			 * 9B26EF4C-BF08-A9A5-F18E-F898DA9ADEB6 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 35
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/live365-
			 * 1.0_2010.08.02-2340-20100802_234348/1280817829/1343239177-1658599177/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/live365-
			 * 1.0_2010.08.02-2340-20100802_234348/1280817829/1343239177-1658599177/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Music Videos from Music
			 * Choice</title> <uuid> 7FDF1BEB-AF92-B8B6-6269-FAE44C8F887D
			 * </uuid> <x-tivo-topleveldisplayorderhint> 49
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/musicchoice-1.0-2010.08.04-0632-
			 * 20100804_161353/1280963634/1328046275-1643406275/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/musicchoice-1.0-2010.08.04-0632-
			 * 20100804_161353/1280963634/1328046275-1643406275/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Podcaster</title>
			 * <uuid> 6838E9A3-F968-683E-110E-6591066452A1 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 45
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/podcast-20100730-qbert-
			 * 20100730_180250/1280538174/1343239384-1658599384/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/podcast-20100730-qbert-
			 * 20100730_180250/1280538174/1343239384-1658599384/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>SameGame</title> <uuid>
			 * DC47F5CD-C97F-1C8F-EA91-CDDB4CFA1EFA </uuid>
			 * <x-tivo-topleveldisplayorderhint> 65
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/samegame_20071012_01-
			 * 20080402_173700/1207183021/1343239500-1658599500/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/samegame_20071012_01
			 * -20080402_173700/1207183021/1343239500-1658599500/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Skull &amp;
			 * Bones</title> <uuid> 1A73F688-60B2-561D-BBEA-354BB36D3DA4 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 80
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/skullbones_20071012-
			 * 20080402_173743/1207183066/1343239554-1658599554/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/skullbones_20071012
			 * -20080402_173743/1207183066/1343239554-1658599554/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Survey App</title>
			 * <uuid> 5A695917-238E-4218-8066-D4713A314063 </uuid> </details>
			 * <links> <content> <contenttype> application/x-hme </contenttype>
			 * <url> https://204.176.49.65:7287/survey_20080530-
			 * 01_signed-20080530_150815/1212185297/1289848835-1605208835/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/survey_20080530
			 * -01_signed-20080530_150815/1212185297/1289848835-1605208835/icon.png
			 * </url> </customicon> </links> </item> <item> <details>
			 * <contenttype> application/x-hme </contenttype> <hidden />
			 * <sourceformat> x-container/folder </sourceformat> <title>Swivel
			 * 2.8 U.S. Jump Target (03/16/11-0631)</title> <uuid>
			 * 49C791F9-7D48-38AB-8177-9C756D4EE2C0 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 50
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/swivel-2.8.3-2012-05-08-0603at553106-
			 * 20120508_094009/1336495211/1336595307-1651955307/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/swivel-2.8.3-2012-05-08-0603at553106-
			 * 20120508_094009/1336495211/1336595307-1651955307/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Swivel 2.8
			 * (03/16/11-0631)</title> <uuid>
			 * 2BD40EF7-D9D2-3CD4-0130-6C18855CE898 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 50
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/swivel-2.8.3-2012-05-08-0603at553106-
			 * 20120508_094009/1336495211/1336595232-1651955232/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/swivel-2.8.3-2012-05-08-0603at553106-
			 * 20120508_094009/1336495211/1336595232-1651955232/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>TiVoCast 2.8
			 * (2011.03.16-0115)</title> <uuid>
			 * 19452283-D00B-C411-595D-447ABF62F37C </uuid> </details> <links>
			 * <content> <contenttype> application/x-hme </contenttype> <url>
			 * https://204.176.49.65:7287/tivocast-2.8-2012-06-26-0603at563743-
			 * 20120626_120132/1340737293/1344881428-1660241428/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/tivocast-2.8-2012-06-26-0603at563743-
			 * 20120626_120132/1340737293/1344881428-1660241428/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <hidden /> <sourceformat>
			 * x-container/folder </sourceformat> <title>Video Player</title>
			 * <uuid> 863CB78F-EFDD-4106-B572-51733983DC76 </uuid> </details>
			 * <links> <content> <contenttype> application/x-hme </contenttype>
			 * <url> https://204.176.49.65:7287/videoplayer-0023-release-sst-
			 * 20090716_122445/1247772287/1333658049-1649018049/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/videoplayer-0023-release-sst-
			 * 20090716_122445/1247772287/1333658049-1649018049/icon.png </url>
			 * </customicon> </links> </item> <item> <details> <contenttype>
			 * application/x-hme </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>Wordsmith</title>
			 * <uuid> 41731A91-6B5D-7E7D-C495-9A1F399965C7 </uuid>
			 * <x-tivo-topleveldisplayorderhint> 70
			 * </x-tivo-topleveldisplayorderhint> </details> <links> <content>
			 * <contenttype> application/x-hme </contenttype> <url>
			 * http://204.176.49.116:7288/wordsmith_20071012-
			 * 20080402_180020/1207184421/1343239627-1658599627/clientip_67.0.59.230/
			 * </url> </content> <customicon> <url>
			 * http://204.176.49.116:7288/wordsmith_20071012
			 * -20080402_180020/1207184421/1343239627-1658599627/icon.png </url>
			 * </customicon> </links> </item> </tivocontainer>
			 * 
			 * 
			 * 
			 * <!--?xml version="1.0" encoding="ISO-8859-1"?--> <html>
			 * <head></head> <body> <tivocontainer> <details> <contenttype>
			 * x-container/tivo-server </contenttype> <sourceformat>
			 * x-container/folder </sourceformat> <title>TiVo Server</title>
			 * <totalitems> 0 </totalitems> </details> <itemcount> 0
			 * </itemcount> <itemstart> 0 </itemstart> </tivocontainer> <!--
			 * Copyright (c) 2002-2009 TiVo, Inc. --> </body> </html>
			 */
			/*
			 * <html> <head></head> <body> <tivocontainer> <details>
			 * <contenttype> x-container/tivo-server </contenttype>
			 * <sourceformat> x-container/folder </sourceformat> <totalitems> 1
			 * </totalitems> <title>apps.tv HME Server</title> </details>
			 * <itemstart> 0 </itemstart> <itemcount> 1 </itemcount> <item>
			 * <details> <contenttype> application/x-hme </contenttype>
			 * <sourceformat> x-container/folder </sourceformat> <title>apps.tv
			 * Subscription Manager</title> </details> <links> <content> <url>
			 * http://209.97.196.52:2281/submanager/ </url> </content> </links>
			 * </item> </tivocontainer> </body> </html>
			 */
			/*
			 * ticketchannel: <html> <head></head> <body> <tivocontainer>
			 * <details> <contenttype> x-container/tivo-server </contenttype>
			 * <sourceformat> x-container/folder </sourceformat> <totalitems> 1
			 * </totalitems> <title>Ticket Channel (Beta)</title> </details>
			 * <itemstart> 0 </itemstart> <itemcount> 1 </itemcount> <item>
			 * <details> <contenttype> application/x-hme </contenttype>
			 * <sourceformat> x-container/folder </sourceformat> <title>Ticket
			 * Channel (Beta)</title> </details> <links> <content> <url>
			 * http://208.116.35.238:7288/hme/ </url> </content> </links>
			 * </item> </tivocontainer> </body> </html>
			 */
			/*
			 * mp3tunes: <html> <head></head> <body> <tivocontainer> <details>
			 * <contenttype> x-container/tivo-server </contenttype>
			 * <sourceformat> x-container/folder </sourceformat> <totalitems> 1
			 * </totalitems> <title>MP3tunes HME Server</title> </details>
			 * <itemstart> 0 </itemstart> <itemcount> 1 </itemcount> <item>
			 * <details> <contenttype> application/x-hme </contenttype>
			 * <sourceformat> x-container/folder </sourceformat> <title>MP3tunes
			 * Locker</title> </details> <links> <content> <url>
			 * http://168.143.122.40:7288/mp3tunes/ </url> </content> </links>
			 * </item> </tivocontainer> </body> </html>
			 */
		} catch (IOException e1) {
			webBrowserApp.log("hmeLaunchWarning", new String[] {link.toString(), e1.toString()+" ITEMS:"+itemcount});
		}
	
		URL httplink;
		try {
			httplink = new URL("http" //$NON-NLS-1$
				+ link.toString().substring(link.getProtocol().length()));
		} catch(MalformedURLException e) {
			e.printStackTrace();
			httplink = link;
			webBrowserApp.log("hmeLaunchWarning", new String[] {link.toString(), e.toString()+" ITEMS:"+itemcount});
		}
	
		if (loc == null) {
			//FIXME support https, too
			loc = httplink.toString();
			webBrowserApp.log("hmeLaunchWarning", new String[] {link.toString(), "no TiVoConnect match found, using "+loc+" ITEMS:"+itemcount});
		} else if (link.getQuery() != null) {
			loc = loc + "?" + link.getQuery(); //$NON-NLS-1$
		}
		
		// handle possible relative URLs
		try {
			loc = URLConnectionHandler.getSrc(httplink, loc);
		} catch(Exception e) {
			//TODO uh... this is a problem.
			e.printStackTrace();
			webBrowserApp.log("hmeLaunchWarning", new String[] {loc.toString(), e.toString()+" ITEMS:"+itemcount});
		}
		return loc;
	}


//	public boolean handleStreamRedirectError(final BView view, HmeEvent
//			event,
//			String errorText, String contentType) {
//		if (errorText != null) {
//			Matcher m = redirectError.matcher(errorText);
//			if (m.matches()) {
//				debugOut("Matched redirect error... "+m.group(1));
//				try {
//					//unfortunately, this url isn't from the error, it's the original one
//					String url = m.group(1);
//					if (url != null) {
//						URL resURL = new URL(url.trim());
//						URL newresURL = resolveRedirects(resURL, null);
//						if(!resURL.equals(newresURL)) {
//							Resource newres = view.createStream( newresURL.toString(), contentType,
//									true);
//							// Resource newres = view.getResource(newresURL);
//							if(newres != null) {
//								newres.addHandler(this);
//								view.setResource(newres);
//
//								Resource res = this.getResource(event.getID());
//								if(res != null) {
//									res.removeHandler(this);
//								}
//								return true;
//							}
//						}
//					}
//				} catch (MalformedURLException e) {
//					// give up and continue with regular error display
//				}
//			}
//		}
//		return false;
//	}

	

	/* END JSoup impl */
}
