package com.blackledge.david.html.hme;

import java.net.URL;

import com.blackledge.david.tivo.V49BApplication;

public abstract class VideoSourceHandler {

	
	protected WebBrowserApp _web;


	public VideoSourceHandler(WebBrowserApp web) {
		_web = web;
	}
	
	/**
	 * Whether this handler thinks it can launch and provide other information
	 * for this URL.
	 * 
	 * @param url
	 * @return
	 */
	abstract public boolean canHandle(URL url);

	/**
	 * the Image that should be used for this video
	 * 
	 * @param url
	 * @return
	 */
	abstract public URL getPosterImage(URL url);

	/**
	 * The Link that should be used in a clickable link to make sure the launch
	 * works right later. Especially for handlers that are proxies that provide
	 * the right information for another handler to do the actual launching.
	 * Default implementation just returns argument
	 * 
	 * @param url
	 * @return
	 */
	public URL getVideoLink(URL url) {
		return url;
	}

	/**
	 * Launch the video player for this url/referer, possibly in PIP mode.
	 * 
	 * @param url
	 * @param referer
	 * @param PIP
	 * @return
	 */
	abstract public boolean launchVideo(URL url, URL referer, boolean PIP);

	/**
	 * If launchVideo was successful, is there a worthwhile web page to display
	 * for this same URL or not?  Default implementation just returns true so the page displays.
	 * 
	 * @param url
	 * @return
	 */
	public boolean launchHasPage(URL url) {
		return true;
	}

	
	/**
	 * If you don't need to do any special launching, just supply details to
	 * launch the usual video viewer.  Handles redirects and tries to handle strange content types, BONKs if it couldn't launch.
	 * 
	 * @param url
	 * @param referer
	 * @param contentType
	 * @param PIP
	 * @param title
	 * @param subtitle
	 * @param description
	 * @param duration
	 * @return
	 */
	protected boolean launchVideoContent(URL url, URL referer, String contentType, boolean PIP, String title, String subtitle, String description, int duration) {
		if(url != null) {
			
			//handle redirects, now.
			URL finalurl = _web.connect.resolveRedirects(url, referer);
			if(finalurl != null) {
				//TODO this could be the original URL which means we had a redirect loop, probably.
//				debugOut("redirected from "+url+" to "+finalurl); //$NON-NLS-1$
				url = finalurl;
			}
			
			//TODO can end up here with an HME application, should handle that.
			// replace wild and "download" types (application/* (octet-stream, x-msdownload)) with a better guess
			if (contentType == null
					|| contentType.trim().toLowerCase().startsWith("application/")) {
				//TODO BUT there are some valid application/ cases! hopefully this doesn't break them.
				String guess = URLConnectionHandler.guessContentType(url);
				if (guess != null) {
					_web.log("replaceMediaType", new String[] {String.valueOf(contentType), guess, String.valueOf(url)});
					contentType = guess;
				}
			}
			

			
			// strip off extra info
			if (contentType != null) {
				int semicolon = contentType.indexOf(";"); //$NON-NLS-1$
				if (semicolon > 0) {
					contentType = contentType.substring(0, semicolon);
				}
			}
			
//			TeDict cookie = getHTTPTeDict(url, referer);
			// .asx - xml, "video/x-ms-asf-plugin" (and audio/?)
			// .ram - includes weird realaudio stuff and mp3 files, too "audio/x-pn-realaudio", 
			// .qtl - xml, one entry only "application/x-quicktimeplayer"
			if(contentType != null && contentType.toLowerCase().startsWith("video/")) { //$NON-NLS-1$
				//Now, ALL video goes to video player... if it fails, try to use internal player.
				
				// if videoplayer launched, we don't have our own stream. Return null to cancel internal handling.
				// (but DON'T BONK)
				if(_web.playStreamVideo(url, contentType, PIP, title, subtitle, description, duration)) {
//_web.log("launchVideoContentOK", new String[]{ String.valueOf(url), contentType, String.valueOf(PIP), title, subtitle, description, String.valueOf(duration)});
					return true;
				}
			}
			_web.log("launchVideoContentFAIL", new String[]{ String.valueOf(url), contentType, String.valueOf(PIP), title, subtitle, description, String.valueOf(duration)});
			// notify user there was a problem - we TRIED to launch video, though.
			_web.play(V49BApplication.SOUND_BONK);
			return true;
			
			

		
		} else {
			_web.log("launchVideoContentFAIL(null URL)", new String[]{ String.valueOf(url), contentType, String.valueOf(PIP), title, subtitle, description, String.valueOf(duration)});
			// notify user there was a problem - we TRIED to launch video, though.
			_web.play(V49BApplication.SOUND_BONK);
//			debugOut("couldn't get media URL for vimeo video: "+urlBase); //$NON-NLS-1$
		}
		return true;
	}
	
//	private boolean handleMediaContent(String contentType, URL url, URL referer) {
////		this.isMediaContent = true;
//		
//		URL finalurl = _web.resolveRedirects(url, referer);
//		if(finalurl != null) {
//			//TODO this could be the original URL which means we had a redirect loop, probably.
////			debugOut("redirected from "+url+" to "+finalurl); //$NON-NLS-1$
//			url = finalurl;
//		}
//		
//		//TODO can end up here with an HME application, should handle that.
//		// replace wild and "download" types (application/* (octet-stream, x-msdownload)) with a better guess
//		if (contentType == null
//				|| contentType.trim().toLowerCase().startsWith("application/")) {
//			//TODO BUT there are some valid application/ cases! hopefully this doesn't break them.
//			String guess = WebBrowserApp.guessContentType(url);
//			if (guess != null) {
//				_web.log("replaceMediaType", new String[] {String.valueOf(contentType), guess, String.valueOf(url)});
//				contentType = guess;
//			}
//		}
//		
//		String[] mediaContentDetails = new String[] {String.valueOf(url), String.valueOf(referer), contentType, null};
//		boolean stream = setUpStream(url, referer, contentType);
//		// if stream == null, then likely no Watcher view was set up, regardless it didn't go well.
//		if(stream == true) {
//			//setUpStream BONKs (and logs handleMediaContentFailure) if it was really an error (vs. a non-stream solution)
//			return false;
//		}
//		_web.log("handleMediaContent", mediaContentDetails);
//		
//		return true;
//	}	
	
//	private boolean setUpStream(URL urlBase, URL referer, String contentType) {
//		WebBrowserApp app = _web;
////		app.hideLiveTV();
//		
//		// strip off extra info
//		if (contentType != null) {
//			int semicolon = contentType.indexOf(";"); //$NON-NLS-1$
//			if (semicolon > 0) {
//				contentType = contentType.substring(0, semicolon);
//			}
//		}
//		
//		TeDict cookie = getHTTPTeDict(urlBase, this.referer);
//		// .asx - xml, "video/x-ms-asf-plugin" (and audio/?)
//		// .ram - includes weird realaudio stuff and mp3 files, too "audio/x-pn-realaudio", 
//		// .qtl - xml, one entry only "application/x-quicktimeplayer"
//		if(contentType != null && contentType.toLowerCase().startsWith("video/")) { //$NON-NLS-1$
//			//Now, ALL video goes to video player... if it fails, try to use internal player.
//			
//			// if videoplayer launched, we don't have our own stream. Return null to cancel internal handling.
//			// (but DON'T BONK)
//			if(app.playStreamVideo(urlBase, contentType, false)) {
//				return true;
//			}
//		}
//		return false;
//	}	
}
