package com.blackledge.david.html.hme;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.regex.Pattern;

import com.blackledge.david.html.hme.WebBrowserApp.MEDIA;
import com.blackledge.david.html.hme.WebBrowserApp.MOBILE;
import com.blackledge.david.tivo.V49BApplication;
import com.tivo.hme.interfaces.IContext;

public class WebBrowserOptions {

	/** Whether User-Agent includes AvantGo, etc (see {@link #activeMobileType} so websites think we're mobile */
	private boolean appearMobile = true;
	/** current scale to use when rendering fonts */
	private float fontScale = 1.0f;
	private boolean leftSameAsBack = true;
	private boolean maximizedDisplay = false;
	private boolean shaded = false;
	private boolean hints = true;
	private boolean showTV = false;
	private MEDIA activeMediaStyle = MEDIA.TV;
	private MOBILE activeMobileType = MOBILE.SAMSUNG;
	private String remote = null;
	private String videoApp = null;
	private String imageProxy = null;
	private String audioProxy = null;
	private String videoProxy = null;
		
	private WebBrowserApp browser;
	private String showMediaURL;
	private static final String OPTIONS_KEY = "OPTIONS"; //$NON-NLS-1$
	public static final String URL_FONTSCALE = "fontscale";
	public static final String URL_THEME = "theme";
	public static final String URL_APPMODE = "disablebookmarks";
	public static final String URL_MOBILE = "mobile";
	public static final String URL_STYLE = "style";
	public static final String URL_TABLES = "tables";
	public static final String URL_MEDIA = "media";

	private enum OPTIONS {
		MOBILE, SHOW_TV, MEDIA_STYLE, FONT_SCALE, MAXIMIZED, LEFT_BACK, SHADED, MOBILE_TYPE, 
		REMOTE, VIDEO_APP, IMAGE_PROXY, AUDIO_PROXY, VIDEO_PROXY, HINTS
	}

	public WebBrowserOptions(WebBrowserApp webBrowser) {
		this.browser = webBrowser;
	}

	private IContext getContext() {
		return this.browser.getContext();
	}

	boolean isMediaStyleActive(MEDIA type) {
		return type.equals(this.activeMediaStyle);
	}

	MEDIA getMediaStyle() {
		return this.activeMediaStyle;
	}

	boolean isAppearingMobile() {
		return this.appearMobile;
	}

	MOBILE getMobileType() {
//		return MOBILE.DOCOMO;
//		return MOBILE.SAMSUNG;
//		return MOBILE.IPAD;
//		return MOBILE.AVANTGO;
		
//		return MOBILE.ANDROID;
//		return MOBILE.OPERA;
//		return MOBILE.BB;
		
//		return MOBILE.PALM;
		return this.activeMobileType;
	}
	
	boolean isMobileTypeActive(MOBILE type) {
		return type.equals(this.activeMobileType);
	}

	boolean isLeftSameAsBack() {
		return this.leftSameAsBack;
	}

	boolean isMaximized() {
		return this.maximizedDisplay;
	}
	
	boolean isShaded() {
		return this.shaded;
	}

	boolean isHintsShown() {
		return this.hints;
	}

	boolean isShowingLiveTV() {
		if(this.showTV) {
			return this.showMediaURL == null
					|| this.showMediaURL.equals(V49BApplication.STREAM_LIVETV);
		} else {
			return false;
		}
	}
	
	String getMediaURL() {
		if (isShowingLiveTV()) {
			return V49BApplication.STREAM_LIVETV;
		} else if (this.showTV) {
			return this.showMediaURL;
		}
		return null;
	}
	
	void loadOptions() {
		String optionsString = getContext().getPersistentData(OPTIONS_KEY);
		loadOptions(optionsString);
	}
	
	void loadOptions(String optionsString) {
		if (optionsString == null)
			return;
		WebBrowserApp.debugOut("LOAD OPTIONS: "+optionsString); //$NON-NLS-1$
		String[] options = optionsString.split(","); //$NON-NLS-1$
		if (options.length > OPTIONS.MOBILE.ordinal())
			this.appearMobile = Boolean.parseBoolean(options[OPTIONS.MOBILE.ordinal()]);
		if (options.length > OPTIONS.SHOW_TV.ordinal()) {
			this.showMediaURL = null;
			String opt = options[OPTIONS.SHOW_TV.ordinal()];
			if (opt.indexOf(':') >= 0) {
				this.showTV = true;
				this.showMediaURL = opt;
			} else {
				this.showTV = Boolean.parseBoolean(opt);
			}
		}
		if (options.length > OPTIONS.MEDIA_STYLE.ordinal() && options[OPTIONS.MEDIA_STYLE.ordinal()].trim().length() > 0)
			this.activeMediaStyle = MEDIA.valueOf(options[OPTIONS.MEDIA_STYLE.ordinal()]);
		if(options.length > OPTIONS.FONT_SCALE.ordinal())
			this.fontScale = Float.parseFloat(options[OPTIONS.FONT_SCALE.ordinal()]);
		if (options.length > OPTIONS.MAXIMIZED.ordinal())
			this.maximizedDisplay = Boolean.parseBoolean(options[OPTIONS.MAXIMIZED.ordinal()]);
		if (options.length > OPTIONS.LEFT_BACK.ordinal())
			this.leftSameAsBack = Boolean.parseBoolean(options[OPTIONS.LEFT_BACK.ordinal()]);
		if (options.length > OPTIONS.SHADED.ordinal())
			this.shaded = Boolean.parseBoolean(options[OPTIONS.SHADED.ordinal()]);
		if (options.length > OPTIONS.MOBILE_TYPE.ordinal() && options[OPTIONS.MOBILE_TYPE.ordinal()].trim().length() > 0)
			this.activeMobileType = MOBILE.valueOf(options[OPTIONS.MOBILE_TYPE.ordinal()]);
		if (options.length > OPTIONS.REMOTE.ordinal() && options[OPTIONS.REMOTE.ordinal()].trim().length() > 0)
			this.remote = String.valueOf(options[OPTIONS.REMOTE.ordinal()]);
		if (options.length > OPTIONS.VIDEO_APP.ordinal() && options[OPTIONS.VIDEO_APP.ordinal()].trim().length() > 0)
			this.videoApp = String.valueOf(options[OPTIONS.VIDEO_APP.ordinal()]);
		if (options.length > OPTIONS.IMAGE_PROXY.ordinal() && options[OPTIONS.IMAGE_PROXY.ordinal()].trim().length() > 0)
			this.imageProxy = String.valueOf(options[OPTIONS.IMAGE_PROXY.ordinal()]);
		if (options.length > OPTIONS.AUDIO_PROXY.ordinal() && options[OPTIONS.AUDIO_PROXY.ordinal()].trim().length() > 0)
			this.audioProxy = String.valueOf(options[OPTIONS.AUDIO_PROXY.ordinal()]);
		if (options.length > OPTIONS.VIDEO_PROXY.ordinal() && options[OPTIONS.VIDEO_PROXY.ordinal()].trim().length() > 0)
			this.videoProxy = String.valueOf(options[OPTIONS.VIDEO_PROXY.ordinal()]);
		if (options.length > OPTIONS.HINTS.ordinal())
			this.hints = Boolean.parseBoolean(options[OPTIONS.HINTS.ordinal()]);
	}

	void saveOptions() {
		String options = getOptionsString();
		getContext().setPersistentData(OPTIONS_KEY, options);
	}

	String getOptionsString() {
		StringBuffer optionsString = new StringBuffer();
		optionsString.append(this.appearMobile).append(","); // 0 //$NON-NLS-1$
		optionsString.append(escapeCommas(this.showMediaURL)).append(","); // 1 //$NON-NLS-1$
		optionsString.append(this.activeMediaStyle).append(","); // 2 //$NON-NLS-1$
		optionsString.append(this.fontScale).append(","); // 3 //$NON-NLS-1$
		optionsString.append(this.maximizedDisplay).append(","); //4 //$NON-NLS-1$
		optionsString.append(this.leftSameAsBack).append(","); //5 //$NON-NLS-1$
		optionsString.append(this.shaded).append(","); //6 //$NON-NLS-1$
		optionsString.append(this.activeMobileType).append(","); //7 //$NON-NLS-1$
		optionsString.append(this.remote==null?"":this.remote).append(","); //8 //$NON-NLS-1$
		optionsString.append(this.videoApp==null?"":this.videoApp).append(","); //9 //$NON-NLS-1$
		optionsString.append(this.imageProxy==null?"":this.imageProxy).append(","); //10 //$NON-NLS-1$
		optionsString.append(this.audioProxy==null?"":this.audioProxy).append(","); //11 //$NON-NLS-1$
		optionsString.append(this.videoProxy==null?"":this.videoProxy).append(","); //12 //$NON-NLS-1$
		optionsString.append(this.hints).append(","); //13 //$NON-NLS-1$
		
		return optionsString.toString();
	}

	private String escapeCommas(String url) {
		if(url == null) return "null";
		return url.replaceAll(Pattern.quote(","), "%2C");
	}

	/**
	 * triggers {@link #saveOptions()}
	 */
	void setMediaStyleActive(MEDIA type) {
		this.activeMediaStyle = type;
		saveOptions();
	}
	
	/**
	 * triggers {@link #saveOptions()}
	 */
	void setMobileType(MOBILE type) {
		this.activeMobileType = type;
		saveOptions();
	}
	
	float getFontScale() {
		return this.fontScale;
		// don't need to re-read it... it was read, and is saved every time it is set, so this is no different.
//		String optionsString = getContext().getPersistentData(OPTIONS_KEY);
//		if (optionsString == null)
//			return 0f;
//		String[] options = optionsString.split(","); //$NON-NLS-1$
//		if(options.length >= OPTIONS.FONT_SCALE.ordinal())
//			return Float.parseFloat(options[OPTIONS.FONT_SCALE.ordinal()]);
//		return 0f;
	}

	public String getRemoteID() {
		return this.remote;
	}

	public String getVideoApp() {
		// be extra cautious
		if (this.videoApp == null || this.videoApp.trim().length() == 0
				|| this.videoApp.equals("null")) {
			return "";
		}
		return this.videoApp;
	}
	
	public void setVideoApp(String app) {
		// options can't have the comma separator we use.
		if(app.contains(","))
			return;
		
		this.videoApp = app;
		
		saveOptions();
	}

	public String getImageProxy() {
//		return "http://192.168.0.20:81/";
		// be extra cautious
		if (this.imageProxy == null || this.imageProxy.trim().length() == 0
				|| this.imageProxy.equals("null")) {
			return "";
		}
		return this.imageProxy;
	}
	
	public void setImageProxy(String proxy) {
		// options can't have the comma separator we use.
		if(proxy.contains(","))
			return;
		
		this.imageProxy = proxy;
		
		saveOptions();
	}

	public String getAudioProxy() {
		// be extra cautious
		if (this.audioProxy == null || this.audioProxy.trim().length() == 0
				|| this.audioProxy.equals("null")) {
			return "";
		}
		return this.audioProxy;
	}
	
	public void setAudioProxy(String proxy) {
		// options can't have the comma separator we use.
		if(proxy.contains(","))
			return;
		
		this.audioProxy = proxy;
		
		saveOptions();
	}

	public String getVideoProxy() {
		// be extra cautious
		if (this.videoProxy == null || this.videoProxy.trim().length() == 0
				|| this.videoProxy.equals("null")) {
			return "";
		}
		return this.videoProxy;
	}
	
	public void setVideoProxy(String proxy) {
		// options can't have the comma separator we use.
		if(proxy.contains(","))
			return;
		
		this.videoProxy = proxy;
		
		saveOptions();
	}

	public void setRemoteID(String remote) {
		this.remote = remote;
		
		saveOptions();
	}
	
	/**
	 * triggers {@link #saveOptions()}
	 */
	void setAppearingMobile(boolean b) {
		this.appearMobile = b;
	
		saveOptions();
	}

	void setFontScale(float fontScale) {
		this.fontScale = fontScale;
		saveOptions();
	}

	void setLeftSameAsBack(boolean leftSameAsBack) {
		this.leftSameAsBack = leftSameAsBack;
		saveOptions();
	}

	/**
	 * triggers {@link #saveOptions()}
	 */
	void setMaximized(boolean b) {
		this.maximizedDisplay = b;
	
		saveOptions();
	}

	public void setShaded(boolean shade) {
		this.shaded = shade;
		
		saveOptions();
	}

	public void setHintsShown(boolean show) {
		this.hints = show;
		
		saveOptions();
	}

	/**
	 * triggers {@link #saveOptions()}
	 */
	void setShowLiveTV(boolean showTV) {
		this.showTV = showTV;
		if(this.showTV) {
			this.showMediaURL = null;
		}
	
		saveOptions();
	}
	
	public void setShowMedia(String media) {
		this.showTV = (media != null);
		this.showMediaURL = media;
		
		saveOptions();
	}

	public String getURLOptions() {
		StringBuffer o = new StringBuffer();
		o.append(URL_MOBILE).append("=");
		if(appearMobile && activeMobileType != null) {
			o.append(activeMobileType);
		} else {
			o.append(appearMobile);
		}
		o.append("&");
		o.append(URL_FONTSCALE).append("=").append(this.fontScale).append("&");
		o.append(URL_STYLE).append("=").append(this.activeMediaStyle.name()).append("&");
		if(this.showTV) {
			o.append(URL_MEDIA).append("=").append(urlEscape(V49BApplication.STREAM_LIVETV)).append("&");
		}
//		this.leftSameAsBack;
//		this.maximizedDisplay;
//		o.append(URL_APPMODE)
//		o.append(URL_TABLES)
//		o.append(URL_THEME)
		return o.toString();
	}

	private String urlEscape(String str) {
		try {
			return URLEncoder.encode(str, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			return str;
		}
	}

	/** @deprecated not complete */
	public void loadURLOptions(String stringURL) {
		URL url;
		try {
			url = new URL(stringURL);
		} catch (MalformedURLException e) {
			// FIXME Auto-generated catch block
			e.printStackTrace();
			return;
		}
		url.getQuery();
		//FIXME etc... if this is useful
	}

}

