<xsl:stylesheet version="2.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:atom="http://www.w3.org/2005/Atom"
    xmlns:atom3="http://purl.org/atom/ns#"
    xmlns:rss="http://purl.org/rss/1.0/"
    xmlns:rss09="http://my.netscape.com/rdf/simple/0.9/"  
    xmlns:media="http://search.yahoo.com/mrss/"  
	xmlns:content="http://purl.org/rss/1.0/modules/content/"
    exclude-result-prefixes="xsl rdf dc rss rss09 atom atom3"
>
<!-- ideas from two sources: rssfeed.xsl from http://ajamyajax.com -->
<!-- combined with  umm... the atom 1.0/.3 rss .90-2.0 template stuff from somewhere I forgot -->
<!--  mods by david.blackledge.com -->

<xsl:output method="html"/>

<xsl:param name="pPath" select="'?0#ENTER_WEBZ_RSS_ENTRY_0'"/>
<xsl:variable name="path" select="substring-before($pPath, '#ENTER_WEBZ_RSS_ENTRY_')"/>
<xsl:variable name="itemnumber" select="substring-after($pPath, '#ENTER_WEBZ_RSS_ENTRY_')"/>
<xsl:if test="$itemnumber = ''">
	<xsl:variable name="itemnumber" select="'0'"/>
</xsl:if>

<xsl:template match="/">
<!--  https://developer.mozilla.org/@api/deki/files/201/=Feed-icon-32x32.png -->
<LINK rel="shortcut icon" href="http://feedicons.com/images/feed-icon-28x28.png" type="image/png"/>
        
    
  <!-- Atom 1.0 -->
  <xsl:apply-templates select="/atom:feed"/>
  <!-- Atom 0.3 -->
  <xsl:apply-templates select="/atom3:feed"/>
  <!-- RSS 0.91 , 0.92 , 2.0 -->
  <xsl:apply-templates select="rss/channel" />
  <!-- RSS 1.0 -->
  <xsl:if test="rdf:RDF/rss:item">
  	<div class="rssitems">
      <xsl:apply-templates select="rdf:RDF/rss:item[position() = number($itemnumber)]"/>
  	</div>
  </xsl:if>
  <!-- RSS 0.9 -->
  <xsl:if test="rdf:RDF/rss09:item">
  	<div class="rssitems">
      <xsl:apply-templates select="rdf:RDF/rss09:item[position() = number($itemnumber)]"/>
  	</div>
  </xsl:if>
  
</xsl:template>


<!--
=============================================================================
 RSS 0.91,0.92,2.0
=============================================================================
-->
<xsl:template match="channel">
	<xsl:if test="number($itemnumber) = 0">
	    <xsl:call-template name="drawHeader">
	        <xsl:with-param name="header_link" select="link" />
	        <xsl:with-param name="header_title" select="title" />
	        <xsl:with-param name="header_image" select="image/url" />
	        <xsl:with-param name="header_image_title" select="image/title" />
	        <xsl:with-param name="header_image_link" select="image/link" />
	        <xsl:with-param name="header_pubDate" select="pubDate" />
	        <xsl:with-param name="header_description" select="description" />
	        <xsl:with-param name="header_postdate" select="dc:date" />
	        <xsl:with-param name="header_updated" select="updated" />
	    </xsl:call-template>
	</xsl:if>
	<xsl:if test="number($itemnumber) &gt; 0">
	    <xsl:apply-templates select="item[position() = number($itemnumber)]" />        
	</xsl:if>
</xsl:template>

<xsl:template match="item">
    <xsl:call-template name="drawItem">
        <xsl:with-param name="item_link" select="link" />
        <xsl:with-param name="item_title" select="title" />
        <xsl:with-param name="item_author" select="dc:creator" />
        <xsl:with-param name="item_pubDate" select="pubDate" />
        <xsl:with-param name="item_description" select="description" />
        <xsl:with-param name="item_postdate" select="dc:date" />
        <xsl:with-param name="item_updated" select="updated" />
        <xsl:with-param name="item_last" select="last()" />
    </xsl:call-template>
    
    <xsl:if test="content:encoded">
    	<xsl:value-of select="content:encoded" disable-output-escaping="yes"/>
    </xsl:if>
    
    <xsl:choose>
	    <xsl:when test="media:thumbnail">
			<xsl:if test="media:group/media:content[@medium='video']">
			 <VIDEO poster="{media:thumbnail/@url}" width="{media:thumbnail/@width}" height="{media:thumbnail/@height}">
			  <xsl:for-each select="media:group/media:content[@medium='video']">
			   <SOURCE src="{@url}" type="{@type}"/>
			  </xsl:for-each>
			 </VIDEO>
			</xsl:if>
			<!--  sometimes medium isn't specified -->
			<xsl:if test="media:group/media:content[@type='video/mp4']">
			 <VIDEO poster="{media:thumbnail/@url}" width="{media:thumbnail/@width}" height="{media:thumbnail/@height}">
			  <xsl:for-each select="media:group/media:content[@type='video/mp4']">
			   <SOURCE src="{@url}" type="{@type}"/>
			  </xsl:for-each>
			 </VIDEO>
			</xsl:if>
			<!--  TODO should probably loop for media:content top level -->
			<xsl:if test="media:content[@medium='video']">
			 <VIDEO poster="{media:thumbnail/@url}" width="{media:thumbnail/@width}" height="{media:thumbnail/@height}">
			   <SOURCE src="{media:content/@url}" type="{media:content/@type}"/>
			 </VIDEO>
			</xsl:if>
			<xsl:if test="media:content[@type='video/mp4']">
			 <VIDEO poster="{media:thumbnail/@url}" width="{media:thumbnail/@width}" height="{media:thumbnail/@height}">
			   <SOURCE src="{media:content/@url}" type="{media:content/@type}"/>
			 </VIDEO>
			</xsl:if>
		</xsl:when>
		<xsl:otherwise>
			<xsl:if test="media:group/media:content[@medium='video']">
			 <VIDEO>
			  <xsl:for-each select="media:group/media:content[@medium='video']">
			   <SOURCE src="{@url}" type="{@type}"/>
			  </xsl:for-each>
			 </VIDEO>
			</xsl:if>
			<!--  TODO should probably loop for media:content top level -->
			<xsl:if test="media:content[@medium='video']">
			 <VIDEO>
			   <SOURCE src="{media:content/@url}" type="{media:content/@type}"/>
			 </VIDEO>
			</xsl:if>
		</xsl:otherwise>
	</xsl:choose>
	
	<xsl:if test="media:group/media:content[@medium='audio']">
	 <xsl:for-each select="media:group">
		 <AUDIO>
		  <xsl:for-each select="media:content[@medium='audio']">
		   <SOURCE src="{@url}" type="{@type}"/>
		  </xsl:for-each>
		 </AUDIO>
	 </xsl:for-each>
	</xsl:if>
	<!--  sometimes medium isn't specified -->
	<xsl:if test="media:group/media:content[@type='audio/mpeg']">
	 <xsl:for-each select="media:group">
		 <AUDIO>
		  <xsl:for-each select="media:content[@type='audio/mpeg']">
		   <SOURCE src="{@url}" type="{@type}"/>
		  </xsl:for-each>
		 </AUDIO>
	 </xsl:for-each>
	</xsl:if>
 	<!--  TODO should probably loop for media:content top level -->
	<xsl:for-each select="media:content[@medium='audio']">
		<AUDIO>
		  <SOURCE src="{@url}" type="{@type}"/>
		</AUDIO>
	</xsl:for-each>
	<xsl:for-each select="media:content[@type='audio/mpeg']">
		<AUDIO>
		  <SOURCE src="{@url}" type="{@type}"/>
		</AUDIO>
	</xsl:for-each>
	
</xsl:template>


<!--
=============================================================================
 RSS 1.0
=============================================================================
-->
<xsl:template match="rss:item">     
<!-- rss:item -->
     <xsl:call-template name="drawItem">
        <xsl:with-param name="item_link" select="rss:link" />
        <xsl:with-param name="item_title" select="rss:title" />
        <xsl:with-param name="item_author" select="dc:creator" />
        <xsl:with-param name="item_pubDate" select="rss:pubDate" />
        <xsl:with-param name="item_description" select="rss:description" />
        <xsl:with-param name="item_postdate" select="dc:date" />
        <xsl:with-param name="item_updated" select="rss:updated" />
        <xsl:with-param name="item_last" select="last()" />
    </xsl:call-template>
</xsl:template>

<!--
=============================================================================
 RSS 0.90
=============================================================================
-->
<xsl:template match="rss09:item">    
<!-- rss9 item -->
    <xsl:call-template name="drawItem">
        <xsl:with-param name="item_link" select="rss09:link" />
        <xsl:with-param name="item_title" select="rss09:title" />
        <xsl:with-param name="item_author" select="dc:creator" />
        <xsl:with-param name="item_pubDate" select="rss09:pubDate" />
        <xsl:with-param name="item_description" select="rss09:description" />
        <xsl:with-param name="item_postdate" select="dc:date" />
        <xsl:with-param name="item_updated" select="rss09:updated" />
        <xsl:with-param name="item_last" select="last()" />
    </xsl:call-template>
</xsl:template>

<!--
=============================================================================
Atom 1.0
=============================================================================
-->

<xsl:template match="/atom:feed">
	<xsl:if test="number($itemnumber) = 0">
	    <xsl:call-template name="drawHeader">
	        <xsl:with-param name="header_link" select="atom:link/@href" />
	        <xsl:with-param name="header_title" select="atom:title" />
	        <xsl:with-param name="header_image" select="atom:logo" />
	        <xsl:with-param name="header_image_link" select="atom:link/@href" />
	<!-- <xsl:with-param name="header_pubDate" select="pubDate" /> -->
	        <xsl:with-param name="header_description" select="atom:subtitle" />
	<!--         <xsl:with-param name="header_postdate" select="dc:date" /> -->
	        <xsl:with-param name="header_updated" select="atom:updated" />
	    </xsl:call-template>
    </xsl:if>
	<xsl:if test="number($itemnumber) &gt; 0">
    	<xsl:apply-templates select="atom:entry[position() = number($itemnumber)]" />
    </xsl:if>
</xsl:template>

<xsl:template match="atom:entry">
    <xsl:call-template name="drawItem">
        <xsl:with-param name="item_link" select="atom:link/@href" />
        <xsl:with-param name="item_title" select="atom:title" />
        <xsl:with-param name="item_author" select="atom:author/name" />
        <!-- <xsl:with-param name="item_pubDate" select="pubDate" /> -->
        <xsl:with-param name="item_description" select="atom:summary" />
        <!-- <xsl:with-param name="item_postdate" select="dc:date" /> --> 
        <xsl:with-param name="item_updated" select="atom:updated" />
        <xsl:with-param name="item_last" select="last()" />
    </xsl:call-template>
</xsl:template> 

<!--
=============================================================================
Atom 0.3
=============================================================================
--> 
<xsl:template match="/atom3:feed">    
	<xsl:if test="number($itemnumber) = 0">
	    <xsl:call-template name="drawHeader">
	        <xsl:with-param name="header_link" select="atom3:link/@href" />
	        <xsl:with-param name="header_title" select="atom3:title" />
	        <xsl:with-param name="header_image" select="atom3:logo" />
	        <xsl:with-param name="header_image_link" select="atom3:link/@href" />
	<!--    <xsl:with-param name="header_pubDate" select="pubDate" /> -->
	        <xsl:with-param name="header_description" select="atom3:subtitle" />
	<!--         <xsl:with-param name="header_postdate" select="dc:date" /> -->
	        <xsl:with-param name="header_updated" select="atom3:updated" />
	    </xsl:call-template>
    </xsl:if>
	<xsl:if test="number($itemnumber) &gt; 0">
    	<xsl:apply-templates select="atom3:entry[position() = number($itemnumber)]" />    
    </xsl:if>
</xsl:template>
 
<xsl:template match="atom3:entry">  
    <xsl:call-template name="drawItem">
        <xsl:with-param name="item_link" select="atom3:link/@href" />
        <xsl:with-param name="item_title" select="atom3:title" />
        <xsl:with-param name="item_author" select="atom3:author/name" />
        <!-- <xsl:with-param name="item_pubDate" select="pubDate" /> -->
        <xsl:with-param name="item_description" select="atom3:summary" />
        <!-- <xsl:with-param name="item_postdate" select="dc:date" /> --> 
        <xsl:with-param name="item_updated" select="atom3:updated" />
        <xsl:with-param name="item_last" select="last()" />
    </xsl:call-template>
</xsl:template> 

<xsl:template name="drawHeader">
    <xsl:param name="header_link" />
    <xsl:param name="header_title" />
    <xsl:param name="header_image" />
    <xsl:param name="header_image_link" />
    <xsl:param name="header_image_title" />
    <xsl:param name="header_pubDate" />
    <xsl:param name="header_description" />
    <xsl:param name="header_postdate" />
    <xsl:param name="header_updated" />
    <xsl:if test="number($itemnumber) &gt;= 0 and number($itemnumber) &lt; last()">
		<META HTTP-EQUIV="refresh" CONTENT="9;url={$path}&amp;{number($itemnumber)+1}#ENTER_WEBZ_RSS_ENTRY_{number($itemnumber)+1}" />
	</xsl:if>
	
  <div class="rssdoctitle">
    <xsl:if test="$header_image">
  	  <a href="{$header_image_link}"><img src="{$header_image}" alt="{$header_image_title}"/></a>
    </xsl:if>
    <a href="{$header_link}"><xsl:value-of select="$header_title"/></a> <br/>
    <xsl:value-of select="$header_description" disable-output-escaping="yes"/> <br/>
    <xsl:if test="$header_pubDate">
      <xsl:value-of select="$header_pubDate"/> <br/>
    </xsl:if>
    <xsl:if test="$header_postdate">
      <xsl:value-of select="$header_postdate"/> <br/>
    </xsl:if>
    <xsl:if test="$header_updated">
      Updated: <xsl:value-of select="$header_updated"/> <br/>
    </xsl:if>
  </div>
  
    <xsl:if test="number($itemnumber) &gt;= 0 and number($itemnumber) &lt; last()">
		<HR/><a href="{$path}&amp;{number($itemnumber)+1}#ENTER_WEBZ_RSS_ENTRY_{number($itemnumber)+1}">Next &gt;</a>
	</xsl:if>
</xsl:template>

<xsl:template name="drawItem">
    <xsl:param name="item_link" />
    <xsl:param name="item_title" />
    <xsl:param name="item_author" />
    <xsl:param name="item_pubDate" />
    <xsl:param name="item_description" />
    <xsl:param name="item_postdate" />
    <xsl:param name="item_updated" />
    <xsl:param name="item_last" />
    
    <xsl:if test="number($itemnumber) &gt;= 0 and number($itemnumber) &lt; number($item_last)">
		<META HTTP-EQUIV="refresh" CONTENT="15;url={$path}&amp;{number($itemnumber)+1}#ENTER_WEBZ_RSS_ENTRY_{number($itemnumber)+1}" />
	</xsl:if>
	<xsl:if test="number($itemnumber) &gt;= 0 and number($itemnumber) &gt;= number($item_last)">
		<META HTTP-EQUIV="refresh" CONTENT="15;url={$path}&amp;0#ENTER_WEBZ_RSS_ENTRY_0" />
	</xsl:if>
	
    <a href="{$item_link}"><xsl:value-of select="$item_title"/></a> <br/>
    <xsl:if test="$item_author">
      By: <xsl:value-of select="$item_author"/> <br/>
    </xsl:if>
    <xsl:if test="$item_pubDate">
      <div class="rsspubdate"> <xsl:value-of select="$item_pubDate"/> </div> <br/>
    </xsl:if>
    <xsl:value-of select="$item_description" disable-output-escaping="yes"/> <br/>
    <xsl:if test="$item_postdate">
      <div class="rsspubdate">Posted: <xsl:value-of select="$item_postdate"/> </div> <br/>
    </xsl:if>
    <xsl:if test="$item_updated">
      Updated: <xsl:value-of select="$item_updated"/> <br/>
    </xsl:if>
    
    <xsl:if test="number($itemnumber) &gt;= 0 and number($itemnumber) &lt; number($item_last)">
		<HR/><a href="{$path}&amp;{number($itemnumber)+1}#ENTER_WEBZ_RSS_ENTRY_{number($itemnumber)+1}">Next &gt;</a>
	</xsl:if>
	<xsl:if test="number($itemnumber) &gt;= 0 and number($itemnumber) &gt;= number($item_last)">
		<HR/><a href="{$path}&amp;0#ENTER_WEBZ_RSS_ENTRY_0">&lt;&lt; Start</a>
	</xsl:if>
</xsl:template>
</xsl:stylesheet>

