package com.blackledge.david.tivo;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.HmeEvent.Key;

public class MediaPlayerView extends BView {
	
	public static interface MediaStateListener {
		public void mediaStateUpdate(
				int activeState /*INACTIVE, DIRECT_VIDEO, DIRECT_AUDIO, APP*/, 
				int screenState /*FS, PIP, HIDDEN_FS, HIDDEN_PIP*/, 
				int updateState /*STARTED, NEXT, SKIP_SO_NEXT, FAIL_SO_NEXT, ENDED, SKIP_AND_ENDED, FAIL_AND_ENDED*/, 
				boolean overriddingSomeDispatchKeyEvents, //FIXME but always at least handling zoom, slow, and pause.
				String startURL, // possibly a playlist, or even first URL, but more were added later
				String currentURL); // URL directly associated with currently active content
	}
	
	public static interface URLResolver {
		public URL resolveURL(String url) throws MalformedURLException;
		public String getText(String url) throws IOException;
	}

	private static final int default_height = 0;
	private static final int default_width = 0;
	private static final int default_y = 0;
	private static final int default_x = 0;
	private MediaStateListener _listener;
	private URLResolver _resolver;
	private String _startURL;
	private String _currentURL;

	public MediaPlayerView(BView parent) {
		this(parent, null);
	}
	
	public MediaPlayerView(BView parent, MediaStateListener listener) {
		this(parent, listener, null);
	}

	/**
	 * 
	 * @param parent
	 * @param listener
	 * @param resolver if null, uses default which simply converts String to URL, and loads contents by opening URL.
	 */
	public MediaPlayerView(BView parent, MediaStateListener listener, URLResolver resolver) {
		super(parent, default_x, default_y, default_width, default_height);
		_listener = listener;
		if(resolver == null) {
			_resolver = new DefaultURLResolver();
		} else {
			_resolver = resolver;
		}
	}
	
	/**
	 * MediaPlayerView users MUST call this method in their application's
	 * dispatchKeyEvent method override, and only call
	 * super.dispatchKeyEvent(ir); if this method returns false
	 */
	public boolean overrideDispatchKeyEvent(Key ir) {
//		if (this.dispatchOverride) {
//			getResourceParent().postEvent(ir);
//			return true;
//		} else
//			return false;
		return false;
	}
	
	public void setURLResolver(URLResolver resolver) {
		_resolver = resolver;
	}
	
	public boolean startMedia(String startURL) {
		boolean success = true;
		_startURL = startURL;
//		clearPlaylist();
//		success = addMedia(startURL);
		//FIXME allow add to fail.
		success = next();
		return success;
	}
	public boolean startMedia(String startURL, boolean PIP) {
		setPIP(PIP);
		return startMedia(startURL);
	}
	/** doesn't start play */
	public boolean addMedia(String nextURL) {
		boolean success = true;
//		MediaPlugin plugin = getMatchingPlaylistPlugin(nextURL);
//		if(plugin != null) {
//			success = plugin.addToPlaylist(nextURL); // TODO allow failure and return false
//		} else {
//			//FIXME do something
//		}
		return success;
	}
	/** returns false if there is no next item. */
	public boolean next() {
		boolean success = true;
//		_currentURL = getNextURL();
//		// TODO if plugin is different than currently running plugin, shut down previous one before starting next.
//		MediaPlugin plugin = getMatchingPlugin(_currentURL);
//		if(plugin != null) {
//			success = plugin.next(_currentURL);
//		} else {
//			//FIXME do something
//		}
		notifyStateChange();
		return success;
	}
	public void pause() {
		//FIXME do something
	}
	
	/** returns false if current media may not change to that mode */
	public boolean setPIP(boolean pip) {
		if(pip) {
			return setPIP();
		} else {
			return setFullScreen();
		}
	}
	/** returns false if current media may not change to that mode */
	public boolean setPIP() {
		//FIXME do something
		//TODO ask plugin to set itself up for PIP(_isHidden).
//		notifyStateChange();
		return false;
	}
	/** returns false if current media may not change to that mode */
	public boolean setFullScreen() {
		//FIXME do something
		//TODO ask plugin to set itself up for FS(_fullscreenInset, _isHidden).
		
//		notifyStateChange();
		return false;
	}
	private boolean _fullScreenInset = false;
	public void setFullScreenAsInset(boolean inset) {
		_fullScreenInset = inset;
	}
	public boolean isFullScreenAsInset() {
		return _fullScreenInset;
	}
	private boolean _isHidden = false;
	public void hide() {
		_isHidden = true;
		//FIXME do something
		//TODO ask plugin to set itself up for hidden.
		notifyStateChange();
	}
	public void show() {
		_isHidden = false;
		//FIXME do something
		//TODO ask plugin to set itself up for show.
		notifyStateChange();
	}
	public void close() {
		//FIXME do something
		//TODO ask plugin to shut down.
		notifyStateChange();
		_startURL = null;
	}

	boolean _isUserSelectable = true;
	/** whether user menu controls are available to change the current media to a known alternative or hide */
	public void setUserSelectable(boolean selectable) {
		_isUserSelectable = selectable;
	}
	public boolean isUserSelectable() {
		return _isUserSelectable;
	}
	boolean _isUserDisplayControl = true;
	/** whether user may change between PIP/FS (maybe whether to hide/show, too or separate setting, but currently that's in the userSelectable menu)*/
	public void setUserDisplayControl(boolean controlable) {
		_isUserDisplayControl = controlable;
	}
	public boolean isUserDisplayControl() {
		return _isUserDisplayControl;
	}
	
	
	private void notifyStateChange() {
		if(_listener != null) {
			//FIXME do something
//			_listener.mediaStateUpdate(activeState, screenState, updateState, isOverridding, _startURL, _currentURL);
		}
	}

	/**
	 * NOTE: returns null if ALL keys are overridden, returns an empty array if NO keys are overridden.
	 */
	public int[] getOverriddenDispatchKeys() {
		//FIXME do something
		return null;
	}

	
	
	private static class DefaultURLResolver implements URLResolver {
		public URL resolveURL(String url) throws MalformedURLException {
			URL u = new URL(url);
			return u;
		}

		public String getText(String url) throws IOException {
			URL u = resolveURL(url);
			BufferedReader reader = new BufferedReader(new InputStreamReader(u.openStream()));
			StringBuffer result = new StringBuffer();
			for(String line = reader.readLine(); line != null ; line = reader.readLine()) {
				result.append(line).append('\n');
			}
			return result.toString();
		}
	}
}
