package com.blackledge.david.tivo.game;

import java.awt.Color;
import java.awt.Rectangle;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TimeZone;

import com.blackledge.david.game.BoardState;
import com.blackledge.david.game.Chat;
import com.blackledge.david.game.GameException;
import com.blackledge.david.game.MultiplayerGame;
import com.blackledge.david.game.PlayerState;
import com.blackledge.david.game.MultiplayerGame.GameData;
import com.blackledge.david.tivo.V49BApplication;
import com.blackledge.david.tivo.game.Game.GameFactory;
import com.tivo.hme.bananas.BScreen;
import com.tivo.hme.bananas.BView;

public abstract class GameScreen extends BScreen {

		private Object _chatFont = "default-15.font";
		private Color _chatColor = Color.white;
		private BView _chatText;
		protected BView _chatArea;
		private BView _playerArea;
		private HashMap<PlayerState, BView> _playerViews;
		private BView _notYourTurnIndicator;
		
		public GameScreen(Game app) {
			super(app);
			this.getBelow().setResource(getResource("com/blackledge/david/tivo/game/table2.gif"), RSRC_HALIGN_LEFT|RSRC_VALIGN_CENTER|RSRC_IMAGE_VFIT);
			
			Rectangle titleArea = ((Game)app).getTitleRegion();
			Rectangle safeArea = ((Game)app).getSafeActionRegion();
			int chatWidth = titleArea.width*1/3;
			
			// create this (invisible) before chat/player views so it appears below them
			_notYourTurnIndicator = new BView(getAbove(), 0, 0, getWidth(), getHeight(),false);
			_notYourTurnIndicator.setResource(Color.black);
			_notYourTurnIndicator.setTransparency(0.5f);
			
			_chatArea = new BView(getAbove(), titleArea.x + titleArea.width-chatWidth, safeArea.y, chatWidth, safeArea.height);
			_chatArea.setResource(new Color(0,0,50));
			_chatText = new BView(_chatArea, 0, 0, _chatArea.getWidth(), _chatArea.getHeight());
			
			_playerViews = new HashMap<PlayerState, BView>();
		}

		protected GameData getGameData() {
			return ((GameFactory)getBApp().getFactory()).getGame().getGameData(((Game)getBApp()).getGameID());
		}

		/**
		 * Arg is expected to be the game id which will be passed to
		 * {@link MultiplayerGame#joinGame(int, PlayerState)} (if not spectating)
		 */
		@Override
		public boolean handleEnter(Object arg, boolean isReturn) {
			int chatFontSize = (int) (13 * ((Game)getBApp()).getHeightMultiplier());
			_chatFont = "default-" + chatFontSize + ".font";

//			System.out.println("ENTER with "+arg);
			if(arg != null && arg instanceof Integer) {
				Integer gameId = (Integer) arg;
				GameFactory gameFactory = (GameFactory) getBApp().getFactory();
				if (((Game) getBApp()).isSpectating()) {
					((Game)getBApp()).setGameID( gameId );
				} else {
					try {
						((Game)getBApp()).setGameID( gameFactory.getGame().joinGame(gameId, ((Game)getBApp())._player));
					} catch (GameException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				
				// build player area now that we know how many players the game is allowed to have.
				if(_playerArea == null) {
//					Rectangle titleArea = ((Game)getBApp()).getTitleRegion();
					int id = ((Game)getBApp()).getGameID();
//					_playerArea = new BView(getAbove(), titleArea.x, titleArea.y, titleArea.width/2, 20*gameFactory.games.get(id).maximumPlayers);
					int playerAreaHeight = 20*gameFactory.getGame().getGameData(id).maximumPlayers;
					_playerArea = new BView(getAbove(), 
							_chatArea.getX(), _chatArea.getY()+_chatArea.getHeight()-playerAreaHeight, 
							_chatArea.getWidth(), playerAreaHeight);
					_playerArea.setResource(Color.gray);
					// adjust chatarea to allow for playerarea
					_chatArea.setSize(_chatArea.getWidth(), _chatArea.getHeight() - _playerArea.getHeight());
					_chatText.setSize(_chatArea.getWidth(), _chatArea.getHeight());
				}
			}
			
			((Game)getBApp()).startGameUpdates();
			
			return super.handleEnter(arg, isReturn);
		}

		/**
		 * The area available for the game board, excluding the title, chat,
		 * player and any other decorative areas
		 */
		public Rectangle getGameRegion() {
			// using scroll region less space on the right for chat area
			Rectangle scrollRegion = ((Game) getBApp()).getScrollRegion();
			return new Rectangle(scrollRegion.x, scrollRegion.y,
					(_chatArea.getX() - scrollRegion.x),
					scrollRegion.height);
		}
		
		/**
		 * The area available for the game title, excluding the chat and any
		 * other decorative areas.  Excludes {@link Game#getLogoRegion()}
		 */
		protected Rectangle getGameTitleRegion() {
			// using title region less space on the right for chat area
			Rectangle titleRegion = ((Game) getBApp()).getTitleRegion();
			return new Rectangle(titleRegion.x, titleRegion.y,
					titleRegion.width - (_chatArea.getX() - titleRegion.x),
					titleRegion.height);
		}

		@Override
		public boolean handleExit() {
			((Game)getBApp()).stopGameUpdates();

			return super.handleExit();
		}
		
		@Override
		public boolean handleKeyPress(int code, long rawcode) {
			switch(code) {
			case KEY_CLEAR:
				leaveGame();
				return true;
			}
			return super.handleKeyPress(code, rawcode);
		}

		protected void leaveGame() {
			Game game = (Game)getBApp();
			game.stopGameUpdates();
			GameFactory gameFactory = (GameFactory) game.getFactory();
			try {
				gameFactory.getGame().leaveGame(game.getGameID(), game._player);
			} catch (GameException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			((Game)getBApp()).setGameID(-1);
			getBApp().pop();
		}

		protected abstract void updateGameBoard(BoardState gameBoard);

//		private static DateFormat timeformat = SimpleDateFormat.getTimeInstance(SimpleDateFormat.MEDIUM);
		private static DateFormat timeformat = SimpleDateFormat.getTimeInstance(SimpleDateFormat.SHORT);
		protected void updateGameChat(ArrayList<Chat> gameChat) {
			int maxEntries = 50;
			StringBuffer latestChats = getChatText(gameChat, maxEntries);
			
			if (_chatText != null) {
				if (_chatText.getResource() != null) {
					_chatText.getResource().remove();
				}
				_chatText.setResource(
						_chatText.createText(_chatFont, _chatColor,
								latestChats.toString()), 
								RSRC_HALIGN_LEFT | RSRC_VALIGN_BOTTOM | RSRC_TEXT_WRAP);
			}
		}


		private StringBuffer getChatText(ArrayList<Chat> gameChat, int maxEntries) {
			StringBuffer latestChats = new StringBuffer();
			// format chat times based on device's timezone
			TimeZone tz = ((V49BApplication)getBApp()).deviceZone;
			Calendar lastTime = Calendar.getInstance(tz);
			Calendar thisTime = Calendar.getInstance(tz);
			lastTime.setTimeInMillis(0);
			PlayerState lastSpeaker = null;
//			for(int i = gameChat.size()-1 ; i >= 0 && i > gameChat.size()-50 ; --i) {
//				latestChats.insert(0,"\n");
			for(int i = (gameChat.size() > maxEntries?gameChat.size()-maxEntries:0) ; i < gameChat.size() ; ++i) {
				StringBuffer buff = new StringBuffer();
				
				// show time if at least a minute passed since the last shown time
				Date timestamp = gameChat.get(i).getTimestamp();
				thisTime.setTime(timestamp);
				thisTime.add(Calendar.MINUTE, -1);
				boolean showTime = thisTime.after(lastTime);
				if(showTime) {
					buff.append(timeformat.format(timestamp));
					buff.append(" ");
					lastTime.setTime(timestamp);
				}
				
				PlayerState speaker = gameChat.get(i).getSpeaker();
				if(lastSpeaker == null || !lastSpeaker.equals(speaker)) {
					buff.append("[");
					buff.append(speaker);
					buff.append("]");
					buff.append(": ");
					lastSpeaker = speaker;
				}
				
				buff.append(gameChat.get(i).getMessage());
				latestChats.append(buff.toString());
				latestChats.append("\n");
				
//				latestChats.insert(0, gameChat.get(i));
			}
			return latestChats;
		}

		protected void updateGamePlayers(ArrayList<PlayerState> gamePlayers) {
			if(_playerViews != null) {
				Iterator<BView> vals = _playerViews.values().iterator();
				while( vals.hasNext()) {
					BView view = vals.next();
					view.setVisible(false);
				}
				for(int i = 0 ; i < gamePlayers.size(); ++i) {
					PlayerState player = gamePlayers.get(i);
					BView view = _playerViews.get(player);
					// TODO extract old key to compare against new one to decide if update needed, and if so replace key
					if(view == null) {
						view = new BView(_playerArea, 0, 0, _playerArea.getWidth(), 20);
						new BView(view, 0, 0, view.getWidth(), view.getHeight());
						_playerViews.put(player, view);
					}
					view.setVisible(true);
					view.setLocation(0, 20*i, getResource("*500"));
					view.setResource(player.color);
					BView txt = (BView) view.getChild(0);
					if(txt.getResource() != null) {
						txt.getResource().remove();
					}
					String playertext = player.name+" ("+player.score+")";
					if(player.equals(((Game)getBApp())._player)) {
						// update player details
						((Game)getBApp())._player = new PlayerState(player);
						
						if(player.hasTurn) {
							_notYourTurnIndicator.setVisible(false);
							txt.setResource(txt.createText("default-18-bold.font", Color.white, playertext), RSRC_HALIGN_RIGHT|RSRC_VALIGN_TOP);
						} else {
							_notYourTurnIndicator.setVisible(true);
							txt.setResource(txt.createText("default-18.font", Color.white, playertext), RSRC_HALIGN_RIGHT|RSRC_VALIGN_TOP);
						}
					} else {
						if(player.hasTurn) {
							txt.setResource(txt.createText("default-18-bold.font", Color.white, playertext), RSRC_HALIGN_RIGHT|RSRC_VALIGN_TOP);
						} else {
							txt.setResource(txt.createText("default-18.font", Color.white, playertext), RSRC_HALIGN_RIGHT|RSRC_VALIGN_TOP);
						}
					}
				}
			}
		
		}
	}