package com.blackledge.david.tivo.game.cribbage;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;

import com.blackledge.david.game.BoardState;
import com.blackledge.david.game.Card;
import com.blackledge.david.game.CardStack;
import com.blackledge.david.game.PlayerState;

import com.blackledge.david.game.cribbage.CribbageGame;
import com.blackledge.david.game.cribbage.CribbageGame.ACTION;
import com.blackledge.david.game.cribbage.CribbageGame.CribbageCard;
import com.blackledge.david.game.cribbage.CribbageGame.CribbageGameData;
import com.blackledge.david.game.cribbage.CribbageGame.GAMESTATE;

import com.tivo.hme.bananas.BApplication;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.Resource;

import com.blackledge.david.tivo.RemotePicker;
import com.blackledge.david.tivo.TiVoRemoteInfo;
import com.blackledge.david.tivo.V49BApplication;

import com.blackledge.david.tivo.game.CardView;
import com.blackledge.david.tivo.game.Game;
import com.blackledge.david.tivo.game.GameScreen;
import com.blackledge.david.tivo.game.cribbage.Cribbage.CribbageFactory;

public class CribbageGameScreen extends GameScreen {

	private BView _helpView;
	private BView _gameArea;
	private CardView[] _cribView;
	private CardView[] _playView;
	private CardView[] _myHandView;
	private CardView[] _hisHandView;
	private BView _notice;
	private BView _pegArea;
	private BView[][] _peg = new BView[2][2];
	private int[] _pegIndex = new int[2];
	private int _focusIndex = -1;
	private BView _focusView;

	public CribbageGameScreen(Game app) {
		super(app);
		
		_cribView = new CardView[4];
		_playView = new CardView[9];
		_myHandView = new CardView[6];
		_hisHandView = new CardView[6];
		
		Rectangle gameArea = getGameRegion();
		
		_gameArea = new BView(getNormal(), gameArea.x, gameArea.y, gameArea.width, gameArea.height);

		// game area width less 9 overlapping cards
		int cardwidth = (_gameArea.getWidth() / 11);
		int pegboardwidth = (int) (_gameArea.getWidth() - ((cardwidth - cardwidth / 3.5) * 9 + cardwidth / 3.5));
		// focusview is first child, so it will be below the cards.
		_focusView = new BView(_gameArea, 0, 0, cardwidth+4, (cardwidth*35/25)+4, false);
		_focusView.setResource(Color.yellow);
		
		_pegArea = new BView(_gameArea, _gameArea.getWidth()-pegboardwidth, 0, pegboardwidth, _gameArea.getHeight());
//		_pegArea.setResource(Color.orange.darker());
		_pegArea.setResource(getResource("com/blackledge/david/tivo/game/cribbage/board.gif"), RSRC_HALIGN_CENTER|RSRC_VALIGN_TOP|RSRC_IMAGE_VFIT);
		int w = 7;
		int h = 7;
		// display peg slots
		Dimension pegDim = new Dimension(w,h);
		for(int pts = 1 ; pts <= 121-4 ; pts += 5) {
			Point loc1 = getPegLocation(pegDim, pts);
			Point loc2 = getPegLocation(pegDim, pts+4);
			Point loc3 = getPegLocation(pegDim, pts+5);
			if(loc2.x != loc3.x){
				// add a path guide for this "turn"
				BView turn = new BView(_pegArea, loc2.x + 15 + 2, loc2.y,
						loc3.x + 2 - (loc2.x + 15 + 2), loc2.y + h - loc2.y);
				turn.setResource(new Color(0x95,0x45,0x35)); // Chestnut brown
			}
			if(loc1.y > loc2.y){
				Point loc = loc1;
				loc1 = loc2;
				loc2 = loc;
			}
			BView set = new BView(_pegArea, loc1.x + 2, loc1.y, w - 4, loc2.y - loc1.y + h);
			set.setResource(Color.DARK_GRAY);
			set = new BView(_pegArea, loc1.x + 15 + 2, loc1.y, w - 4, loc2.y - loc1.y + h);
			set.setResource(Color.DARK_GRAY);
		}
		Point loc1 = getPegLocation(pegDim, 0);
		Point loc2 = getPegLocation(pegDim, 121);
		BView set = new BView(_pegArea, loc1.x + 2, loc1.y + 2, w - 4, h - 4);
		set.setResource(Color.DARK_GRAY);
		set = new BView(_pegArea, loc1.x + 15 + 2, loc1.y + 2, w - 4, h - 4);
		set.setResource(Color.DARK_GRAY);
		set = new BView(_pegArea, loc2.x + 2, loc2.y + 2, w - 4, h - 4);
		set.setResource(Color.DARK_GRAY);
		set = new BView(_pegArea, loc2.x + 15 + 2, loc2.y + 2, w - 4, h - 4);
		set.setResource(Color.DARK_GRAY);
		

		_peg[0][0] = new BView(_pegArea, 0, _pegArea.getHeight(), w, h);
		_peg[0][1] = new BView(_pegArea, 0, _pegArea.getHeight(), w, h);
		_peg[1][0] = new BView(_pegArea, 15, _pegArea.getHeight(), w, h);
		_peg[1][1] = new BView(_pegArea, 15, _pegArea.getHeight(), w, h);
		
		_notice = new BView(getAbove(), gameArea.x, gameArea.y-35
				/*+gameArea.height/2-70*/, gameArea.width, 70, false);
		new BView(_notice, 0, 0, _notice.getWidth(), _notice.getHeight());
		_notice.setResource(new Color(0, 0, 50));
		_notice.setTransparency(0.10f);
		
		
		Rectangle titleArea = getGameTitleRegion();
		BView title = new BView(getNormal(), titleArea.x, titleArea.y, titleArea.width, titleArea.height);
		title.setResource(title.createText("default-30.font", Color.cyan, "Cribbage"), RSRC_VALIGN_TOP|RSRC_HALIGN_CENTER);
		
		BView hint = new BView(getNormal(), titleArea.x, titleArea.y+titleArea.height/2, titleArea.width, titleArea.height/2);
		TiVoRemoteInfo remote = RemotePicker.getRemoteForTiVo(((V49BApplication)app).tivoID);
		RemotePicker.addHint(remote, hint, new int[] {KEY_CLEAR}, "Quit Match\n", Color.yellow, true);
		RemotePicker.addHint(remote, hint, new int[] {KEY_INFO}, "Help/Rules\n", Color.yellow, true);
		RemotePicker.addHint(remote, hint, new int[] {KEY_NUM1, KEY_NUM6}, "Deal/Play\n/Discard\n", Color.yellow, true);
		
		
		_helpView = new BView(getAbove(), gameArea.x, gameArea.y, gameArea.width, gameArea.height, false);
		_helpView.setResource(Color.DARK_GRAY);
		_helpView.setTransparency(0.10f);
		BView text = new BView(_helpView, 0, 0, _helpView.getWidth(), _helpView.getHeight());
		int helpFontSize = (int) (15 * ((Game)getBApp()).getHeightMultiplier());
		text.setResource(text.createText("default-"+helpFontSize+"-italic.font", Color.white, "" +
				"Cribbage Rules Help\n" +
				"\n" +
				"Discard 2 cards to the dealer's extra hand (the crib) then the dealer starts with the turn-up\n" +
				"\n" +
				"Play to no more than 31 (face cards are 10, Ace is 1), pegging points when your played card makes an of-a-kind, run, 15, or 31.\n" +
//				"\n" +
				"Repeat until all cards are played, then peg points for flushes, of-a-kinds, runs, and 15s in any combination using your hand and the turn-up card.\n" +
				"\n" +
				"Extra points when the other player has to say 'Go', you turn-up a Jack as dealer, or your hand contains the Jack that goes with the turn-up.\n" +
				"\n" +
				"Change dealers and repeat.  First to peg 121 points wins!" +
				""), RSRC_VALIGN_TOP|RSRC_HALIGN_CENTER|RSRC_TEXT_WRAP);
		
	}
	
	@Override
	public boolean handleEnter(Object arg, boolean isReturn) {
		boolean result = super.handleEnter(arg, isReturn);
//		getBelow().setResource(V49BApplication.STREAM_LOOP_BLUE_SETUP);
//		getBelow().setResource(Color.blue.darker().darker());
		return result;
	}
	
	@Override
	public boolean handleExit() {
		if (getBelow().getResource() != null) {
			getBelow().getResource().remove();
			getBelow().clearResource();
		}
		return super.handleExit();
	}

	@Override
	protected void updateGameBoard(BoardState gameBoard) {

		CribbageFactory mainGame = (CribbageFactory) getBApp().getFactory();
		Cribbage app = (Cribbage) getBApp();
		CribbageGameData gameData = (CribbageGameData) mainGame.getGame().getGameData(app.getGameID());
		
		PlayerState player;
		if (app.isSpectating()) {
			player = gameData.players.get(0);
		} else {
			player = app.getPlayer();
		}
		int playerIndex = gameData.players.indexOf(player);		
		
		CardStack crib = mainGame.getCribCards( app.getGameID());
		CardStack play = mainGame.getPlayCards( app.getGameID());
		CardStack mine = mainGame.getPlayerCards(app.getGameID(), player);
		CardStack his = null;
		if(gameData.players.size() > 1) {
			his = mainGame.getPlayerCards( app.getGameID(), gameData.players.get(1-playerIndex));
		}

		boolean iAmDealer = (gameData.dealer == playerIndex);

		int cardwidth = _gameArea.getWidth() / 11;
		int cardheight = CardView.getHeight(cardwidth);

		int criby = iAmDealer ? 3 * (cardheight+2) : (cardheight+2);
		layoutCards(crib, cardwidth, cardwidth, criby, _cribView, gameData.gameState == GAMESTATE.ROUND_BEGIN, false, null, null, gameData.countOffset);

		int playy = (int) ((iAmDealer?1.5:2.5)*(cardheight+2));
		layoutCards(play, 0, cardwidth, playy, _playView, true, false, his, mine, gameData.countOffset);
		
		int hisy = 0;
		layoutCards(his, 0, cardwidth, hisy, _hisHandView, false, true, null, null, gameData.countOffset);
		
		int myy = 4 * (cardheight+2);
		layoutCards(mine, 0, cardwidth, myy, _myHandView, !app.isSpectating(), true, null, null, gameData.countOffset);
		
		if(app.isSpectating()) {
			// everything after this is interactive-specific
			return;
		}
		
		String output = "";
//		if(iAmDealer) {
//			output += "Pone: ";
//		} else {
//			output += "Dealer: ";
//		}
////		output += his;
//		if (his != null) {
//			for (Object card : his) {
//				output += "# ";
//			}
//		}
//		output += "\n\n";
//		
//		if(iAmDealer) {
//		} else {
//			output += "CRIB: ";
////			output += crib;
//			if(crib != null) {
//				for(Object card : crib) {
//					output += "# ";
//				}
//			}
//			output += "\n\n";
//		}
//
//		output += "PLAYED: ";
//		output += play;
//		output += "\n\n";
//		
//		if(iAmDealer) {
//			output += "CRIB: ";
////			output += crib;
//			if(crib != null) {
//				for(Object card : crib) {
//					output += "# ";
//				}
//			}
//			output += "\n\n";
//		}
//		
		if (player.hasTurn) {
			switch (gameData.gameState) {
////			case GAME_WAITING:
////				output += "Waiting for another player.";
////				output += "\n";
////				break;
			case GO_NEXT:
				output += "You can't play, Press a button to say 'Go'";
				output += "\n";
				getBApp().play(V49BApplication.SOUND_SLOWDOWN1);
				break;
//			case GO_GOING:
//				output += "The other player can't play, it will stay your turn.";
//				output += "\n";
//				break;
			}
		}
		if(iAmDealer) {
			switch(gameData.gameState) {
			case ROUND_BEGIN:
				output += "Press a button to deal the next hands.";
				output += "\n";
				break;
			case DISCARDS:
				output += "After discards, you will turn up the first card.";
				output += "\n";
				break;
			}
//			output += "Dealer: ";
		} else {
			switch(gameData.gameState) {
			case ROUND_BEGIN:
				output += "The dealer will begin the next round.";
				output += "\n";
				break;
			case DISCARDS:
				output += "After discards, your turn will begin.";
				output += "\n";
				break;
			}
//			output += "Pone: ";
		}
//		output += mine;
//		output += "\n\n";
//
//		if(_gameArea.getResource() != null) {
//			_gameArea.getResource().remove();
//			_gameArea.clearResource();
//		}
//		_gameArea.setResource(getNormal().createText("default-20.font", Color.white, output), RSRC_HALIGN_CENTER|RSRC_VALIGN_TOP);
		
		if(output.trim().length() > 0) {
			_notice.getChild(0).setResource(_notice.createText("default-25.font", Color.yellow, output),
					RSRC_VALIGN_CENTER|RSRC_HALIGN_CENTER|RSRC_TEXT_WRAP);
			_notice.setVisible(true);
		} else {
			if (_notice.getChild(0).getResource() != null) {
				_notice.getChild(0).getResource().remove();
				_notice.getChild(0).clearResource();
			}
			_notice.setVisible(false);
		}
		
		focusOn(_focusIndex);
	}

	private void layoutCards(CardStack mine, int xadj, int cardwidth, int myy,
			CardView[] handViews, boolean faceup, boolean hidefacedowns, CardStack upHand, CardStack downHand, int countOffset) {
		if (mine != null) {
			int count = 0;
			for (int i = 0; i < mine.size() && i < handViews.length; ++i) {
				Card card1 = (Card) mine.get(i);
				// make a copy
				CribbageCard card = new CribbageCard(card1.getSuit(), card1.getValue(),
						card1.isFaceUp());
				
				boolean hide = (hidefacedowns && !card.isFaceUp());
				
				card.setFaceUp(faceup);
				
				int yadj = 0;
				Point origLoc = null;
				if(upHand != null && upHand.contains(card)) {
					yadj = -CardView.getHeight(cardwidth)/3;
					CardView cardView = _hisHandView[upHand.indexOf(card)];
					if(cardView != null)
						origLoc = cardView.getLocation();
				} else if (downHand != null && downHand.contains(card)) {
					yadj = CardView.getHeight(cardwidth)/3;
					CardView cardView = _myHandView[downHand.indexOf(card)];
					if(cardView != null)
						origLoc = cardView.getLocation();
				}
				int x = xadj + i * (cardwidth + (origLoc == null ? 5 : -(int)(cardwidth/3.5)));
				int y = myy + yadj;
				if(origLoc == null) {
					// start/end in the same place.
					origLoc = new Point(x,y);
				}
				//FIXME animate based on origLoc...but only when it actually moved!
				boolean animate = false;
				if (handViews[i] == null) {
					animate = true;
					handViews[i] = new CardView(_gameArea,
							origLoc.x, origLoc.y, cardwidth, true, null);
				} else {
					animate = handViews[i].getVisible() != !hide;
				}
				if(animate) {
					handViews[i].setLocation(origLoc.x, origLoc.y);
					handViews[i].setLocation(x, y, getResource("*300"));
				} else {
					handViews[i].setLocation(x, y);
				}
				handViews[i].setState(card);
				handViews[i].setVisible(!hide);
				
				if(upHand != null && downHand != null) {
					count += card.getCribbageCountValue();
					Resource anim = getResource("*1000,-0.5");
					if(count <= countOffset) {
						handViews[i].setTransparency(0.5f, anim);
					} else {
						handViews[i].setTransparency(0.0f, anim);
					}
				}
				
			}
		}
		for(int i = mine==null?0:mine.size(); i < handViews.length; ++i) {
			if(handViews[i] != null)
				handViews[i].setVisible(false);
		}
	}
	
	@Override
	public boolean handleKeyPress(int code, long rawcode) {
		CribbageFactory mainGame = (CribbageFactory) getBApp().getFactory();
		Cribbage app = (Cribbage) getBApp();
		if(app.isSpectating()) {
			switch(code) {
			case KEY_LEFT:
				leaveGame();
				return true;
			case KEY_INFO:
				_helpView.setVisible(!_helpView.getVisible());
				return true;
			}
			return super.handleKeyPress(code, rawcode);
		}
		CribbageGameData gameData = (CribbageGameData) mainGame.getGame().getGameData(app.getGameID());
		// get the game instance's player instance
		PlayerState player = gameData.players.get(gameData.players.indexOf(app.getPlayer()));
//		PlayerState player = app.getPlayer();
		
		ACTION action;
		action = CribbageGame.ACTION.PLAY;
		switch(gameData.gameState) {
		case DISCARDS:
			action = CribbageGame.ACTION.DISCARD;
			break;
		case ROUND_BEGIN:
			action = CribbageGame.ACTION.DEAL;
			break;
		case GO_NEXT:
			action = CribbageGame.ACTION.GO;
			break;
		}
		switch(code) {
		case KEY_LEFT:
			getBApp().play(V49BApplication.SOUND_UPDOWN);
			return focusOn(_focusIndex - 1);
		case KEY_RIGHT:
			getBApp().play(V49BApplication.SOUND_UPDOWN);
			return focusOn(_focusIndex + 1);
		case KEY_SELECT:
		case KEY_PLAY:
			getBApp().play(V49BApplication.SOUND_UPDOWN);
			return actOn(mainGame, app, player, action, _focusIndex);
		case KEY_INFO:
			_helpView.setVisible(!_helpView.getVisible());
			return true;
		case KEY_NUM0:
			return actOn(mainGame, app, player, CribbageGame.ACTION.GO, 0);
//			mainGame.playerAction(app.getGameID(), player, CribbageGame.ACTION.GO.ordinal(), new Integer(0));
//			return true;
		case KEY_NUM1:
			return actOn(mainGame, app, player, action, 0);
		case KEY_NUM2:
			return actOn(mainGame, app, player, action, 1);
		case KEY_NUM3:
			return actOn(mainGame, app, player, action, 2);
		case KEY_NUM4:
			return actOn(mainGame, app, player, action, 3);
		case KEY_NUM5:
			return actOn(mainGame, app, player, action, 4);
		case KEY_NUM6:
			return actOn(mainGame, app, player, action, 5);
		}
		
		return super.handleKeyPress(code, rawcode);
	}

	private boolean actOn(CribbageFactory mainGame, Cribbage app,
			PlayerState player, ACTION action, int index) {
		// adjust focus if needed (just to be safe)
		focusOn(index);
		boolean result = mainGame.playerAction(app.getGameID(), player, action.ordinal(), new Integer(_focusIndex));
		// adjust focus if needed (not unlikely)
		focusOn(_focusIndex);
		return result;
	}
	
	private boolean focusOn(int i) {
		Cribbage app = (Cribbage) getBApp();
		CribbageFactory mainGame = (CribbageFactory) app.getFactory();
		CardStack mine = mainGame.getPlayerCards( app.getGameID(), app.getPlayer());
		if(i >= mine.size()) {
			i = mine.size()-1;
		} else if(i < 0) {
			i = 0;
		}
		
		boolean result = (_focusIndex != i || _focusIndex < 0);
		
		_focusIndex = i;
		updateFocus();
		
		return result;
	}

	private void updateFocus() {
		if(_focusIndex >= 0 && _focusIndex < _myHandView.length && _myHandView[_focusIndex] != null) {
			Resource anim = getResource("*150,0.25");
			_focusView.setVisible(true, anim);
			_focusView.setLocation(_myHandView[_focusIndex].getX()-2, _myHandView[_focusIndex].getY()-2, anim);
		}
	}

	@Override
	protected void updateGamePlayers(ArrayList<PlayerState> gamePlayers) {
		
		// update scoreboard pegs (jump back peg into new score if it changed)
		
		super.updateGamePlayers(gamePlayers);
		
		for(int p = 0 ; p < gamePlayers.size() ; ++p) {
			PlayerState player = gamePlayers.get(p);
			if(_peg == null || _peg[p]==null || _pegIndex == null || _peg[p][_pegIndex[p]] == null)
				continue;
			BView peg = _peg[p][_pegIndex[p]];
			peg.setResource(player.color);
			
			long score = player.score;
			
			int xoff = p*15;
			Point loc = getPegLocation(peg.getSize(), score);
			loc.x += xoff;
			
			// if neither peg is in that location, move this peg.
			if (peg.getY() != loc.y || peg.getX() != loc.x) {
				BView peg2 = _peg[p][1 - _pegIndex[p]];
				if (peg2.getY() != loc.y || peg2.getX() != loc.x) {
					// and play a "score!" sound
					playScoreSound(player);
					
					peg.setLocation(loc.x, loc.y, getResource("*1000"));
					_pegIndex[p] = 1 - _pegIndex[p];
				}
			}
		}
	}

	private void playScoreSound(PlayerState player) {
		boolean gameOver = ((Cribbage)getBApp()).isGameWon();
		if(gameOver) {
			if(((Game)getBApp()).getPlayer().score >= 121) {
				// you won!
				getBApp().play(V49BApplication.SOUND_TIVO);
			} else {
				// he won.
				getBApp().play(V49BApplication.SOUND_BONK);
			}
		} else {
			if(player.equals(((Game)getBApp()).getPlayer())) {
				// you got points!
				getBApp().play(V49BApplication.SOUND_THUMBSUP);
			} else {
				// he got points.
				getBApp().play(V49BApplication.SOUND_THUMBSDOWN);
			}
		}
	}

	private Point getPegLocation(Dimension dimension, long score) {
		if(score > 121) score = 121;
		int coloff = _pegArea.getWidth()/4;
		int areaHeight = _pegArea.getHeight();
//		int areaHeight = (dimension.height+2)*33;
		int gap = (int) ((areaHeight - dimension.getHeight()*31)/30);
		
		int x = 0;
		int y = 0;
		if (score > 30 && score <= 60) {
			y = (int) ((score - 30) * (dimension.getHeight() + gap));
			x = (1 * coloff)+coloff/4;
		} else if (score > 90) {
			y = (int) ((score - 90) * (dimension.getHeight() + gap));
			x = (3 * coloff)+coloff/4;
		} else {
			if (score <= 30) {
				y = (int) (areaHeight - 
						((score + 1) * (dimension.getHeight() + gap)));
				x = (0 * coloff)+coloff/4;
			} else {
				y = (int) (areaHeight - 
						((score-60 + 1) * (dimension.getHeight() + gap)));
				x = (2 * coloff)+coloff/4;
			}
		}
		Point loc = new Point(x, y);
		return loc;
	}

}
