package com.blackledge.david.tivo.game.minesweeper;

import java.awt.Color;
import java.awt.Rectangle;

import com.blackledge.david.tivo.RemotePicker;
import com.blackledge.david.tivo.TiVoModelInfo;
import com.blackledge.david.tivo.TiVoRemoteInfo;
import com.blackledge.david.tivo.V49BApplication;
import com.blackledge.david.tivo.V49BApplicationFactory;

import com.tivo.hme.bananas.BApplication;
import com.tivo.hme.bananas.BButton;
import com.tivo.hme.bananas.BEvent;
import com.tivo.hme.bananas.BScreen;
import com.tivo.hme.bananas.BView;

public class MineSweeper extends V49BApplication {

	
	public static class MineSweeperFactory extends V49BApplicationFactory {
	}
	/** the title presented to the TiVo */
	public static final String TITLE = "Minesweeper"; //$NON-NLS-1$
	/** the URI path used by the TiVo to access this application */
	public static final String URI = "minesweeper"; //$NON-NLS-1$
	
//	private static final String MINE = "http://enterwebz.tv/fatcow/FatCow_Icons32x32/bomb.png"; // grenade.png
//	private static final String FLAG = "http://enterwebz.tv/fatcow/FatCow_Icons32x32/radioactivity.png";// pirate_flag.png warning.png
//	private static final String HIDDEN = "http://enterwebz.tv/fatcow/FatCow_Icons32x32/emotion_question.png";
	private static final String MINE = "bomb.png"; // grenade.png
	private static final String FLAG = "radioactivity.png";// pirate_flag.png warning.png
	private static final String HIDDEN = "emotion_question.png";
	
	public static class MineSweeperScreen extends BScreen {

		private int _gameheight = 10;
		private int _gamewidth = 20;
		private GAMECELL _game[][] = new GAMECELL[_gamewidth][_gameheight];
		private BButton[][] _views = new BButton[_gamewidth][_gameheight];
		private boolean _gameover = false;
		private BView _lastFocus;
		private BView _focusView;
		
		private enum GAMECELL {MINE_HIDDEN, MINE_SHOWN, HIDDEN, FLAGGED, MINE_FLAGGED, SHOWN};
		
		public MineSweeperScreen(BApplication app) {
			super(app);
			buildNewGame();
		}
		
		private void buildGameViews() {

			clearGameOver();

			MineSweeper app = (MineSweeper) getBApp();
			boolean isHD = app.getCurrentResolution().getHeight() > 480;
			BView parent = getNormal();
			
			Rectangle titleRegion = app.getTitleRegion();
			BView infoArea = new BView(parent, titleRegion.x, titleRegion.y, titleRegion.width, titleRegion.height);

			String instructionFont = "default-12.font";
			if(isHD)
				instructionFont = "default-18.font";
			int fontH = isHD?18:12;
			int textW = isHD?65:50;
			int hintAreaWidth = textW*4;
			
			BView titleTextArea = new BView(infoArea, 0, 0, infoArea.getWidth(), 20);
			titleTextArea.setResource(createText("default-30-italic.font",Color.yellow, "MineSweeper!"));
			BView instructionArea = new BView(infoArea, 0, titleTextArea.getHeight(), infoArea.getWidth()-hintAreaWidth, infoArea.getHeight()-titleTextArea.getHeight());
			instructionArea.setResource(createText(instructionFont,Color.white, 
					"Check a spot.  If it shows a number, there are that many mines in the 8 spots around it.  " +
					"Flag spots you think are mines so you don't accidentally check them.  " +
					"Check the area around a spot if you know all mines are flagged around it.  " +
					"You win when all safe spots are checked."), RSRC_HALIGN_CENTER|RSRC_VALIGN_TOP|RSRC_TEXT_WRAP);
			
			BView hintArea = new BView(infoArea, instructionArea.getWidth(), instructionArea.getY(), hintAreaWidth, instructionArea.getHeight());
			TiVoRemoteInfo remote = RemotePicker.getRemoteForTiVo(app.tivoID);
			RemotePicker.addHint(remote, hintArea, new int[]{KEY_SELECT}, "Check", instructionFont, fontH, Color.white, textW, true);
			RemotePicker.addHint(remote, hintArea, new int[]{KEY_THUMBSUP, KEY_RECORD, KEY_PAUSE}, "Flag Mine", instructionFont, fontH, Color.white, textW, true);
			RemotePicker.addHint(remote, hintArea, new int[]{KEY_PLAY, KEY_ENTER}, "Check Area", instructionFont, fontH, Color.white, textW, true);
			RemotePicker.addHint(remote, hintArea, new int[]{KEY_CLEAR}, "Quit", instructionFont, fontH, Color.white, textW, true);
			
			Rectangle gameArea = app.getScrollRegion();
			
			int cellWidth = gameArea.width / _gamewidth;
			int cellHeight = gameArea.height / _gameheight;
			
			_focusView = new BView(getBelow(), 0, 0, cellWidth + 2, cellHeight + 2);
			_focusView.setResource(Color.lightGray);
			
			for(int y = 0 ; y < _gameheight; ++y) {
				for(int x = 0; x < _gamewidth ; ++x) {
					_views[x][y] = new BButton(parent, gameArea.x+ x*cellWidth, gameArea.y+ y*cellHeight, cellWidth-2, cellHeight-2);
//					_views[x][y].setBarAndArrows(
//							(x > 0 ? "left" : null),
//							(x < _gamewidth - 1 ? "right" : null), 
//							(y > 0 ? "up" : null),
//							(y < _gameheight - 1 ? "down" : null), 
//							true);
				}
			}
			
			setFocusDefault(_views[0][0]);
		}
		
		@Override
		public boolean handleKeyRepeat(int code, long rawcode) {
			switch(code) {
			case KEY_LEFT:
			case KEY_RIGHT:
			case KEY_UP:
			case KEY_DOWN:
				return handleKeyPress(code, rawcode);
			}
			return super.handleKeyRepeat(code, rawcode);
		}
		
		@Override
		public boolean handleKeyPress(int code, long rawcode) {
			if(_gameover) {
				clearGameOver();
				buildNewGame();
				setUpGame();
			}
			
			switch(code) {
			case KEY_NUM1:
				postEvent(new BEvent.Action(getFocus(),"up"));
				postEvent(new BEvent.Action(getFocus(),"left"));
				break;
			case KEY_NUM3:
				postEvent(new BEvent.Action(getFocus(),"up"));
				postEvent(new BEvent.Action(getFocus(),"right"));
				break;
			case KEY_NUM9:
				postEvent(new BEvent.Action(getFocus(),"down"));
				postEvent(new BEvent.Action(getFocus(),"right"));
				break;
			case KEY_NUM7:
				postEvent(new BEvent.Action(getFocus(),"down"));
				postEvent(new BEvent.Action(getFocus(),"left"));
				break;
			case KEY_UP:
			case KEY_NUM2:
				postEvent(new BEvent.Action(getFocus(),"up"));
				break;
			case KEY_DOWN:
			case KEY_NUM8:
				postEvent(new BEvent.Action(getFocus(),"down"));
				break;
			case KEY_LEFT:
			case KEY_NUM4:
				postEvent(new BEvent.Action(getFocus(),"left"));
				break;
			case KEY_RIGHT:
			case KEY_NUM6:
				postEvent(new BEvent.Action(getFocus(),"right"));
				break;
			case KEY_SELECT:
			case KEY_NUM5:
				if(getFocus() != null) {
					selected(getFocus());
				}
				return true;
			case KEY_PLAY:
			case KEY_ENTER:
//			case KEY_OPT_ACTION_A:
				if(getFocus() != null) {
					selectArea(getFocus());
				}
				return true;
			case KEY_THUMBSUP:
			case KEY_THUMBSDOWN:
			case KEY_RECORD:
			case KEY_PAUSE:
//			case KEY_OPT_ACTION_D:
			case KEY_NUM0:
				if(getFocus() != null) {
					flag(getFocus());
				}
				return true;
			case KEY_CLEAR:
//			case KEY_PAUSE:
				getBApp().setActive(false);
				return true;
			}
			return super.handleKeyPress(code, rawcode);
		}
		
		@Override
		public boolean handleAction(BView view, Object action) {
			boolean result = super.handleAction(view, action);
			
			updateFocusHighlight();
			
			return result;
		}

		private void updateFocusHighlight() {
			if(getFocus() != _lastFocus) {
				if(_lastFocus != null) {
				}
				if(getFocus() != null) {
					BView focus = getFocus();
//					MineSweeper app = (MineSweeper) getBApp();
//					Rectangle gameArea = app.getScrollRegion();
//					int cellWidth = gameArea.width / _gamewidth;
//					int cellHeight = gameArea.height / _gameheight;
//					int x = (focus.getX()-gameArea.x)/cellWidth;
//					int y = (focus.getY()-gameArea.y)/cellHeight;
					
					_focusView.setLocation(focus.getX()-1, focus.getY()-1, getResource("*200"));
				}
				_lastFocus = getFocus();
			}
		}

		private void selected(BView focus) {
//			_views[x][y] = new BButton(parent, x*cellWidth, y*cellHeight, cellWidth-2, cellHeight-2);
			MineSweeper app = (MineSweeper) getBApp();
			Rectangle gameArea = app.getScrollRegion();
			int cellWidth = gameArea.width / _gamewidth;
			int cellHeight = gameArea.height / _gameheight;
			int x = (focus.getX()-gameArea.x)/cellWidth;
			int y = (focus.getY()-gameArea.y)/cellHeight;
			
			selected(x, y, true);
			
			updateViews();
		}

		private void selected(int x, int y, boolean includeFlagged) {
			if(x >= 0 && x < _gamewidth && y >= 0 && y < _gameheight) {
				switch(_game[x][y]) {
				case FLAGGED:
					if (includeFlagged) {
						getBApp().play(SOUND_DESELECT);
						_game[x][y] = GAMECELL.HIDDEN;
					}
					break;
				case MINE_FLAGGED:
					if (includeFlagged) {
						getBApp().play(SOUND_DESELECT);
						_game[x][y] = GAMECELL.MINE_HIDDEN;
					}
					break;
				case HIDDEN:
					getBApp().play(SOUND_UPDOWN);
					clearNearby(x,y);
					_game[x][y] = GAMECELL.SHOWN;
					break;
				case MINE_HIDDEN:
					_game[x][y] = GAMECELL.MINE_SHOWN;
					_gameover = true;
					break;
				case MINE_SHOWN:
				case SHOWN:
					getBApp().play(SOUND_UPDOWN);
					break;
				}
			}
		}
		
		private void flag(BView focus) {
//			_views[x][y] = new BButton(parent, x*cellWidth, y*cellHeight, cellWidth-2, cellHeight-2);
			MineSweeper app = (MineSweeper) getBApp();
			Rectangle gameArea = app.getScrollRegion();
			int cellWidth = gameArea.width / _gamewidth;
			int cellHeight = gameArea.height / _gameheight;
			int x = (focus.getX()-gameArea.x)/cellWidth;
			int y = (focus.getY()-gameArea.y)/cellHeight;
			
			if(x >= 0 && x < _gamewidth && y >= 0 && y < _gameheight) {
				switch(_game[x][y]) {
				case FLAGGED:
					getBApp().play(SOUND_DESELECT);
					_game[x][y] = GAMECELL.HIDDEN;
					break;
				case MINE_FLAGGED:
					getBApp().play(SOUND_DESELECT);
					_game[x][y] = GAMECELL.MINE_HIDDEN;
					break;
				case HIDDEN:
					getBApp().play(SOUND_SELECT);
					_game[x][y] = GAMECELL.FLAGGED;
					break;
				case MINE_HIDDEN:
					getBApp().play(SOUND_SELECT);
					_game[x][y] = GAMECELL.MINE_FLAGGED;
					break;
				case MINE_SHOWN:
				case SHOWN:
//					getBApp().play(SOUND_BONK);
					break;
				}
			}
			
			updateViews();
		}

		private void buildNewGame() {
			double minepercent = 0.15;
			for(int y = 0 ; y < _gameheight; ++y) {
				for(int x = 0; x < _gamewidth ; ++x) {
					if(Math.random() <= minepercent) {
						_game[x][y] = GAMECELL.MINE_HIDDEN;
					} else {
						_game[x][y] = GAMECELL.HIDDEN;
					}
				}
			}
		}

		@Override
		public boolean handleEnter(Object arg, boolean isReturn) {
			if(_focusView == null) {
				buildGameViews();
			}
			setUpGame();
			startGame();
			boolean result = super.handleEnter(arg, isReturn);
			if(getFocus() == null) {
				setFocus(_views[0][0]);
			}
			updateViews();
			updateFocusHighlight();
			return result;
		}

		private void startGame() {
			// TODO Auto-generated method stub
			
		}

		private void setUpGame() {
			updateViews();
		}
		
		private void updateViews() {
			boolean won = true;
			for(int y = 0 ; y < _gameheight; ++y) {
				for(int x = 0; x < _gamewidth ; ++x) {
					if(_views[x][y].getResource() != null) {
						_views[x][y].getResource().remove();
					}
					
//					if(_views[x][y].getResource() != null) _views[x][y].getResource().remove();
					switch(_game[x][y]) {
					case FLAGGED:
						won = false;
						_views[x][y].setResource(FLAG);
						_views[x][y].setFocusable(true);
						break;
					case MINE_FLAGGED:
						_views[x][y].setResource(FLAG);
						_views[x][y].setFocusable(true);
						break;
					case HIDDEN:
						won = false;
						_views[x][y].setResource(HIDDEN);
						_views[x][y].setFocusable(true);
						break;
					case MINE_HIDDEN:
						// if all mines are hidden, but not flagged, it's still a win.
						_views[x][y].setResource(HIDDEN);
						_views[x][y].setFocusable(true);
						break;
					case MINE_SHOWN:
						won = false;
						_views[x][y].setResource(MINE);
						_views[x][y].setFocusable(false);
						break;
					case SHOWN:
						String numberString = getNumberString(x, y);
						if(numberString != null && numberString.length() > 0) {
							_views[x][y].setResource(createText("system-20.font", Color.white, numberString));
							_views[x][y].setFocusable(true);
						} else {
							_views[x][y].clearResource();
							_views[x][y].setFocusable(false);
						}
						break;
					}
				}
			}

			if(won) {
				_gameover = true;
				showGameOver(true);
			} else if(_gameover) {
				showGameOver(false);
			}
			
		}

		private void selectArea(BView focus) {
//			_views[x][y] = new BButton(parent, x*cellWidth, y*cellHeight, cellWidth-2, cellHeight-2);
			MineSweeper app = (MineSweeper) getBApp();
			Rectangle gameArea = app.getScrollRegion();
			int cellWidth = gameArea.width / _gamewidth;
			int cellHeight = gameArea.height / _gameheight;
			int x = (focus.getX()-gameArea.x)/cellWidth;
			int y = (focus.getY()-gameArea.y)/cellHeight;
			
			selectArea(x, y);
			
			updateViews();
		}
		
		private void selectArea(int x, int y) {
			for(int i = x-1; i <= x+1; ++i) {
				for(int j = y-1; j <= y+1; ++j) {
					selected(i,j, false);
				}
			}
		}
		
		private void clearNearby(int x, int y) {
			if(!isHidden(x, y)) {
				return;
			}
			String numberString = getNumberString(x, y);
			if(numberString == null || numberString.length() == 0) {
				_game[x][y] = GAMECELL.SHOWN;
			} else {
				if (_game[x][y] == GAMECELL.HIDDEN) {
					_game[x][y] = GAMECELL.SHOWN;
				}
				return;
			}
			if(x > 0) {
				clearNearby(x-1, y);
				if(y > 0) {
					clearNearby(x-1, y-1);
				}
				if(y < _gameheight-1) {
					clearNearby(x-1, y+1);
				}
			}
			if(x < _gamewidth-1) {
				clearNearby(x+1, y);
				if(y > 0) {
					clearNearby(x+1, y-1);
				}
				if(y < _gameheight-1) {
					clearNearby(x+1, y+1);
				}
			}
			if(y > 0) {
				clearNearby(x, y-1);
				if(x > 0) {
					clearNearby(x-1, y-1);
				}
				if(x < _gameheight-1) {
					clearNearby(x+1, y-1);
				}
			}
			if(y < _gameheight-1) {
				clearNearby(x, y+1);
				if(x > 0) {
					clearNearby(x-1, y+1);
				}
				if(x < _gameheight-1) {
					clearNearby(x+1, y+1);
				}
			}
			
		}

		private boolean isHidden(int x, int y) {
			return (_game[x][y] == GAMECELL.HIDDEN || _game[x][y] == GAMECELL.MINE_HIDDEN);
		}
//		private boolean isFlagged(int x, int y) {
//			return (_game[x][y] == GAMECELL.FLAGGED|| _game[x][y] == GAMECELL.MINE_FLAGGED);
//		}
//		private boolean isMine(int x, int y) {
//			return (_game[x][y] == GAMECELL.MINE_FLAGGED|| _game[x][y] == GAMECELL.MINE_HIDDEN|| _game[x][y] == GAMECELL.MINE_SHOWN);
//		}

		private void showGameOver(boolean win) {
			if(getBelow().getResource() != null) {
				getBelow().getResource().remove();
			}
			getBelow().clearResource();
			flush();
			TiVoModelInfo info = TiVoModelInfo.getInfo(((MineSweeper)getBApp()).tivoID);
			if(win) {
				getBApp().play(SOUND_ALERT);
				if(info == null || ! info.isVideoBackgrounds()) {
					getBelow().setResource(getResource(Color.green.darker().darker()));
				} else {
					getBelow().setResource(getResource(V49BApplication.STREAM_LOOP_GREEN_NOWPLAYING));
				}
			} else {
				getBApp().play(SOUND_BONK);
				if(info == null || ! info.isVideoBackgrounds()) {
					getBelow().setResource(getResource(Color.red.darker().darker()));
				} else {
					getBelow().setResource(getResource(V49BApplication.STREAM_LOOP_RED_CENTRAL));
				}
			}
		}
		
		private void clearGameOver() {
			_gameover = false;
			if(getBelow().getResource() != null) {
				getBelow().getResource().remove();
			}
			getBelow().clearResource();
			flush();
			getBelow().setResource(getResource(V49BApplication.STREAM_LOOP_BLUE_SETUP));
		}

		private String getNumberString(int x, int y) {
			int count = 0;
			for(int i = -1 ; i <= 1 ; ++i) {
				for(int j = -1 ; j <= 1 ; ++j) {
					int xi = x+i;
					int yj = y+j;
					if (xi >= 0 && xi < _gamewidth && yj >= 0 && yj < _gameheight) {
						if (_game[xi][yj] == GAMECELL.MINE_HIDDEN
								|| _game[xi][yj] == GAMECELL.MINE_SHOWN 
								|| _game[xi][yj] == GAMECELL.MINE_FLAGGED) {
							count += 1;
						}
					}
				}
			}
			if(count > 0) {
				return String.valueOf(count);
			} else {
				return "";
			}
		}

	}

	@Override
	protected void showFirstPage() {
		push(new MineSweeperScreen(this), TRANSITION_FORWARD);
	}
	
	@Override
	public void playSoundForKey(int code, boolean wasHandled, boolean inScreen) {
		// disable default (dpad, chan, thumb, select) bonks
		if(!wasHandled)
			return;		
//		// disable updn bonks
//		if(!wasHandled && (code == KEY_UP || code == KEY_DOWN || code == KEY_LEFT || code == KEY_RIGHT)) {
//			return;
//		}
		
		// disable all the annoying updn's
		if(wasHandled && inScreen && (code == KEY_UP || code == KEY_DOWN || code == KEY_LEFT || code == KEY_RIGHT)) {
			return;
		}
		
		super.playSoundForKey(code, wasHandled, inScreen);
	}

}
