package util;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;

import com.blackledge.david.html.ProxyCoder;

import fi.iki.elonen.NanoHTTPD;
import fi.iki.elonen.NanoHTTPD.Response.IStatus;
import fi.iki.elonen.NanoHTTPD.Response.Status;
import fi.iki.elonen.ServerRunner;

public class TiVoMediaProxy extends NanoHTTPD {

    public static void main(String[] args) {
        ServerRunner.run(TiVoMediaProxy.class);
    }
    
	public TiVoMediaProxy() {
		this(81);
	}
	
	public TiVoMediaProxy(int port) {
		super(port);
System.out.println(port);
	}

	public TiVoMediaProxy(String hostname, int port) {
		super(hostname, port);
System.out.println(hostname+":"+port);
	}

    @Override public Response serve(IHTTPSession session) {
        Method methodObject = session.getMethod();
        String uri = session.getUri();
        String query = session.getQueryParameterString();
        ProxyCoder decoder = new ProxyCoder();
        
        ArrayList<String> args = new ArrayList<String>();
		String destination = decoder.getOriginalResourceAndArgs(query, args);

System.out.print(" from:"+query);
System.out.print(" decoded="+destination);
		String referer = null;
		String cookie = null;
		String method = null;
		try {
			for (String arg : args) {
				if (arg.startsWith("r=")) {
					referer = URLDecoder.decode(arg.substring(2), "UTF-8");
System.out.print(" referer="+referer);
				}
				if (arg.startsWith("c=")) {
					cookie = URLDecoder.decode(arg.substring(2), "UTF-8");
System.out.print(" cookie="+referer);
				}
				if (arg.startsWith("m=")) {
					method = URLDecoder.decode(arg.substring(2), "UTF-8");
System.out.print(" method="+referer);
				}
			}
		} catch (UnsupportedEncodingException e) {
			// give up, we've lost our mind.
			e.printStackTrace();
		}
System.out.println();

		if(destination == null || destination.trim().length() == 0) {
//        Map<String, String> parms = session.getParms();
//        if(parms.size() == 0 || parms.get("u") == null) {
        	return new NanoHTTPD.Response("no url provided.");
        }
//System.out.println(method + " '" + uri + "' "+parms);
        
        URL remoteURL;
		try {
//			remoteURL = new URL(parms.get("u"));
			remoteURL = new URL(destination);
		} catch (MalformedURLException e2) {
			// FIXME no url to process.  return error result
			e2.printStackTrace();
			remoteURL = null;
		}
        
		IStatus status = Status.OK;
		BufferedImage bi = null;
		try {
			URLConnection connection = remoteURL.openConnection();
			final HttpURLConnection hcon = (HttpURLConnection) connection;
			
			//pass headers through to the hcon.
			Map<String, String> headers = session.getHeaders();
			for (String key : headers.keySet()) {
				hcon.setRequestProperty(key, headers.get(key));
			}
			// override some headers based on proxy args
			if (referer != null) {
				hcon.setRequestProperty("Referer", referer);
			}
			if (cookie != null) {
				hcon.setRequestProperty("Cookie", cookie);
			}
			
			if (method != null) {
				hcon.setRequestMethod(method);
			} else {
				hcon.setRequestMethod(session.getMethod().toString());
			}
			
//			connection.setRequestProperty("Accept-Charset", charset);
			long originalLength = connection.getContentLengthLong();
			String originalType = connection.getContentType();
System.out.println("original length:"+originalLength+" type:"+originalType);

			InputStream contentStream = connection.getInputStream();
			
			if (
//					// just pass gif through in case it's animated.
//					originalType.toLowerCase().indexOf("image/gif") > -1
//					|| 
					// not an image, just pass it through
					originalType.toLowerCase().indexOf("image/") < 0) {
				IStatus status2 = getStatus(hcon);
System.out.println("pass through "+originalType+" - "+status2.getDescription());
				return getFinalResponse(originalType, contentStream, status2, connection);
			}
			
			try {
				//"image exceeded max # of pixels 2073600 (1920x1080)"
				int maxwidth = 1920;//"max image width exceeded (nnnn > 1920)"
				int maxheight = 1080;//"max image height exceeded (nnnn > 1080)"
				int maxsize = 2097152;//"max data bytes exceeded (nnnnnnn > 2097152)"
				bi = readAndTransformImage(contentStream, maxwidth, maxheight);
				
		    	ByteArrayOutputStream out = new ByteArrayOutputStream();
	    		String type = writeOutImage(bi, out);
	    		
	    		out.close();
	    		byte[] bytes = out.toByteArray();
System.out.println("length:" + bytes.length + " type:" + type);
	    		InputStream msg = new ByteArrayInputStream(bytes);
	    		
	    		return getFinalResponse(type, msg, status, connection);
			} catch (IOException e) {
				// FIXME couldn't process image, just pass the input stream on!
				// but probably can't do that since it was partly read already.
//				e.printStackTrace();
				System.out.println("FAIL "+ e);
	    		return new NanoHTTPD.Response("FAIL"); // FIXME status code
			}
		} catch (IOException e1) {
			// FIXME connection failed, return error result somehow
//			e1.printStackTrace();
			System.out.println("FAIL "+ e1);
    		return new NanoHTTPD.Response("FAIL"); // FIXME status code
		}
		
        
////    	String pathToWeb = getServletContext().getRealPath(File.separator);
////    	File f = new File(pathToWeb + "avajavalogo.jpg");
////    	BufferedImage bi = ImageIO.read(f);
    }

	private Response getFinalResponse(String originalType, InputStream contentStream,
			IStatus status, URLConnection connection) {
		NanoHTTPD.Response response = new NanoHTTPD.Response(status, originalType, contentStream);

		// pass connection information along in response
		int i = 0;
		while (connection.getHeaderFieldKey(i) != null) {
			response.addHeader(connection.getHeaderFieldKey(i),
					connection.getHeaderField(i));
			++i;
		}
		if (connection instanceof HttpURLConnection) {
			String method = ((HttpURLConnection) connection).getRequestMethod();
			response.setRequestMethod(Method.valueOf(method.toUpperCase()));
		}
		return response;
	}

	private IStatus getStatus(HttpURLConnection hcon) {
		String msg1;
		try {
			msg1 = hcon.getResponseMessage();
		} catch (IOException e) {
System.out.println("use OK desc");
			msg1 = "OK"; // getDescription includes the status code :/
		}
		int code1;
		try {
			code1 = hcon.getResponseCode();
		} catch (IOException e) {
System.out.println("use OK status");
			code1 = Status.OK.getRequestStatus();
		}
		final String msg = msg1;
		final int code = code1;
		return new IStatus() {public String getDescription() {return "" + code + " " + msg;}public int getRequestStatus() { return code;}};
	}

	private BufferedImage readAndTransformImage(InputStream contentStream, int maxwidth, int maxheight)
			throws IOException {
		BufferedImage originalImage = ImageIO.read(contentStream);
		int width = originalImage.getWidth();
		int height = originalImage.getHeight();
		
		// scale to proper size
		if(width > maxwidth) {
			double percent = (double)maxwidth/(double)width;
			width *= percent;
			height *= percent;
		}
		if(height > maxheight) {
			double percent = (double)maxheight/(double)height;
			width *= percent;
			height *= percent;
		}

		//FIXME other considerations
		// don't process.
		if (height == originalImage.getHeight() && width == originalImage.getWidth())
		{
			return originalImage;
		}
System.out.println("resize "+originalImage.getWidth()+"x"+originalImage.getHeight()+" -> "+width+"x"+height);
		
		BufferedImage bi = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D graphics = bi.createGraphics();
		graphics.drawImage(
			originalImage.getScaledInstance(width, height, 
					Image.SCALE_FAST)
				, 0, 0, null);
//		graphics.setColor(Color.white);
//		graphics.drawArc(0, 0, 20, 20, 0, 320);
//		graphics.setColor(Color.black);
//		graphics.drawArc(1, 1, 20, 20, 0, 320);
		return bi;
	 }

	private String writeOutImage(BufferedImage bi, ByteArrayOutputStream out)
			throws IOException {
//		ImageIO.write(bi, "jpg", out);
		
		String name = "jpeg";
		String type = "image/jpeg";
//		String name = "gif";
//		String type = "image/gif";
//		String name = "png";
//		String type = "image/png";
		
//		boolean compression = true;
		boolean compression = false;
		
		ImageOutputStream  ios =  ImageIO.createImageOutputStream(out);
		Iterator<ImageWriter> iter = ImageIO.getImageWritersByFormatName(name);
		ImageWriter writer = iter.next();
		ImageWriteParam iwp = writer.getDefaultWriteParam();
		
//		iwp.setDestinationType(ImageTypeSpecifier.createFromBufferedImageType(BufferedImage.TYPE_BYTE_GRAY));

		if(compression) {
			iwp.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
//			iwp.setCompressionType("lzw");
			iwp.setCompressionQuality(0.85f);
		}
		writer.setOutput(ios);
		writer.write(null, new IIOImage(bi,null,null),iwp);
		writer.dispose();
		
		return type;
	}


	
	
	
	
	
	
	
	
	
	
	
//	import javax.imageio.ImageIO;
//	import javax.servlet.http.HttpServlet;
//	import javax.servlet.http.HttpServletRequest;
//	import javax.servlet.http.HttpServletResponse;
//
//	response.setContentType("image/jpeg");
//
//	String pathToWeb = getServletContext().getRealPath(File.separator);
//	File f = new File(pathToWeb + "avajavalogo.jpg");
//	BufferedImage bi = ImageIO.read(f);
//	OutputStream out = response.getOutputStream();
//	ImageIO.write(bi, "jpg", out);
//	out.close();

	
//	public static class TiVoMediaProxyServlet extends HttpServlet {
//
//		  protected void doGet( HttpServletRequest request,
//		                        HttpServletResponse response)
//		        throws ServletException, IOException {
//
//		      response.getWriter().write("<html><body>GET response</body></html>");
//		  }
//		}

// basic proxy based in part on http://www.java2s.com/Code/Java/Network-Protocol/Asimpleproxyserver.htm
	
//	  public static void main(String[] args) throws IOException {
//		    try {
//		      String host = "your Proxy Server";
//		      int remoteport = 100;
//		      int localport = 111;
//		      // Print a start-up message
//		      System.out.println("Starting proxy for " + host + ":" + remoteport
//		          + " on port " + localport);
//		      // And start running the server
//		      runServer(host, remoteport, localport); // never returns
//		    } catch (Exception e) {
//		      System.err.println(e);
//		    }
//		  }
//
//		  /**
//		   * runs a single-threaded proxy server on
//		   * the specified local port. It never returns.
//		   */
//		  public static void runServer(String host, int remoteport, int localport)
//		      throws IOException {
//		    // Create a ServerSocket to listen for connections with
//		    ServerSocket ss = new ServerSocket(localport);
//
//		    final byte[] request = new byte[1024];
//		    byte[] reply = new byte[4096];
//
//		    while (true) {
//		      Socket client = null, server = null;
//		      try {
//		        // Wait for a connection on the local port
//		        client = ss.accept();
//
//		        final InputStream streamFromClient = client.getInputStream();
//		        final OutputStream streamToClient = client.getOutputStream();
//
//		        // Make a connection to the real server.
//		        // If we cannot connect to the server, send an error to the
//		        // client, disconnect, and continue waiting for connections.
//		        try {
//		          server = new Socket(host, remoteport);
//		        } catch (IOException e) {
//		          PrintWriter out = new PrintWriter(streamToClient);
//		          out.print("Proxy server cannot connect to " + host + ":"
//		              + remoteport + ":\n" + e + "\n");
//		          out.flush();
//		          client.close();
//		          continue;
//		        }
//
//		        // Get server streams.
//		        final InputStream streamFromServer = server.getInputStream();
//		        final OutputStream streamToServer = server.getOutputStream();
//
//		        // a thread to read the client's requests and pass them
//		        // to the server. A separate thread for asynchronous.
//		        Thread t = new Thread() {
//		          public void run() {
//		            int bytesRead;
//		            try {
//		              while ((bytesRead = streamFromClient.read(request)) != -1) {
//		                streamToServer.write(request, 0, bytesRead);
//		                streamToServer.flush();
//		              }
//		            } catch (IOException e) {
//		            }
//
//		            // the client closed the connection to us, so close our
//		            // connection to the server.
//		            try {
//		              streamToServer.close();
//		            } catch (IOException e) {
//		            }
//		          }
//		        };
//
//		        // Start the client-to-server request thread running
//		        t.start();
//
//		        // Read the server's responses
//		        // and pass them back to the client.
//		        int bytesRead;
//		        try {
//		          while ((bytesRead = streamFromServer.read(reply)) != -1) {
//		            streamToClient.write(reply, 0, bytesRead);
//		            streamToClient.flush();
//		          }
//		        } catch (IOException e) {
//		        }
//
//		        // The server closed its connection to us, so we close our
//		        // connection to our client.
//		        streamToClient.close();
//		      } catch (IOException e) {
//		        System.err.println(e);
//		      } finally {
//		        try {
//		          if (server != null)
//		            server.close();
//		          if (client != null)
//		            client.close();
//		        } catch (IOException e) {
//		        }
//		      }
//		    }
//		  }

}
